@@ -41,6 +41,12 @@
 #define TALLBULLET_FLAG       0x00100000
 #define SHORT_TOC_FLAG        0x00200000
 #define XREF_FLAG             0x00400000
+#define HEADER_FLAG           0x00800000
+#define START_TITLE_FLAG      0x01000000
+#define END_TITLE_FLAG        0x02000000
+
+
+#define TOC_SIZE     8192
 
 
 typedef struct LINE
@@ -78,6 +84,7 @@ int strip_indent = -1;
 int last_toc_line = INT_MIN;
 int last_toc_line2 = INT_MIN;
 int multiwordheaders = 0;
+int warn_on_long_lines = 0;
 
 char locale[256] = "";
 int dos_codepage = 1;
@@ -119,7 +126,7 @@ int myisalnum(int c)
    return (((c >= '0') && (c <= '9')) ||
 	   ((c >= 'A') && (c <= 'Z')) ||
 	   ((c >= 'a') && (c <= 'z')) ||
-	   (c >= 0x80));
+	   (c >= 0x80) || (c < 0));
 }
 
 
@@ -156,6 +163,8 @@ void set_locale(char *l)
 
    if (mystricmp(l, "china") == 0)
       dos_codepage = 0;
+   else if (mystricmp(l, "korea") == 0)
+      dos_codepage = 0;
 }
 
 
@@ -210,7 +219,7 @@ int is_empty(char *s)
 
 
 
-void free_data()
+void free_data(void)
 {
    LINE *line = head;
    LINE *prev;
@@ -281,7 +290,7 @@ void add_toc_line(char *buf, char *alt, int root, int num, int texinfoable, int
 
    toc->next = NULL;
    toc->root = root;
-   toc->texinfoable = ((texinfoable) && (num > last_toc_line2+1));
+   toc->texinfoable = ((texinfoable) && (num > last_toc_line+1));
    toc->htmlable = htmlable;
    toc->otherfile = otherfile;
 
@@ -406,7 +415,7 @@ int strincmp(char *s1, char *s2)
 
 
 
-int read_file(char *filename)
+int read_file(char *filename, char *htmlname)
 {
    char buf[1024];
    char *p;
@@ -461,6 +470,10 @@ int read_file(char *filename)
 	    flags |= (HEADING_FLAG | NOCONTENT_FLAG);
 	 else if (mystricmp(buf+1, "heading") == 0)
 	    flags |= HEADING_FLAG;
+	 else if (strincmp(buf+1, "titlepage") == 0)
+	    add_line("", START_TITLE_FLAG);
+	 else if (strincmp(buf+1, "!titlepage") == 0)
+	    add_line("", END_TITLE_FLAG);
 	 else if (mystricmp(buf+1, "contents") == 0)
 	    add_line("", TOC_FLAG);
 	 else if (mystricmp(buf+1, "shortcontents") == 0)
@@ -482,6 +495,8 @@ int read_file(char *filename)
 	    if (last_toc_line == line-1)
 	       last_toc_line = line;
 	 }
+         else if (strincmp(buf+1, "header ") == 0)
+            add_line(buf+8, HEADER_FLAG);
 	 else if (strincmp(buf+1, "startoutput ") == 0)
 	    add_line(buf+13, STARTOUTPUT_FLAG);
 	 else if (strincmp(buf+1, "endoutput ") == 0)
@@ -492,6 +507,10 @@ int read_file(char *filename)
 	    strcpy(fileheader, buf+3);
 	 else if ((mytolower(buf[1]=='f')) && (buf[2]=='='))
 	    strcpy(filefooter, buf+3);
+	 else if ((mytolower(buf[1]=='f')) && (buf[2]=='1') && (buf[3]=='='))
+	    sprintf(filefooter, "%s%s", buf+4, get_filename(htmlname));
+	 else if ((mytolower(buf[1]=='f')) && (buf[2]=='2') && (buf[3]=='='))
+	    strcat(filefooter, buf+4);
 	 else if (strincmp(buf+1, "rtfh=") == 0)
 	    strcpy(rtfheader, buf+6);
 	 else if (strincmp(buf+1, "manh=") == 0)
@@ -534,6 +553,13 @@ int read_file(char *filename)
 
 	 add_line(buf, flags);
 
+	 if (warn_on_long_lines) {
+	    int len = strlen (buf);
+	    if (len > 77)
+	       printf("Warning: line %d exceded in %d caracters '...%s'\n",
+		      line, len - 77, buf + 77);
+	 }
+
 	 flags &= ~(HEADING_FLAG | NOCONTENT_FLAG | NONODE_FLAG);
       }
    }
@@ -722,11 +748,21 @@ int toc_size(int part)
 
 
 
+int toc_scmp(const void *e1, const void *e2)
+{
+   TOC *t1 = *((TOC **)e1);
+   TOC *t2 = *((TOC **)e2);
+
+   return mystricmp(t1->text, t2->text);
+}
+
+
+
 void output_toc(FILE *f, char *filename, int root, int body, int part)
 {
    char name[256];
    char *s;
-   TOC *toc = tochead;
+   TOC *toc;
    int nested = 0;
    int section_number = 0;
 
@@ -747,10 +783,7 @@ void output_toc(FILE *f, char *filename, int root, int body, int part)
 	       hfprintf(f, "<li><a href=\"%s\">%s</a>\n", name, ALT_TEXT(toc));
 	    }
 	    else if (body) {
-	       if (toc_size(section_number) == 0)
-		  hfprintf(f, "<li><a href=\"#%s\">%s</a>\n", toc->text, ALT_TEXT(toc));
-	       else
-		  hfprintf(f, "<li><a href=\"#%s_toc\">%s</a>\n", toc->text, ALT_TEXT(toc));
+	       hfprintf(f, "<li><a href=\"#%s\">%s</a>\n", toc->text, ALT_TEXT(toc));
 	       section_number++;
 	    }
 	    else {
@@ -775,6 +808,8 @@ void output_toc(FILE *f, char *filename, int root, int body, int part)
 	 toc = toc->next;
 
       if (part <= 0) {
+	 TOC *ptr[TOC_SIZE];
+	 int j, i = 0;
 	 if (root)
 	    fprintf(f, "<ul><h2>\n");
 	 else
@@ -790,18 +825,22 @@ void output_toc(FILE *f, char *filename, int root, int body, int part)
 	       }
 	       else {
 		  if (nested) {
+		     if (i > 1)
+			qsort(ptr, i, sizeof(TOC *), toc_scmp);
+		     for (j = 0; j < i; j++)
+			hfprintf(f, "<li><a href=\"#%s\">%s</a>\n", ptr[j]->text, ALT_TEXT(ptr[j]));
+		  
 		     fprintf(f, "</h4></ul><p>\n");
-		     nested = 0;
+		     nested = i = 0;
 		  }
 	       }
 
-	       if (!nested)
-		  hfprintf(f, "<a name=\"%s_toc\">", toc->text);
-
-	       hfprintf(f, "<li><a href=\"#%s\">%s</a>\n", toc->text, ALT_TEXT(toc));
-
-	       if (!nested)
-		  fprintf(f, "</a>");
+	       if (nested) {
+		  if(i < TOC_SIZE)
+		     ptr[i++] = toc;
+	       }
+	       else
+		  hfprintf(f, "<li><a href=\"#%s\">%s</a>\n", toc->text, ALT_TEXT(toc));
 	    }
 
 	    toc = toc->next;
@@ -816,6 +855,8 @@ void output_toc(FILE *f, char *filename, int root, int body, int part)
 	    fprintf(f, "</h4></ul>\n");
       }
       else {
+	 TOC *ptr[TOC_SIZE];
+	 int j, i = 0;
 	 section_number = 0;
 	 fprintf(f, "<p>\n<ul><h4>\n");
 
@@ -825,12 +866,18 @@ void output_toc(FILE *f, char *filename, int root, int body, int part)
 	    toc = toc->next;
 	 }
 
-	 while ((toc) && (!toc->root)) {
+	 while ((toc) && (!toc->root) && (i < TOC_SIZE)) {
 	    if (toc->htmlable)
-	       hfprintf(f, "<li><a href=\"#%s\">%s</a>\n", toc->text, ALT_TEXT(toc));
+	       ptr[i++] = toc;
 	    toc = toc->next;
 	 }
 
+	 if (i > 1)
+	    qsort(ptr, i, sizeof(TOC *), toc_scmp);
+
+	 for (j = 0; j < i; j++)
+	    hfprintf(f, "<li><a href=\"#%s\">%s</a>\n", ptr[j]->text, ALT_TEXT(ptr[j]));
+
 	 fprintf(f, "</h4></ul>\n<p><br><br>\n");
       }
    }
@@ -867,10 +914,8 @@ int scmp(const void *e1, const void *e2)
 
 void output_texinfo_toc(FILE *f, int root, int body, int part)
 {
-   #define TOC_SIZE     8192
-
-   TOC *toc = tochead;
-   int section_number = 0;
+   TOC *toc;
+   int section_number;
    char **ptr;
    char *s;
    int i, j;
@@ -974,7 +1019,6 @@ char *node_name(int i)
 {
    TOC *toc = tochead;
 
-   toc = tochead;
    if (toc)
       toc = toc->next;
 
@@ -1300,7 +1344,7 @@ void write_textinfo_xref(FILE *f, char *xref)
 int write_texinfo(char *filename)
 {
    char buf[256];
-   LINE *line = head;
+   LINE *line = head, *title_line = 0;
    char *p, *str, *next, *prev;
    char *xref[256];
    int xrefs = 0;
@@ -1308,6 +1352,7 @@ int write_texinfo(char *filename)
    int section_number = 0;
    int toc_waiting = 0;
    int continue_def = 0;
+   int title_pass = 0;
    int i;
    FILE *f;
 
@@ -1351,6 +1396,7 @@ int write_texinfo(char *filename)
 		     xrefs = 0;
 		  }
 		  tfprintf(f, "@node %s, %s, %s, %s\n", buf, next, prev, node_name(section_number-1));
+		  tfprintf(f, "@section %s\n", buf);
 	       }
 
 	       fputs("@ftable @asis\n", f);
@@ -1425,7 +1471,7 @@ int write_texinfo(char *filename)
 	 }
 	 else {
 	    if ((line->flags & DEFINITION_FLAG) && (!continue_def))
-	       fputs("@item ", f);
+	       fputs("@item @t{", f);
 
 	    while (*p) {
 	       /* less-than HTML tokens */
@@ -1461,8 +1507,11 @@ int write_texinfo(char *filename)
 
 	    if (line->flags & CONTINUE_FLAG)
 	       fputs(" ", f);
-	    else
+	    else {
+	       if (line->flags & DEFINITION_FLAG)
+		  fputs("}", f);
 	       fputs("\n", f);
+	    }
 	 }
       }
       else if (line->flags & NODE_FLAG) {
@@ -1482,6 +1531,7 @@ int write_texinfo(char *filename)
 	       xrefs = 0;
 	    }
 	    tfprintf(f, "@node %s, %s, %s, %s\n", line->text, next, prev, node_name(section_number-1));
+	    tfprintf(f, "@section %s\n", line->text);
 	 }
 
 	 fputs("@ftable @asis\n", f);
@@ -1496,6 +1546,22 @@ int write_texinfo(char *filename)
       else if (line->flags & XREF_FLAG) {
 	 xref[xrefs++] = line->text;
       }
+      else if (line->flags & START_TITLE_FLAG) {
+	 /* remember where the title starts */
+	 title_line = line;
+	 if (!title_pass)
+	    fputs("@titlepage\n", f);
+      }
+      else if (line->flags & END_TITLE_FLAG) {
+	 if (!title_pass)
+	 {
+	    title_pass++;
+	    fputs("@end titlepage\n@ifinfo\n", f);
+	    line = title_line;
+	 }
+	 else
+	    fputs("@end ifinfo\n", f);
+      }
 
       continue_def = (line->flags & CONTINUE_FLAG);
       line = line->next;
@@ -1511,10 +1577,9 @@ int write_rtf(char *filename)
 {
    LINE *line = head;
    LINE *l;
-   char *p;
+   char *p, *last = 0;
    FILE *f;
    int preformat = 0;
-   int title = 0;
    int prevhead = 0;
    int prevdef = 0;
    int multidef = 0;
@@ -1587,6 +1652,11 @@ int write_rtf(char *filename)
       fputs("{\\f1\\fmodern\\fcharset134\\fprq1 ;}\n", f);
       fputs("{\\f2\\froman\\fcharset134\\fprq2 ;}}\n", f);
    }
+   else if (mystricmp(locale, "korea") == 0) {
+      fputs("{\\fonttbl{\\f0\\froman\\fcharset129\\fprq2 ;}\n", f);
+      fputs("{\\f1\\fmodern\\fcharset129\\fprq1 ;}\n", f);
+      fputs("{\\f2\\froman\\fcharset129\\fprq2 ;}}\n", f);
+   }
    else {
       fputs("{\\fonttbl{\\f0\\froman\\fcharset0\\fprq2 Times New Roman;}\n", f);
       fputs("{\\f1\\fmodern\\fcharset0\\fprq1 Courier New;}\n", f);
@@ -1682,6 +1752,7 @@ int write_rtf(char *filename)
 	 }
 
 	 while (*p) {
+	    last = 0;
 	    if (strincmp(p, "<p>") == 0) {
 	       /* paragraph breaks */
 	       PAR();
@@ -1724,13 +1795,11 @@ int write_rtf(char *filename)
 	    else if (strincmp(p, "<title>") == 0) {
 	       /* start document title */
 	       fputs("{\\info{\\title ", f);
-	       title = 1;
 	       p += 7;
 	    }
 	    else if (strincmp(p, "</title>") == 0) {
 	       /* end document title */
 	       fputs("}{\\author Allegro makedoc utility}}\n", f);
-	       title = 0;
 	       p += 8;
 	    }
 	    else if (strincmp(p, "<hr>") == 0) {
@@ -1880,7 +1949,7 @@ int write_rtf(char *filename)
 
 	       /* normal character */
 	       rfputc((unsigned char)*p, f);
-	       p++;
+	       last = p++;
 	    }
 	 }
 
@@ -1918,8 +1987,10 @@ int write_rtf(char *filename)
 	       }
 	    }
 	    else {
-	       /* normal EOL */
-	       fputs("\n", f);
+	       if (last && *last != 32)
+		  fputs(" ", f); /* add artificial space */
+
+	       fputs("\n", f); /* normal EOL */
 	    }
 	 }
       }
@@ -1935,7 +2006,7 @@ int write_rtf(char *filename)
       else if (line->flags & TOC_FLAG) {
 	 /* table of contents */
 	 PAR();
-	 fputs("\n{\\field{\\*\\fldinst TOC \\\\o \"1-1\" }{\\fldrslt {\\b\\i\\ul\\fs24\\cf2 Update this field to generate the table of contents.}}}\n", f);
+	 fputs("\n{\\field{\\*\\fldinst TOC \\\\t \"Heading 1\" }{\\fldrslt {\\b\\i\\ul\\fs24\\cf2 Update this field to generate the table of contents.}}}\n", f);
       }
       else if (line->flags & INDEX_FLAG) {
 	 /* index */
@@ -2241,6 +2312,22 @@ int write_man(char *filename)
 	       if (mansynopsis[0])
 		  fprintf(f, ".B %s\n\n", mansynopsis);
 
+               /* go ahead and find all the header lines */
+               l2 = line;
+               while (l2 && l2->flags & CONTINUE_FLAG) l2=l2->next;
+
+               if (l2) {
+                  l2 = l2->next;
+                  while (l2 && (l2->flags & HEADER_FLAG)) {
+                    fprintf(f, ".B %s\n", l2->text);
+                    l2 = l2->next;
+                    if (l2 && (l2->flags & HEADER_FLAG)) {
+                       fprintf(f, ".br\n");
+                    }
+                  }
+               }
+               fprintf(f, ".sp\n");
+
 	       fputs(".B ", f);
 	       mfputs(line->text, f);
 
@@ -2264,7 +2351,6 @@ int write_man(char *filename)
 			if (!f2)
 			   return 1;
 
-			fprintf(f2, ".\\\" Generated by the Allegro makedoc utility\n");
 			fprintf(f2, ".so man%s/%s\n", extension(filename), get_filename(buf));
 			fclose(f2);
 		     }
@@ -2342,6 +2428,9 @@ int main(int argc, char *argv[])
       else if (mystricmp(argv[i], "-part") == 0) {
 	 partial = 1;
       }
+      else if (mystricmp(argv[i], "-warn") == 0) {
+	 warn_on_long_lines = 1;
+      }
       else if (argv[i][0] == '-') {
 	 fprintf(stderr, "Unknown option '%s'\n", argv[i]);
 	 return 1;
@@ -2352,7 +2441,7 @@ int main(int argc, char *argv[])
    }
 
    if (!filename[0]) {
-      printf("Usage: makedoc [outputs] [-part] filename._tx\n\n");
+      printf("Usage: makedoc [outputs] [-part] [-warn] filename._tx\n\n");
       printf("Output formats:\n");
       printf("\t-ascii filename.txt\n");
       printf("\t-htm[l] filename.htm[l]\n");
@@ -2376,7 +2465,7 @@ int main(int argc, char *argv[])
       strcpy(extension(rtfname), "rtf");
    }
 
-   if (read_file(filename) != 0) {
+   if (read_file(filename, htmlname) != 0) {
       fprintf(stderr, "Error reading input file\n");
       err = 1;
       goto getout;
