@@ -1,32 +1,18 @@
 /*
  *    Example program for the Allegro library, by Shawn Hargreaves.
  *
- *    A follow up of the exgui.c example showing how to customise the
- *    default Allegro framework. In this case a dialog procedure
- *    animates a graphical clock without disrupting other GUI
- *    dialogs.  A more simple option shows how to dynamically change
- *    the font used by all GUI elements.
+ *    This program demonstrates how to write your own GUI objects.
  */
 
 
 #include <time.h>
 
-#include <allegro.h>
-#include "example.h"
+#include "allegro.h"
 
 
 
-/* we need to load example.dat to access the big font */
-DATAFILE *datafile;
-
-
 /* for the d_edit_proc() object */
-#define LEN 32
-char the_string[(LEN + 1) * 6] = "Change Me!";
-
-
-/* since we change the font, we need to store a copy of the original one */
-FONT *original_font;
+char the_string[32] = "Change Me!";
 
 
 /* the current time, for the clock object */
@@ -34,35 +20,6 @@ struct tm the_time;
 
 
 
-/* A custom dialog procedure for the 'change font' button. This uses a
- * simple form of inheritance: it calls d_button_proc() to do most of
- * the work, so it behaves exactly like any other button, but when the
- * button is clicked and d_button_proc() returns D_CLOSE, it intercepts
- * the message and changes the font instead.
- */
-int change_font_proc(int msg, DIALOG *d, int c)
-{
-   int ret;
-
-   /* call the parent object */
-   ret = d_button_proc(msg, d, c);
-
-   /* trap the close return value and change the font */
-   if (ret == D_CLOSE) {
-      if (font == original_font)
-	 font = datafile[BIG_FONT].dat;
-      else
-	 font = original_font;
-
-      return D_REDRAW; 
-   }
-
-   /* otherwise just return */
-   return ret;
-}
-
-
-
 /* helper function to draw a hand on the clock */
 void draw_hand(BITMAP *bmp, int value, int range, int v2, int range2, fixed length, int color)
 {
@@ -141,18 +98,21 @@ int clock_proc(int msg, DIALOG *d, int c)
 	     (the_time.tm_hour != t->tm_hour)) {
 	    the_time = *t;
 
-	    /* Redraw ourselves if the time has changed. Note that the dialog
-	     * manager automatically turns off the mouse pointer whenever a
-	     * MSG_DRAW message is sent to an individual object or an entire
-	     * dialog, so we don't have to do it explicitly. Also note the use
-	     * of the object_message function rather than a simple recursive
-	     * call to clock_proc(). This vectors the call through the function
+	    /* Redraw ourselves if the time has changed. Note that we have
+	     * to turn the mouse off before doing this: the dialog manager
+	     * turns it off whenever it sends us a draw message, but we
+	     * are sending the message ourselves here so we are responsible 
+	     * for making sure the mouse is off first. Also note the use of
+	     * the object_message function rather than a simple recursive call
+	     * to clock_proc(). This vectors the call through the function
 	     * pointer in the dialog object, which allows other object
 	     * procedures to hook it, for example a different type of clock
 	     * could process the draw messages itself but pass idle messages
 	     * on to this procedure.
 	     */
+	    show_mouse(NULL);
 	    object_message(d, MSG_DRAW, 0);
+	    show_mouse(screen);
 	 }
 	 break;
 
@@ -185,37 +145,30 @@ int clock_proc(int msg, DIALOG *d, int c)
 
 DIALOG the_dialog[] =
 {
-   /* (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key) (flags)  (d1) (d2)  (dp)           (dp2) (dp3) */
-   { d_clear_proc,      0,    0,    0,    0,    255,  0,    0,    0,       0,   0,    NULL,          NULL, NULL  },
-   { d_edit_proc,       12,   82,   256,  48,   255,  0,    0,    0,       LEN, 0,    the_string,    NULL, NULL  },
-   { d_check_proc,      12,   12,   161,  49,   255,  0,    't',  0,       0,   0,    "&Toggle Me",  NULL, NULL  },
-   { clock_proc,        242,  12,   64,   64,   255,  0,    0,    0,       0,   0,    NULL,          NULL, NULL  },
-   { change_font_proc,  12,   142,  141,  49,   255,  0,    'f',  D_EXIT,  0,   0, "Change &Font",   NULL, NULL  },
-   { d_button_proc,     162,  142,  141,  49,   255,  0,    0,    D_EXIT,  0,   0,    "Exit",        NULL, NULL  },
-   { NULL,              0,    0,    0,    0,    0,    0,    0,    0,       0,   0,    NULL,          NULL, NULL  }
+   /* (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key) (flags)  (d1)                    (d2)  (dp)           (dp2) (dp3) */
+   { d_clear_proc,      0,    0,    0,    0,    255,  0,    0,    0,       0,                      0,    NULL,          NULL, NULL  },
+   { d_edit_proc,       32,   32,   256,  8,    255,  0,    0,    0,       sizeof(the_string)-1,   0,    the_string,    NULL, NULL  },
+   { d_check_proc,      32,   64,   89,   13,   255,  0,    't',  0,       0,                      0,    "&Toggle Me",  NULL, NULL  },
+   { clock_proc,        192,  64,   64,   64,   255,  0,    0,    0,       0,                      0,    NULL,          NULL, NULL  },
+   { d_button_proc,     120,  160,  81,   17,   255,  0,    0,    D_EXIT,  0,                      0,    "Exit",        NULL, NULL  },
+   { NULL,              0,    0,    0,    0,    0,    0,    0,    0,       0,                      0,    NULL,          NULL, NULL  }
 };
 
 
 
-int main(int argc, char *argv[])
+int main()
 {
    int item;
-   char buf[256];
 
-   if (allegro_init() != 0)
-      return 1;
+   allegro_init();
    install_keyboard(); 
    install_mouse();
    install_timer();
-
-   if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0) != 0) {
-      if (set_gfx_mode(GFX_SAFE, 320, 200, 0, 0) != 0) {
-	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-	 allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
-	 return 1;
-      }
+   if (set_gfx_mode(GFX_SAFE, 320, 200, 0, 0) != 0) {
+      set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
+      allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
+      return 1;
    }
-
    set_palette(desktop_palette);
 
    /* We set up colors to match screen color depth (in case it changed) */
@@ -223,24 +176,10 @@ int main(int argc, char *argv[])
       the_dialog[item].fg = makecol(0, 0, 0);
       the_dialog[item].bg = makecol(255, 255, 255);
    }
-   
-   /* load the datafile */
-   replace_filename(buf, argv[0], "example.dat", sizeof(buf));
-   datafile = load_datafile(buf);
-   if (!datafile) {
-      set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-      allegro_message("Error loading %s!\n", buf);
-      return 1;
-   }
-
-   /* store a copy of the default font */
-   original_font = font;
 
    do_dialog(the_dialog, -1);
-   
-   unload_datafile (datafile);
 
    return 0;
 }
 
-END_OF_MAIN()
+END_OF_MAIN();
