@@ -11,13 +11,12 @@
 
 
 
-int main(void)
+int main()
 {
    /* declare three 32 bit (16.16) fixed point variables */
    fixed x, y, z;
 
-   if (allegro_init() != 0)
-      return 1;
+   allegro_init();
 
    /* convert integers to fixed point like this */
    x = itofix(10);
