@@ -6,26 +6,23 @@
  */
 
 
-#include <allegro.h>
+#include "allegro.h"
 
 
 
-int main(void)
+int main()
 {
    /* you should always do this at the start of Allegro programs */
-   if (allegro_init() != 0)
-      return 1;
+   allegro_init();
 
    /* set up the keyboard handler */
    install_keyboard(); 
 
    /* set a graphics mode sized 320x200 */
-   if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0) != 0) {
-      if (set_gfx_mode(GFX_SAFE, 320, 200, 0, 0) != 0) {
-	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-	 allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
-	 return 1;
-      }
+   if (set_gfx_mode(GFX_SAFE, 320, 200, 0, 0) != 0) {
+      set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
+      allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
+      return 1;
    }
 
    /* set the color palette */
@@ -40,8 +37,11 @@ int main(void)
     */
    acquire_screen();
 
-   /* write some text to the screen with black letters and transparent background */
-   textout_centre_ex(screen, font, "Hello, world!", SCREEN_W/2, SCREEN_H/2, makecol(0,0,0), -1);
+   /* set transparent text */
+   text_mode(-1);
+
+   /* write some text to the screen with black letters */
+   textout_centre(screen, font, "Hello, world!", SCREEN_W/2, SCREEN_H/2, makecol(0,0,0));
 
    /* you must always release bitmaps before calling any input functions */
    release_screen();
@@ -52,4 +52,4 @@ int main(void)
    return 0;
 }
 
-END_OF_MAIN()
+END_OF_MAIN();
