@@ -12,7 +12,7 @@
 
 
 
-int main(void)
+int main()
 {
    BITMAP *bmp;            /* we create a pointer to a virtual screen */
    int x=160, y=100;       /* these will be used to show the target sight */
@@ -20,27 +20,23 @@ int main(void)
    AL_CONST char *msg;
    int c;
 
-   if (allegro_init() != 0)/* you NEED this man! ;-) */
-      return 1;
+   allegro_init();         /* you NEED this man! ;-) */
 
    install_keyboard();     /* ahh... read the docs. I will explain only
 			    * joystick specific routines
 			    */
 
-   if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0) != 0) {
-      if (set_gfx_mode(GFX_SAFE, 320, 200, 0, 0) != 0) {
-	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-	 allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
-	 return 1;
-      }
+   if (set_gfx_mode(GFX_SAFE, 320, 200, 0, 0) != 0) {
+      set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
+      allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
+      return 1;
    }
-
    set_palette(default_palette);
    clear_bitmap(screen);
-   textout_centre_ex(screen, font, "Please center the joystick", SCREEN_W/2,
-		     SCREEN_H/2 - 36, palette_color[255], 0);
-   textout_centre_ex(screen, font, "and press a key.", SCREEN_W/2,
-		     SCREEN_H/2 - 20, palette_color[255], 0);
+   textout_centre(screen, font, "Please center the joystick", SCREEN_W/2,
+		  SCREEN_H/2 - 36, palette_color[255]);
+   textout_centre(screen, font, "and press a key.", SCREEN_W/2,
+		  SCREEN_H/2 - 20, palette_color[255]);
 
    if ((readkey()&0xFF) == 27)
       return 0;
@@ -68,8 +64,8 @@ int main(void)
       msg = calibrate_joystick_name(0);
 
       clear_bitmap(screen);
-      textout_centre_ex(screen, font, msg, SCREEN_W/2, 64, palette_color[255], 0);
-      textout_centre_ex(screen, font, "and press a key.", SCREEN_W/2, 80, palette_color[255], 0);
+      textout_centre(screen, font, msg, SCREEN_W/2, 64, palette_color[255]);
+      textout_centre(screen, font, "and press a key.", SCREEN_W/2, 80, palette_color[255]);
 
       if ((readkey()&0xFF) == 27)
 	 return 0;
@@ -87,10 +83,10 @@ int main(void)
    */
    if (joy[0].stick[0].flags & JOYFLAG_ANALOGUE) {
       clear_bitmap(screen);
-      textout_centre_ex(screen, font, "Now press 'D' to use a digital",
-			SCREEN_W/2, 64, palette_color[255], 0);
-      textout_centre_ex(screen, font, "joystick or 'A' for analogue mode.",
-			SCREEN_W/2, 80, palette_color[255], 0);
+      textout_centre(screen, font, "Now press 'D' to use a digital",
+		     SCREEN_W/2, 64, palette_color[255]);
+      textout_centre(screen, font, "joystick or 'A' for analogue mode.",
+		     SCREEN_W/2, 80, palette_color[255]);
 
       for (;;) {
 	 c = readkey()&0xFF;
@@ -121,21 +117,21 @@ int main(void)
 
       clear_bitmap(bmp);
 
-      textout_centre_ex(bmp, font, joystick_driver->name, 160, 150, palette_color[255], 0);
+      textout_centre(bmp, font, joystick_driver->name, 160, 150, palette_color[255]);
 
       if (analogmode)
-	 textout_centre_ex(bmp, font, "Analog mode selected", 160, 160, palette_color[255], 0);
+	 textout_centre(bmp, font, "Analog mode selected", 160, 160, palette_color[255]);
       else
-	 textout_centre_ex(bmp, font, "Digital mode selected", 160, 160, palette_color[255], 0);
+	 textout_centre(bmp, font, "Digital mode selected", 160, 160, palette_color[255]);
 
-      textout_centre_ex(bmp, font, "Move the joystick all around", 160, 170, palette_color[255], 0);
-      textout_centre_ex(bmp, font, "Press any key to exit", 160, 180, palette_color[255], 0);
-      textout_centre_ex(bmp, font, "Made by Grzegorz Adam Hankiewicz", 160, 190, palette_color[255], 0);
+      textout_centre(bmp, font, "Move the joystick all around", 160, 170, palette_color[255]);
+      textout_centre(bmp, font, "Press any key to exit", 160, 180, palette_color[255]);
+      textout_centre(bmp, font, "Made by Grzegorz Adam Hankiewicz", 160, 190, palette_color[255]);
 
       /* if we detect any buttons, we print a message on the screen */
       for (c=0; c<joy[0].num_buttons; c++) {
 	 if (joy[0].button[c].b)
-	    textprintf_centre_ex(bmp, font, 160, c*10, palette_color[15], 0, "%s pressed", joy[0].button[c].name);
+	    textprintf_centre(bmp, font, 160, c*10, palette_color[15], "%s pressed", joy[0].button[c].name);
       }
 
       if (!analogmode) {
@@ -145,22 +141,22 @@ int main(void)
 	 if (joy[0].stick[0].axis[0].d1) {
 	    if (x > 0)
 	       x--;
-	    textout_centre_ex(bmp, font, "Left", 120, 100, palette_color[255], 0);
+	    textout_centre(bmp, font, "Left", 120, 100, palette_color[255]);
 	 }
 	 if (joy[0].stick[0].axis[0].d2) {
 	    if (x < 319)
 	       x++;
-	    textout_centre_ex(bmp, font, "Right", 200, 100, palette_color[255], 0);
+	    textout_centre(bmp, font, "Right", 200, 100, palette_color[255]);
 	 }
 	 if (joy[0].stick[0].axis[1].d1) {
 	    if (y > 0)
 	       y--;
-	    textout_centre_ex(bmp, font, "Up", 160, 70, palette_color[255], 0);
+	    textout_centre(bmp, font, "Up", 160, 70, palette_color[255]);
 	 }
 	 if (joy[0].stick[0].axis[1].d2) {
 	    if (y < 199)
 	       y++;
-	    textout_centre_ex(bmp, font, "Down", 160, 130, palette_color[255], 0);
+	    textout_centre(bmp, font, "Down", 160, 130, palette_color[255]);
 	 }
       }
       else {
@@ -171,23 +167,23 @@ int main(void)
 	 y += joy[0].stick[0].axis[1].pos/40;
 
 	 /* for informational purposes, show the input values on screen */
-         textprintf_ex(bmp, font, 0,  0, palette_color[255], 0, "Axis 0: %d", joy[0].stick[0].axis[0].pos);
-         textprintf_ex(bmp, font, 0, 10, palette_color[255], 0, "Axis 1: %d", joy[0].stick[0].axis[1].pos);
+         textprintf(bmp, font, 0,  0, palette_color[255], "Axis 0: %d", joy[0].stick[0].axis[0].pos);
+         textprintf(bmp, font, 0, 10, palette_color[255], "Axis 1: %d", joy[0].stick[0].axis[1].pos);
 
 	 /* by checking if the values were positive or negative, we
 	  * can know in which the direction the user pulled the joy.
 	  */
 	 if (joy[0].stick[0].axis[0].pos/40 < 0) 
-	    textout_centre_ex(bmp, font, "Left", 120, 100, palette_color[255], 0);
+	    textout_centre(bmp, font, "Left", 120, 100, palette_color[255]);
 
 	 if (joy[0].stick[0].axis[0].pos/40 > 0) 
-	    textout_centre_ex(bmp, font, "Right", 200, 100, palette_color[255], 0);
+	    textout_centre(bmp, font, "Right", 200, 100, palette_color[255]);
 
 	 if (joy[0].stick[0].axis[1].pos/40 < 0) 
-	    textout_centre_ex(bmp, font, "Up", 160, 70, palette_color[255], 0);
+	    textout_centre(bmp, font, "Up", 160, 70, palette_color[255]);
 
 	 if (joy[0].stick[0].axis[1].pos/40 > 0) 
-	    textout_centre_ex(bmp, font, "Down", 160, 130, palette_color[255], 0);
+	    textout_centre(bmp, font, "Down", 160, 130, palette_color[255]);
 
 	 /* WARNING! An analog joystick can move more than 1 pixel at
 	  * a time and the checks we did with the digital part don't
