@@ -1,16 +1,14 @@
 /*
  *    Example program for the Allegro library, by Bertrand Coconnier.
  *
- *    This program demonstrates how to use scanline sorting algo
- *    in Allegro (create_scene, clear_scene, ... functions). It
- *    also provides an example of how to use the 3D clipping
- *    function. The example consists of a flyby through a lot of
- *    rotating 3d cubes.
+ *    This program demonstrates how to use scanline sorting algo in Allegro 
+ *    (create_scene, clear_scene, ... functions). It also provides an example
+ *    of how to use the 3D clipping function.
  *
  */
 
 
-#include <allegro.h>
+#include "allegro.h"
 
 
 
@@ -60,7 +58,7 @@ void tick(void)
    t++;
 }
 
-END_OF_FUNCTION(tick)
+END_OF_FUNCTION(tick);
 
 
 
@@ -76,7 +74,7 @@ int init_gfx(PALETTE *pal)
    /* color 0 = black */
    (*pal)[0].r = (*pal)[0].g = (*pal)[0].b = 0;
    /* color 1 = red */
-   (*pal)[1].r = 63;
+   (*pal)[1].r = 255;
    (*pal)[1].g = (*pal)[1].b = 0;
 
    /* copy the desktop palette */
@@ -89,9 +87,6 @@ int init_gfx(PALETTE *pal)
       (*pal)[i].g = (*pal)[i].r = 0;
    }
 
-   for (i=96; i<256; i++)
-      (*pal)[i].r = (*pal)[i].g = (*pal)[i].b = 0; 
-
    /* set the graphics mode */
    if (set_gfx_mode(GFX_SAFE, 320, 200, 0, 0) != 0) {
       set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
@@ -124,7 +119,7 @@ int init_gfx(PALETTE *pal)
 /* draw_cube:
  *  Translates, rotates, clips, projects, culls backfaces and draws a cube.
  */
-void draw_cube(MATRIX_f *matrix, int num_poly)
+void draw_cube(BITMAP *buffer, MATRIX_f *matrix, int num_poly)
 {
    int i, j, nv;
    int out[12];
@@ -137,12 +132,10 @@ void draw_cube(MATRIX_f *matrix, int num_poly)
       }
       
       /* nv: number of vertices after clipping is done */
-      nv = clip3d_f(POLYTYPE_GCOL, 0.1, 1000., 4, (AL_CONST V3D_f**)pv, pvout,
-		    pvtmp, out);
+      nv = clip3d_f(POLYTYPE_GCOL, 0.1, 1000., 4, (AL_CONST V3D_f**)pv, pvout, pvtmp, out);
       if (nv) {
 	 for (j=0; j<nv; j++)
-	    persp_project_f(vout[j].x, vout[j].y, vout[j].z, &vout[j].x,
-			    &vout[j].y);
+	    persp_project_f(vout[j].x, vout[j].y, vout[j].z, &vout[j].x, &vout[j].y);
 
 	 if (polygon_z_normal_f(&vout[0], &vout[1], &vout[2]) > 0)
 	    scene_polygon3d_f(POLYTYPE_GCOL, NULL, nv, pvout);
@@ -164,8 +157,7 @@ int main(void)
    int frame = 0;
    float fps = 0.;
    
-   if (allegro_init() != 0)
-      return 1;
+   allegro_init();
    install_keyboard();
    install_mouse();
    install_timer();
@@ -183,9 +175,9 @@ int main(void)
    
    /* initialize buffers and viewport */
    buffer = create_bitmap(SCREEN_W, SCREEN_H);
-   create_scene(24 * MAX_CUBES * MAX_CUBES * MAX_CUBES,
-		6 * MAX_CUBES * MAX_CUBES * MAX_CUBES);
+   create_scene(24*MAX_CUBES*MAX_CUBES*MAX_CUBES, 6*MAX_CUBES*MAX_CUBES*MAX_CUBES);
    set_projection_viewport(0, 0, SCREEN_W, SCREEN_H);
+   text_mode(-1);
    
    /* initialize pointers */
    for (j=0; j<4; j++)
@@ -210,24 +202,21 @@ int main(void)
          for (j=0; j<MAX_CUBES; j++)
 	      for (i=0; i<MAX_CUBES; i++) {
 		/* matrix1: locates cubes */
-		get_translation_matrix_f(&matrix1, j*40-MAX_CUBES*20+20,
-					 i*40-MAX_CUBES*20+20, tz+k*40);
+		get_translation_matrix_f(&matrix1, j*40-MAX_CUBES*20+20, i*40-MAX_CUBES*20+20, tz+k*40);
 		
-		/* matrix: rotates cube THEN locates cube THEN turns
-		 * head right/left */
+		/* matrix: rotates cube THEN locates cube THEN turns head right/left */
 		matrix_mul_f(&matrix2, &matrix1, &matrix);
 		matrix_mul_f(&matrix, &matrix3, &matrix);
 
 		/* cubes are just added to the scene.
 		 * No sorting is done at this stage.
 		 */
-		draw_cube(&matrix, 6);
+		draw_cube(buffer, &matrix, 6);
 	      }
       
       /* sorts and renders polys */
       render_scene();
-      textprintf_ex(buffer, font, 2, 2, palette_color[1], -1,
-		    "(%.1f fps)", fps);
+      textprintf(buffer, font, 2, 2, palette_color[1], "(%.1f fps)", fps);
       blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
       frame++;
       
@@ -253,4 +242,4 @@ int main(void)
    return 0;
 }
 
-END_OF_MAIN()
+END_OF_MAIN();
