@@ -19,13 +19,13 @@
 #ifndef ALLEGRO_GFX_H
 #define ALLEGRO_GFX_H
 
-#include "base.h"
-#include "fixed.h"
-
 #ifdef __cplusplus
    extern "C" {
 #endif
 
+#include "base.h"
+#include "fixed.h"
+
 struct RLE_SPRITE;
 struct FONT_GLYPH;
 struct RGB;
@@ -143,7 +143,6 @@ typedef struct GFX_VTABLE        /* functions for drawing onto bitmaps */
    AL_METHOD(void, hline, (struct BITMAP *bmp, int x1, int y, int x2, int color));
    AL_METHOD(void, hfill, (struct BITMAP *bmp, int x1, int y, int x2, int color));
    AL_METHOD(void, line, (struct BITMAP *bmp, int x1, int y1, int x2, int y2, int color));
-   AL_METHOD(void, fastline, (struct BITMAP *bmp, int x1, int y1, int x2, int y2, int color));
    AL_METHOD(void, rectfill, (struct BITMAP *bmp, int x1, int y1, int x2, int y2, int color));
    AL_METHOD(int,  triangle, (struct BITMAP *bmp, int x1, int y1, int x2, int y2, int x3, int y3, int color));
    AL_METHOD(void, draw_sprite, (struct BITMAP *bmp, struct BITMAP *sprite, int x, int y));
@@ -158,8 +157,8 @@ typedef struct GFX_VTABLE        /* functions for drawing onto bitmaps */
    AL_METHOD(void, draw_trans_rle_sprite, (struct BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y));
    AL_METHOD(void, draw_trans_rgba_rle_sprite, (struct BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y));
    AL_METHOD(void, draw_lit_rle_sprite, (struct BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y, int color));
-   AL_METHOD(void, draw_character, (struct BITMAP *bmp, struct BITMAP *sprite, int x, int y, int color, int bg));
-   AL_METHOD(void, draw_glyph, (struct BITMAP *bmp, AL_CONST struct FONT_GLYPH *glyph, int x, int y, int color, int bg));
+   AL_METHOD(void, draw_character, (struct BITMAP *bmp, struct BITMAP *sprite, int x, int y, int color));
+   AL_METHOD(void, draw_glyph, (struct BITMAP *bmp, AL_CONST struct FONT_GLYPH *glyph, int x, int y, int color));
    AL_METHOD(void, blit_from_memory, (struct BITMAP *source, struct BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
    AL_METHOD(void, blit_to_memory, (struct BITMAP *source, struct BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
    AL_METHOD(void, blit_from_system, (struct BITMAP *source, struct BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
@@ -231,7 +230,7 @@ typedef struct BITMAP            /* a bitmap structure */
    int x_ofs;                    /* horizontal offset (for sub-bitmaps) */
    int y_ofs;                    /* vertical offset (for sub-bitmaps) */
    int seg;                      /* bitmap segment */
-   ZERO_SIZE_ARRAY(unsigned char *, line);
+   unsigned char *line[ZERO_SIZE];
 } BITMAP;
 
 
@@ -344,9 +343,7 @@ AL_VAR(BITMAP *, screen);
 AL_FUNC(GFX_MODE_LIST *, get_gfx_mode_list, (int card));
 AL_FUNC(void, destroy_gfx_mode_list, (GFX_MODE_LIST *gfx_mode_list));
 AL_FUNC(void, set_color_depth, (int depth));
-AL_FUNC(int, get_color_depth, (void));
 AL_FUNC(void, set_color_conversion, (int mode));
-AL_FUNC(int, get_color_conversion, (void));
 AL_FUNC(void, request_refresh_rate, (int rate));
 AL_FUNC(int, get_refresh_rate, (void));
 AL_FUNC(int, set_gfx_mode, (int card, int w, int h, int v_w, int v_h));
@@ -362,8 +359,7 @@ AL_FUNC(BITMAP *, create_sub_bitmap, (BITMAP *parent, int x, int y, int width, i
 AL_FUNC(BITMAP *, create_video_bitmap, (int width, int height));
 AL_FUNC(BITMAP *, create_system_bitmap, (int width, int height));
 AL_FUNC(void, destroy_bitmap, (BITMAP *bitmap));
-AL_FUNC(void, set_clip_rect, (BITMAP *bitmap, int x1, int y1, int x2, int y2));
-AL_FUNC(void, add_clip_rect, (BITMAP *bitmap, int x1, int y1, int x2, int y2));
+AL_FUNC(void, set_clip, (BITMAP *bitmap, int x1, int y1, int x2, int y2));
 AL_FUNC(void, clear_bitmap, (BITMAP *bitmap));
 AL_FUNC(void, vsync, (void));
 
@@ -385,12 +381,12 @@ AL_FUNC(void, remove_display_switch_callback, (AL_METHOD(void, cb, (void))));
 
 AL_FUNC(void, lock_bitmap, (struct BITMAP *bmp));
 
+#include "inline/gfx.inl"
+
 #ifdef __cplusplus
    }
 #endif
 
-#include "inline/gfx.inl"
-
 #endif          /* ifndef ALLEGRO_GFX_H */
 
 
