@@ -28,10 +28,6 @@
 #endif
 
 
-/* length in bytes of the cpu_vendor string */
-#define _AL_CPU_VENDOR_SIZE 32
-
-
 /* flag for how many times we have been initialised */
 AL_VAR(int, _allegro_count);
 
@@ -136,8 +132,6 @@ AL_VAR(int, _timer_installed);
 AL_VAR(int, _timer_use_retrace);
 AL_VAR(volatile int, _retrace_hpp_value);
 
-AL_VAR(long, _vsync_speed);
-
 
 /* various bits of keyboard stuff */
 AL_FUNC(void, _handle_key_press, (int keycode, int scancode));
@@ -286,9 +280,7 @@ AL_VAR(GFX_VTABLE, _screen_vtable);
 AL_VAR(int, _gfx_mode_set_count);
 
 AL_VAR(int, _refresh_rate_request);
-AL_FUNC(void, _set_current_refresh_rate, (int rate));
-
-AL_VAR(int, _wait_for_vsync);
+AL_VAR(int, _current_refresh_rate);
 
 AL_VAR(int, _sub_bitmap_id_count);
 
@@ -318,30 +310,11 @@ AL_VAR(int, _color_conv);
 AL_FUNC(BITMAP *, _fixup_loaded_bitmap, (BITMAP *bmp, PALETTE pal, int bpp));
 
 
-/* default truecolor pixel format */
-#define DEFAULT_RGB_R_SHIFT_15  0
-#define DEFAULT_RGB_G_SHIFT_15  5
-#define DEFAULT_RGB_B_SHIFT_15  10
-#define DEFAULT_RGB_R_SHIFT_16  0
-#define DEFAULT_RGB_G_SHIFT_16  5
-#define DEFAULT_RGB_B_SHIFT_16  11
-#define DEFAULT_RGB_R_SHIFT_24  0
-#define DEFAULT_RGB_G_SHIFT_24  8
-#define DEFAULT_RGB_B_SHIFT_24  16
-#define DEFAULT_RGB_R_SHIFT_32  0
-#define DEFAULT_RGB_G_SHIFT_32  8
-#define DEFAULT_RGB_B_SHIFT_32  16
-#define DEFAULT_RGB_A_SHIFT_32  24
-
-
-/* display switching support */
-AL_FUNC(void, _switch_in, (void));
-AL_FUNC(void, _switch_out, (void));
-
+/* console switching support */
 AL_FUNC(void, _register_switch_bitmap, (BITMAP *bmp, BITMAP *parent));
 AL_FUNC(void, _unregister_switch_bitmap, (BITMAP *bmp));
 AL_FUNC(void, _save_switch_state, (int switch_mode));
-AL_FUNC(void, _restore_switch_state, (void));
+AL_FUNC(void, _restore_switch_state, ());
 
 AL_VAR(int, _dispsw_status);
 
@@ -354,6 +327,8 @@ AL_VAR(int, _drawing_y_anchor);
 AL_VAR(unsigned int, _drawing_x_mask);
 AL_VAR(unsigned int, _drawing_y_mask);
 
+AL_VAR(int, _textmode);
+
 AL_FUNCPTR(int *, _palette_expansion_table, (int bpp));
 
 AL_VAR(int, _color_depth);
@@ -464,8 +439,8 @@ AL_FUNC(void, _linear_draw_lit_sprite8, (BITMAP *bmp, BITMAP *sprite, int x, int
 AL_FUNC(void, _linear_draw_rle_sprite8, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y));
 AL_FUNC(void, _linear_draw_trans_rle_sprite8, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y));
 AL_FUNC(void, _linear_draw_lit_rle_sprite8, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y, int color));
-AL_FUNC(void, _linear_draw_character8, (BITMAP *bmp, BITMAP *sprite, int x, int y, int color, int bg));
-AL_FUNC(void, _linear_draw_glyph8, (BITMAP *bmp, AL_CONST FONT_GLYPH *glyph, int x, int y, int color, int bg));
+AL_FUNC(void, _linear_draw_character8, (BITMAP *bmp, BITMAP *sprite, int x, int y, int color));
+AL_FUNC(void, _linear_draw_glyph8, (BITMAP *bmp, AL_CONST FONT_GLYPH *glyph, int x, int y, int color));
 AL_FUNC(void, _linear_blit8, (BITMAP *source,BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
 AL_FUNC(void, _linear_blit_backward8, (BITMAP *source,BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
 AL_FUNC(void, _linear_masked_blit8, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
@@ -502,8 +477,8 @@ AL_FUNC(void, _linear_draw_rle_sprite16, (BITMAP *bmp, AL_CONST struct RLE_SPRIT
 AL_FUNC(void, _linear_draw_trans_rle_sprite16, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y));
 AL_FUNC(void, _linear_draw_trans_rgba_rle_sprite16, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y));
 AL_FUNC(void, _linear_draw_lit_rle_sprite16, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y, int color));
-AL_FUNC(void, _linear_draw_character16, (BITMAP *bmp, BITMAP *sprite, int x, int y, int color, int bg));
-AL_FUNC(void, _linear_draw_glyph16, (BITMAP *bmp, AL_CONST FONT_GLYPH *glyph, int x, int y, int color, int bg));
+AL_FUNC(void, _linear_draw_character16, (BITMAP *bmp, BITMAP *sprite, int x, int y, int color));
+AL_FUNC(void, _linear_draw_glyph16, (BITMAP *bmp, AL_CONST FONT_GLYPH *glyph, int x, int y, int color));
 AL_FUNC(void, _linear_blit16, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
 AL_FUNC(void, _linear_blit_backward16, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
 AL_FUNC(void, _linear_masked_blit16, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
@@ -529,8 +504,8 @@ AL_FUNC(void, _linear_draw_rle_sprite24, (BITMAP *bmp, AL_CONST struct RLE_SPRIT
 AL_FUNC(void, _linear_draw_trans_rle_sprite24, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y));
 AL_FUNC(void, _linear_draw_trans_rgba_rle_sprite24, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y));
 AL_FUNC(void, _linear_draw_lit_rle_sprite24, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y, int color));
-AL_FUNC(void, _linear_draw_character24, (BITMAP *bmp, BITMAP *sprite, int x, int y, int color, int bg));
-AL_FUNC(void, _linear_draw_glyph24, (BITMAP *bmp, AL_CONST FONT_GLYPH *glyph, int x, int y, int color, int bg));
+AL_FUNC(void, _linear_draw_character24, (BITMAP *bmp, BITMAP *sprite, int x, int y, int color));
+AL_FUNC(void, _linear_draw_glyph24, (BITMAP *bmp, AL_CONST FONT_GLYPH *glyph, int x, int y, int color));
 AL_FUNC(void, _linear_blit24, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
 AL_FUNC(void, _linear_blit_backward24, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
 AL_FUNC(void, _linear_masked_blit24, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
@@ -554,8 +529,8 @@ AL_FUNC(void, _linear_draw_lit_sprite32, (BITMAP *bmp, BITMAP *sprite, int x, in
 AL_FUNC(void, _linear_draw_rle_sprite32, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y));
 AL_FUNC(void, _linear_draw_trans_rle_sprite32, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y));
 AL_FUNC(void, _linear_draw_lit_rle_sprite32, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y, int color));
-AL_FUNC(void, _linear_draw_character32, (BITMAP *bmp, BITMAP *sprite, int x, int y, int color, int bg));
-AL_FUNC(void, _linear_draw_glyph32, (BITMAP *bmp, AL_CONST FONT_GLYPH *glyph, int x, int y, int color, int bg));
+AL_FUNC(void, _linear_draw_character32, (BITMAP *bmp, BITMAP *sprite, int x, int y, int color));
+AL_FUNC(void, _linear_draw_glyph32, (BITMAP *bmp, AL_CONST FONT_GLYPH *glyph, int x, int y, int color));
 AL_FUNC(void, _linear_blit32, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
 AL_FUNC(void, _linear_blit_backward32, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
 AL_FUNC(void, _linear_masked_blit32, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
@@ -578,8 +553,8 @@ AL_FUNC(void, _x_draw_lit_sprite, (BITMAP *bmp, BITMAP *sprite, int x, int y, in
 AL_FUNC(void, _x_draw_rle_sprite, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y));
 AL_FUNC(void, _x_draw_trans_rle_sprite, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y));
 AL_FUNC(void, _x_draw_lit_rle_sprite, (BITMAP *bmp, AL_CONST struct RLE_SPRITE *sprite, int x, int y, int color));
-AL_FUNC(void, _x_draw_character, (BITMAP *bmp, BITMAP *sprite, int x, int y, int color, int bg));
-AL_FUNC(void, _x_draw_glyph, (BITMAP *bmp, AL_CONST FONT_GLYPH *glyph, int x, int y, int color, int bg));
+AL_FUNC(void, _x_draw_character, (BITMAP *bmp, BITMAP *sprite, int x, int y, int color));
+AL_FUNC(void, _x_draw_glyph, (BITMAP *bmp, AL_CONST FONT_GLYPH *glyph, int x, int y, int color));
 AL_FUNC(void, _x_blit_from_memory, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
 AL_FUNC(void, _x_blit_to_memory, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
 AL_FUNC(void, _x_blit, (BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height));
@@ -1112,7 +1087,7 @@ typedef struct DATAFILE_TYPE
 {
    int type;
    AL_METHOD(void *, load, (PACKFILE *f, long size));
-   AL_METHOD(void, destroy, (void *));
+   AL_METHOD(void, destroy, ());
 } DATAFILE_TYPE;
 
 
@@ -1128,9 +1103,6 @@ AL_FUNC(void, _construct_datafile, (DATAFILE *data));
 /* for readbmp.c */
 AL_FUNC(void, _register_bitmap_file_type_init, (void));
 
-/* for readsmp.c */
-AL_FUNC(void, _register_sample_file_type_init, (void));
-
 
 /* for module linking system; see comment in allegro.c */
 struct _AL_LINKER_MIDI
@@ -1152,20 +1124,15 @@ AL_VAR(struct _AL_LINKER_MOUSE *, _al_linker_mouse);
 
 
 /* dynamic driver lists */
-AL_FUNC(_DRIVER_INFO *, _create_driver_list, (void));
+AL_FUNC(_DRIVER_INFO *, _create_driver_list, ());
 AL_FUNC(void, _destroy_driver_list, (_DRIVER_INFO *drvlist));
 AL_FUNC(void, _driver_list_append_driver, (_DRIVER_INFO **drvlist, int id, void *driver, int autodetect));
 AL_FUNC(void, _driver_list_prepend_driver, (_DRIVER_INFO **drvlist, int id, void *driver, int autodetect));
 AL_FUNC(void, _driver_list_append_list, (_DRIVER_INFO **drvlist, _DRIVER_INFO *srclist));
 
 
-/* various libc stuff */
 AL_FUNC(void *, _al_sane_realloc, (void *ptr, size_t size));
 
-#define _AL_RAND_MAX  0xFFFF
-AL_FUNC(void, _al_srand, (int seed));
-AL_FUNC(int, _al_rand, (void));
-
 
 #ifdef __cplusplus
    }
