@@ -19,12 +19,12 @@
 #ifndef ALLEGRO_MOUSE_H
 #define ALLEGRO_MOUSE_H
 
-#include "base.h"
-
 #ifdef __cplusplus
    extern "C" {
 #endif
 
+#include "base.h"
+
 #define MOUSEDRV_AUTODETECT  -1
 #define MOUSEDRV_NONE         0
 
@@ -40,12 +40,10 @@ typedef struct MOUSE_DRIVER
    AL_METHOD(void, poll, (void));
    AL_METHOD(void, timer_poll, (void));
    AL_METHOD(void, position, (int x, int y));
-   AL_METHOD(void, set_range, (int x1, int y_1, int x2, int y2));
+   AL_METHOD(void, set_range, (int x1, int y1, int x2, int y2));
    AL_METHOD(void, set_speed, (int xspeed, int yspeed));
    AL_METHOD(void, get_mickeys, (int *mickeyx, int *mickeyy));
    AL_METHOD(int,  analyse_data, (AL_CONST char *buffer, int size));
-   AL_METHOD(void, enable_hardware_cursor, (int mode));
-   AL_METHOD(int,  select_system_cursor, (int cursor));
 } MOUSE_DRIVER;
 
 
@@ -59,18 +57,6 @@ AL_FUNC(void, remove_mouse, (void));
 AL_FUNC(int, poll_mouse, (void));
 AL_FUNC(int, mouse_needs_poll, (void));
 
-AL_FUNC(void, enable_hardware_cursor, (void));
-AL_FUNC(void, disable_hardware_cursor, (void));
-
-/* Mouse cursors */
-#define MOUSE_CURSOR_NONE        0
-#define MOUSE_CURSOR_ALLEGRO     1
-#define MOUSE_CURSOR_ARROW       2
-#define MOUSE_CURSOR_BUSY        3
-#define MOUSE_CURSOR_QUESTION    4
-#define MOUSE_CURSOR_EDIT        5
-#define NUM_MOUSE_CURSORS        6
-
 AL_VAR(struct BITMAP *, mouse_sprite);
 AL_VAR(int, mouse_x_focus);
 AL_VAR(int, mouse_y_focus);
@@ -100,14 +86,12 @@ AL_FUNC(void, scare_mouse_area, (int x, int y, int w, int h));
 AL_FUNC(void, unscare_mouse, (void));
 AL_FUNC(void, position_mouse, (int x, int y));
 AL_FUNC(void, position_mouse_z, (int z));
-AL_FUNC(void, set_mouse_range, (int x1, int y_1, int x2, int y2));
+AL_FUNC(void, set_mouse_range, (int x1, int y1, int x2, int y2));
 AL_FUNC(void, set_mouse_speed, (int xspeed, int yspeed));
-AL_FUNC(void, select_mouse_cursor, (int cursor));
-AL_FUNC(void, set_mouse_cursor_bitmap, (int cursor, struct BITMAP *bmp));
+AL_FUNC(void, set_mouse_sprite, (struct BITMAP *sprite));
 AL_FUNC(void, set_mouse_sprite_focus, (int x, int y));
 AL_FUNC(void, get_mouse_mickeys, (int *mickeyx, int *mickeyy));
-AL_FUNC(void, set_mouse_sprite, (struct BITMAP *sprite));
-AL_FUNC(int, show_os_cursor, (int cursor));
+
 #ifdef __cplusplus
    }
 #endif
