@@ -27,6 +27,10 @@
    #error bad include
 #endif
 
+#ifdef __cplusplus
+   extern "C" {
+#endif
+
 
 #include "winalleg.h"
 
@@ -45,11 +49,9 @@
 #endif
 
 
-#ifdef __cplusplus
-   extern "C" {
-#endif
-
-/* generals */
+/*******************************************/
+/***************** general *****************/
+/*******************************************/
 AL_VAR(HINSTANCE, allegro_inst);
 AL_VAR(HANDLE, allegro_thread);
 AL_VAR(CRITICAL_SECTION, allegro_critical_section);
@@ -58,17 +60,19 @@ AL_VAR(int, _dx_ver);
 #define _enter_critical()  EnterCriticalSection(&allegro_critical_section)
 #define _exit_critical()   LeaveCriticalSection(&allegro_critical_section)
 
+
 AL_FUNC(int, init_directx_window, (void));
 AL_FUNC(void, exit_directx_window, (void));
+AL_FUNC(int, wnd_call_proc, (AL_METHOD(int, proc, (void))));
 AL_FUNC(int, get_dx_ver, (void));
-AL_FUNC(int, adjust_window, (int w, int h));
+AL_FUNC(void, set_sync_timer_freq, (int freq));
 AL_FUNC(void, restore_window_style, (void));
-AL_FUNC(void, save_window_pos, (void));
 
 
 /* main window */
 #define WND_TITLE_SIZE  128
 
+AL_VAR(HWND, allegro_wnd);
 AL_ARRAY(char, wnd_title);
 AL_VAR(int, wnd_x);
 AL_VAR(int, wnd_y);
@@ -89,92 +93,69 @@ AL_VAR(int, gfx_crit_sect_nesting);
                                gfx_crit_sect_nesting--
 #define GFX_CRITICAL_RELEASED  (!gfx_crit_sect_nesting)
 
+AL_FUNC(void, gfx_directx_restore, (void));
 
-/* switch routines */
-AL_VAR(int, _win_app_foreground);
+
+/* focus switch routines */
+AL_VAR(BOOL, app_foreground);
+AL_VAR(HANDLE, _foreground_event);
 
 AL_FUNC(void, sys_directx_display_switch_init, (void));
 AL_FUNC(void, sys_directx_display_switch_exit, (void));
 AL_FUNC(int, sys_directx_set_display_switch_mode, (int mode));
+AL_FUNC(int, sys_directx_set_display_switch_callback, (int dir, AL_METHOD(void, cb, (void))));
+AL_FUNC(void, sys_directx_remove_display_switch_callback, (AL_METHOD(void, cb, (void))));
 
-AL_FUNC(void, _win_switch_in, (void));
-AL_FUNC(void, _win_switch_out, (void));
-AL_FUNC(void, _win_reset_switch_mode, (void));
+AL_FUNC(void, sys_switch_in, (void));
 
-AL_FUNC(int, _win_thread_switch_out, (void));
+AL_FUNC(void, sys_switch_out, (void));
+AL_FUNC(void, thread_switch_out, (void));
+AL_FUNC(void, midi_switch_out, (void));
+
+AL_FUNC(void, sys_directx_switch_out_callback, (void));
+AL_FUNC(void, sys_directx_switch_in_callback, (void));
 
 
-/* main window routines */
-AL_FUNC(int, wnd_call_proc, (int (*proc)(void)));
-AL_FUNC(void, wnd_schedule_proc, (int (*proc)(void)));
 
-
-/* input routines */
-AL_VAR(int, _win_input_events);
-AL_ARRAY(HANDLE, _win_input_event_id);
-AL_FUNCPTRARRAY(void, _win_input_event_handler, (void));
-
-AL_FUNC(void, _win_input_init, (int need_thread));
-AL_FUNC(void, _win_input_exit, (void));
-AL_FUNC(int, _win_input_register_event, (HANDLE event_id, void (*event_handler)(void)));
-AL_FUNC(void, _win_input_unregister_event, (HANDLE event_id));
-
-
-/* keyboard routines */
+/*******************************************/
+/************** keyboard routines **********/
+/*******************************************/
 AL_FUNC(int, key_dinput_acquire, (void));
 AL_FUNC(int, key_dinput_unacquire, (void));
+AL_FUNC(void, wnd_acquire_keyboard, (void));
+AL_FUNC(void, wnd_unacquire_keyboard, (void));
 
 
-/* mouse routines */
-AL_VAR(HCURSOR, _win_hcursor);
+
+/*******************************************/
+/************** mouse routines *************/
+/*******************************************/
 AL_FUNC(int, mouse_dinput_acquire, (void));
 AL_FUNC(int, mouse_dinput_unacquire, (void));
-AL_FUNC(int, mouse_dinput_grab, (void));
-AL_FUNC(int, mouse_set_syscursor, (void));
+AL_FUNC(int, mouse_set_syscursor, (int state));
 AL_FUNC(int, mouse_set_sysmenu, (int state));
+AL_FUNC(void, wnd_acquire_mouse, (void));
+AL_FUNC(void, wnd_unacquire_mouse, (void));
+AL_FUNC(void, wnd_set_syscursor, (int state));
 
 
-/* joystick routines */
-#define WINDOWS_MAX_AXES   6
 
-#define WINDOWS_JOYSTICK_INFO_MEMBERS        \
-   int caps;                                 \
-   int num_axes;                             \
-   int axis[WINDOWS_MAX_AXES];               \
-   char *axis_name[WINDOWS_MAX_AXES];        \
-   int hat;                                  \
-   char *hat_name;                           \
-   int num_buttons;                          \
-   int button[MAX_JOYSTICK_BUTTONS];         \
-   char *button_name[MAX_JOYSTICK_BUTTONS];
-
-typedef struct WINDOWS_JOYSTICK_INFO {
-   WINDOWS_JOYSTICK_INFO_MEMBERS
-} WINDOWS_JOYSTICK_INFO;
-
-AL_FUNC(int, win_add_joystick, (WINDOWS_JOYSTICK_INFO *win_joy));
-AL_FUNC(void, win_remove_all_joysticks, (void));
-AL_FUNC(int, win_update_joystick_status, (int n, WINDOWS_JOYSTICK_INFO *win_joy));
-
-AL_FUNC(int, joystick_dinput_acquire, (void));
-AL_FUNC(int, joystick_dinput_unacquire, (void));
+/*******************************************/
+/************* thread routines *************/
+/*******************************************/
+AL_FUNC(void, win_init_thread, (void));
+AL_FUNC(void, win_exit_thread, (void));
 
 
-/* thread routines */
-AL_FUNC(void, _win_thread_init, (void));
-AL_FUNC(void, _win_thread_exit, (void));
 
-
-/* synchronization routines */
-AL_FUNC(void *, sys_directx_create_mutex, (void));
-AL_FUNC(void, sys_directx_destroy_mutex, (void *handle));
-AL_FUNC(void, sys_directx_lock_mutex, (void *handle));
-AL_FUNC(void, sys_directx_unlock_mutex, (void *handle));
-
-
-/* sound routines */
+/******************************************/
+/************* sound routines *************/
+/******************************************/
 AL_FUNC(_DRIVER_INFO *, _get_win_digi_driver_list, (void));
+AL_FUNC(_DRIVER_INFO *, _get_win_midi_driver_list, (void));
+
 AL_FUNC(void, _free_win_digi_driver_list, (void));
+AL_FUNC(void, _free_win_midi_driver_list, (void));
 
 AL_FUNC(DIGI_DRIVER *, _get_dsalmix_driver, (char *name, LPGUID guid, int num));
 AL_FUNC(DIGI_DRIVER *, _get_woalmix_driver, (int num));
@@ -190,27 +171,24 @@ AL_FUNC(void, digi_directsound_rec_stop, (void));
 AL_FUNC(int, digi_directsound_rec_read, (void *buf));
 
 
-/* midi routines */
-AL_FUNC(_DRIVER_INFO *, _get_win_midi_driver_list, (void));
-AL_FUNC(void, _free_win_midi_driver_list, (void));
 
-AL_FUNC(void, midi_switch_out, (void));
-
-
-/* error handling */
+/*******************************************/
+/************* error handling **************/
+/*******************************************/
 AL_FUNC(char* , win_err_str, (long err));
 AL_FUNC(void, thread_safe_trace, (char *msg, ...));
 
-#if DEBUGMODE >= 2
+#ifdef DEBUGMODE
    #define _TRACE                 thread_safe_trace
 #else
    #define _TRACE                 1 ? (void) 0 : thread_safe_trace
 #endif
 
 
+
 #ifdef __cplusplus
    }
 #endif
 
-#endif          /* !defined AINTWIN_H */
+#endif          /* ifndef AINTWIN_H */
 
