@@ -13,7 +13,7 @@ shift; shift
 sources=$*
 objects=`echo $sources | sed 's,[^	 ]*/,,g;s,\.[^.	 ]*,,g'`
 
-MODNAME=`echo $modname | tr a-z A-Z`
+MODNAME=`echo $modname | tr [a-z] [A-Z]`
 
 module="alleg-${modname}.so"
 objlist="MODULE_${MODNAME}_OBJECTS"
@@ -33,9 +33,9 @@ echo ""
 # module rule
 echo "\$(LIBDIR)/$module: \$(${objlist})"
 if test "$modlibs" = "--"; then
-   echo "	\$(CC) -shared \$(ALLEGRO_SHAREDLIB_CFLAGS) -o \$@ \$(${objlist}) \$(LDFLAGS)"
+   echo "	gcc -shared \$(ALLEGRO_SHAREDLIB_CFLAGS) -o \$@ \$(${objlist}) \$(LDFLAGS)"
 else
-   echo "	\$(CC) -shared \$(ALLEGRO_SHAREDLIB_CFLAGS) -o \$@ \$(${objlist}) \$(LDFLAGS) $modlibs"
+   echo "	gcc -shared \$(ALLEGRO_SHAREDLIB_CFLAGS) -o \$@ \$(${objlist}) \$(LDFLAGS) $modlibs"
 fi
 echo ""
 
