@@ -769,7 +769,7 @@ static int scroller_proc(int msg, DIALOG *d, int c)
       p = scroller_msg;
       for (i=0; i<SCROLLER_LENGTH; i++) {
 	 x = i*8+scroller_pos;
-	 a = 16 + MID(0, 15-ABS(SCREEN_W/2-x)/(SCREEN_W/32), 15) * scroller_alpha/256;
+	 a = 16 + CLAMP(0, 15-ABS(SCREEN_W/2-x)/(SCREEN_W/32), 15) * scroller_alpha/256;
 	 textprintf_ex(screen, font, x, SCREEN_H-16, a, 0, uconvert_ascii("%c", tmp), ugetx(&p));
       }
 
@@ -917,7 +917,7 @@ static int update(void)
       else
 	 pos *= 128;
 
-      pos = MID(0, 4096-pos, 4096);
+      pos = CLAMP(0, 4096-pos, 4096);
       pppos = (4096 - pos * pos / 4096);
       pos = pppos / 16;
 
@@ -1338,7 +1338,7 @@ static void mouse_proc(int type)
 static void get_mouse_drivers(_DRIVER_INFO **list, int *list_size);
 
 /* Number of devices tested. */
-#define NUM_FD 4
+#define NUM_FD 6
 
 /* Some devices can't be read one byte at a time. */
 #define PACKET_SIZE 16
@@ -1367,7 +1367,14 @@ static void reset_for_select(int *fd, fd_set *readfds, struct timeval *timeout)
 static int detect_mouse(void)
 {
    int fd[NUM_FD];
-   char *devices[NUM_FD] = {"/dev/mouse", "dev/input/mice", "/dev/input/mouse", "/dev/input/mouse0"};
+   char *devices[NUM_FD] = {
+      "/dev/mouse",
+      "/dev/input/mice", /* Collects all the evdev mice, PS/2-style */
+      "/dev/input/event0",
+      "/dev/input/event1",
+      "/dev/input/event2",
+      "/dev/input/event3"
+   };
    fd_set readfds;
    struct timeval timeout;
    int n, i;
