@@ -16,8 +16,8 @@
  */
 
 #include "bealleg.h"
-#include "allegro/aintern.h"
-#include "allegro/aintbeos.h"
+#include "allegro/internal/aintern.h"
+#include "allegro/platform/aintbeos.h"
 
 #ifndef ALLEGRO_BEOS
 #error something is wrong with the makefile
