@@ -44,8 +44,8 @@ void destroy_compiled_sprite(COMPILED_SPRITE *sprite)
  *  Draws a compiled sprite onto the specified bitmap at the specified
  *  position.
  */
-void draw_compiled_sprite(BITMAP *dst, COMPILED_SPRITE *src, int x, int y)
+void draw_compiled_sprite(BITMAP *dst, AL_CONST COMPILED_SPRITE *src, int x, int y)
 {
-   return draw_rle_sprite(dst, src, x, y);
+   draw_rle_sprite(dst, (COMPILED_SPRITE *)src, x, y);
 }
 
