@@ -12,8 +12,6 @@
  *
  *      By Shawn Hargreaves.
  *
- *      Elias Pschernig made load_datafile_object() handle properties.
- *
  *      See readme.txt for copyright information.
  */
 
@@ -26,20 +24,12 @@
 
 
 static void unload_midi(MIDI *m);
-static void initialise_datafile(DATAFILE *data);
 
 
 
 /* hack to let the grabber prevent compiled sprites from being compiled */
 int _compile_sprites = TRUE;
 
-/* flag to detect whether compiled datafiles are constructed:
- *  A runtime flag is required because the shared version of the library may
- *  have to deal both with executables that use constructors and with
- *  executables that don't.
- */
-static int constructed_datafiles = FALSE;
-
 static void (*datafile_callback)(DATAFILE *) = NULL;
 
 
@@ -106,9 +96,9 @@ static void *read_block(PACKFILE *f, int size, int alloc_size)
 static BITMAP *read_bitmap(PACKFILE *f, int bits, int allowconv)
 {
    int x, y, w, h, c, r, g, b, a;
-   int destbits, rgba;
+   int destbits, dither8, prev_drawmode, rgba;
    unsigned short *p16;
-   uint32_t *p32;
+   unsigned long *p32;
    BITMAP *bmp;
 
    if (bits < 0) {
@@ -123,15 +113,25 @@ static BITMAP *read_bitmap(PACKFILE *f, int bits, int allowconv)
    else
       destbits = 8;
 
+   if ((destbits == 8) && (bits > 8) && (_color_conv & COLORCONV_DITHER_PAL)) {
+      destbits = bits;
+      dither8 = TRUE;
+   }
+   else
+      dither8 = FALSE;
+
    w = pack_mgetw(f);
    h = pack_mgetw(f);
 
-   bmp = create_bitmap_ex(MAX(bits, 8), w, h);
+   bmp = create_bitmap_ex(destbits, w, h);
    if (!bmp) {
       *allegro_errno = ENOMEM;
       return NULL;
    }
 
+   prev_drawmode = _drawing_mode;
+   _drawing_mode = DRAW_MODE_SOLID;
+
    switch (bits) {
 
       case 4:
@@ -139,90 +139,281 @@ static BITMAP *read_bitmap(PACKFILE *f, int bits, int allowconv)
 	 load_st_data(bmp->dat, w*h/2, f);
 	 break;
 
+
       case 8:
 	 /* 256 color bitmap */
-	 pack_fread(bmp->dat, w*h, f);
-
+	 if (destbits == 8) {
+	    pack_fread(bmp->dat, w*h, f);
+	 }
+	 else {
+	    /* expand 256 colors into truecolor */
+	    for (y=0; y<h; y++) {
+	       for (x=0; x<w; x++) {
+		  c = pack_getc(f);
+		  r = getr8(c);
+		  g = getg8(c);
+		  b = getb8(c);
+		  c = makecol_depth(destbits, r, g, b);
+		  putpixel(bmp, x, y, c);
+	       }
+	    }
+	 }
 	 break;
 
+
       case 15:
-	 /* 15bit hicolor */
-	 for (y=0; y<h; y++) {
-	    p16 = (uint16_t *)bmp->line[y];
-
-	    for (x=0; x<w; x++) {
-	       c = pack_igetw(f);
-	       r = _rgb_scale_5[(c >> 11) & 0x1F]; /* stored as 16 bit */
-	       g = _rgb_scale_6[(c >> 5) & 0x3F];
-	       b = _rgb_scale_5[c & 0x1F];
-	       p16[x] = makecol15(r, g, b);
-	    }
-	 }
-	 break;
-
       case 16:
-	 /* 16bit hicolor */
-	 for (y=0; y<h; y++) {
-	    p16 = (uint16_t *)bmp->line[y];
+	 /* hicolor */
+	 switch (destbits) {
+
+	 #ifdef ALLEGRO_COLOR8
+
+	    case 8:
+	       /* reduce hicolor to 256 colors */
+	       for (y=0; y<h; y++) {
+		  for (x=0; x<w; x++) {
+		     c = pack_igetw(f);
+		     r = _rgb_scale_5[(c >> 11) & 0x1F];
+		     g = _rgb_scale_6[(c >> 5) & 0x3F];
+		     b = _rgb_scale_5[c & 0x1F];
+		     bmp->line[y][x] = makecol8(r, g, b);
+		  }
+	       }
+	       break;
+
+	 #endif
+
+	 #ifdef ALLEGRO_COLOR16
+
+	    case 15:
+	       /* load hicolor to a 15 bit hicolor bitmap */
+	       for (y=0; y<h; y++) {
+		  p16 = (unsigned short *)bmp->line[y];
+
+		  for (x=0; x<w; x++) {
+		     c = pack_igetw(f);
+		     r = _rgb_scale_5[(c >> 11) & 0x1F];
+		     g = _rgb_scale_6[(c >> 5) & 0x3F];
+		     b = _rgb_scale_5[c & 0x1F];
+		     p16[x] = makecol15(r, g, b);
+		  }
+	       }
+	       break;
+
+	    case 16:
+	       /* load hicolor to a 16 bit hicolor bitmap */
+	       for (y=0; y<h; y++) {
+		  p16 = (unsigned short *)bmp->line[y];
+
+		  for (x=0; x<w; x++) {
+		     c = pack_igetw(f);
+		     r = _rgb_scale_5[(c >> 11) & 0x1F];
+		     g = _rgb_scale_6[(c >> 5) & 0x3F];
+		     b = _rgb_scale_5[c & 0x1F];
+		     p16[x] = makecol16(r, g, b);
+		  }
+	       }
+	       break;
+
+	 #endif
+
+	 #ifdef ALLEGRO_COLOR24
+
+	    case 24:
+	       /* load hicolor to a 24 bit truecolor bitmap */
+	       for (y=0; y<h; y++) {
+		  for (x=0; x<w; x++) {
+		     c = pack_igetw(f);
+		     r = _rgb_scale_5[(c >> 11) & 0x1F];
+		     g = _rgb_scale_6[(c >> 5) & 0x3F];
+		     b = _rgb_scale_5[c & 0x1F];
+		     bmp->line[y][x*3+_rgb_r_shift_24/8] = r;
+		     bmp->line[y][x*3+_rgb_g_shift_24/8] = g;
+		     bmp->line[y][x*3+_rgb_b_shift_24/8] = b;
+		  }
+	       }
+	       break;
+
+	 #endif
+
+	 #ifdef ALLEGRO_COLOR32
+
+	    case 32:
+	       /* load hicolor to a 32 bit truecolor bitmap */
+	       for (y=0; y<h; y++) {
+		  p32 = (unsigned long *)bmp->line[y];
+
+		  for (x=0; x<w; x++) {
+		     c = pack_igetw(f);
+		     r = _rgb_scale_5[(c >> 11) & 0x1F];
+		     g = _rgb_scale_6[(c >> 5) & 0x3F];
+		     b = _rgb_scale_5[c & 0x1F];
+		     p32[x] = makecol32(r, g, b);
+		  }
+	       }
+	       break;
+
+	 #endif
 
-	    for (x=0; x<w; x++) {
-	       c = pack_igetw(f);
-	       r = _rgb_scale_5[(c >> 11) & 0x1F];
-	       g = _rgb_scale_6[(c >> 5) & 0x3F];
-	       b = _rgb_scale_5[c & 0x1F];
-	       p16[x] = makecol16(r, g, b);
-	    }
 	 }
 	 break;
 
+
       case 24:
-	 /* 24bit truecolor */
-	 for (y=0; y<h; y++) {
-	    for (x=0; x<w; x++) {
-	       r = pack_getc(f);
-	       g = pack_getc(f);
-	       b = pack_getc(f);
-
-	       if (rgba)
-		  pack_getc(f);
-
-	       c = makecol24(r, g, b);
-	       WRITE3BYTES(bmp->line[y] + (x * 3), c);
-	    }
-	 }
-	 break;
-
       case 32:
-	 /* 32bit rgba */
-	 for (y=0; y<h; y++) {
-	    p32 = (uint32_t *)bmp->line[y];
+	 /* truecolor */
+	 switch (destbits) {
 
-	    for (x=0; x<w; x++) {
-	       r = pack_getc(f);
-	       g = pack_getc(f);
-	       b = pack_getc(f);
+	 #ifdef ALLEGRO_COLOR8
 
-	       if (rgba)
-		  a = pack_getc(f);
-	       else
-		  a = 0;
+	    case 8:
+	       /* reduce truecolor to 256 colors */
+	       for (y=0; y<h; y++) {
+		  for (x=0; x<w; x++) {
+		     r = pack_getc(f);
+		     g = pack_getc(f);
+		     b = pack_getc(f);
+		     if (rgba)
+			pack_getc(f);
+		     bmp->line[y][x] = makecol8(r, g, b);
+		  }
+	       }
+	       break;
+
+	 #endif
+
+	 #ifdef ALLEGRO_COLOR16
+
+	    case 15:
+	       /* load truecolor to a 15 bit hicolor bitmap */
+	       for (y=0; y<h; y++) {
+		  p16 = (unsigned short *)bmp->line[y];
+
+		  if (_color_conv & COLORCONV_DITHER_HI) {
+		     for (x=0; x<w; x++) {
+			r = pack_getc(f);
+			g = pack_getc(f);
+			b = pack_getc(f);
+
+			if (rgba)
+			   pack_getc(f);
+
+			p16[x] = makecol15_dither(r, g, b, x, y);
+		     }
+		  }
+		  else {
+		     for (x=0; x<w; x++) {
+			r = pack_getc(f);
+			g = pack_getc(f);
+			b = pack_getc(f);
+
+			if (rgba)
+			   pack_getc(f);
+
+			p16[x] = makecol15(r, g, b);
+		     }
+		  }
+	       }
+	       break;
+
+	    case 16:
+	       /* load truecolor to a 16 bit hicolor bitmap */
+	       for (y=0; y<h; y++) {
+		  p16 = (unsigned short *)bmp->line[y];
+
+		  if (_color_conv & COLORCONV_DITHER_HI) {
+		     for (x=0; x<w; x++) {
+			r = pack_getc(f);
+			g = pack_getc(f);
+			b = pack_getc(f);
+
+			if (rgba)
+			   pack_getc(f);
+
+			p16[x] = makecol16_dither(r, g, b, x, y);
+		     }
+		  }
+		  else {
+		     for (x=0; x<w; x++) {
+			r = pack_getc(f);
+			g = pack_getc(f);
+			b = pack_getc(f);
+
+			if (rgba)
+			   pack_getc(f);
+
+			p16[x] = makecol16(r, g, b);
+		     }
+		  }
+	       }
+	       break;
+
+	 #endif
+
+	 #ifdef ALLEGRO_COLOR24
+
+	    case 24:
+	       /* load truecolor to a 24 bit truecolor bitmap */
+	       for (y=0; y<h; y++) {
+		  for (x=0; x<w; x++) {
+		     r = pack_getc(f);
+		     g = pack_getc(f);
+		     b = pack_getc(f);
+
+		     if (rgba)
+			pack_getc(f);
+
+		     bmp->line[y][x*3+_rgb_r_shift_24/8] = r;
+		     bmp->line[y][x*3+_rgb_g_shift_24/8] = g;
+		     bmp->line[y][x*3+_rgb_b_shift_24/8] = b;
+		  }
+	       }
+	       break;
+
+	 #endif
+
+	 #ifdef ALLEGRO_COLOR32
+
+	    case 32:
+	       /* load truecolor to a 32 bit truecolor bitmap */
+	       for (y=0; y<h; y++) {
+		  p32 = (unsigned long *)bmp->line[y];
+
+		  for (x=0; x<w; x++) {
+		     r = pack_getc(f);
+		     g = pack_getc(f);
+		     b = pack_getc(f);
+
+		     if (rgba)
+			a = pack_getc(f);
+		     else
+			a = 0;
+
+		     p32[x] = makeacol32(r, g, b, a);
+		  }
+	       }
+	       break;
+
+	 #endif
 
-	       p32[x] = makeacol32(r, g, b, a);
-	    }
 	 }
 	 break;
-
    }
 
-   if (bits != destbits) {
-      BITMAP *tmp = bmp;
-      bmp = create_bitmap_ex(destbits, w, h);
-      if (!bmp) {
+   _drawing_mode = prev_drawmode;
+
+   /* postprocess for paletted dithering */
+   if (dither8) {
+      BITMAP *tmp = create_bitmap_ex(8, bmp->w, bmp->h);
+      if (!tmp) {
+	 destroy_bitmap(bmp);
 	 *allegro_errno = ENOMEM;
 	 return NULL;
       }
-      blit(tmp, bmp, 0, 0, 0, 0, bmp->w, bmp->h);
-      destroy_bitmap(tmp);
+
+      blit(bmp, tmp, 0, 0, 0, 0, bmp->w, bmp->h);
+      destroy_bitmap(bmp);
+      bmp = tmp;
    }
 
    return bmp;
@@ -409,7 +600,7 @@ static FONT_MONO_DATA *read_font_mono(PACKFILE *f, int *hmax)
 /* read_font_color:
  *  Helper for read_font, below.
  */
-static FONT_COLOR_DATA *read_font_color(PACKFILE *pack, int *hmax, int depth)
+static FONT_COLOR_DATA *read_font_color(PACKFILE *pack, int *hmax)
 {
    FONT_COLOR_DATA *cf = NULL;
    int max = 0, i = 0;
@@ -432,12 +623,9 @@ static FONT_COLOR_DATA *read_font_color(PACKFILE *pack, int *hmax, int depth)
       *allegro_errno = ENOMEM;
       return NULL;
    }
-
+   
    for (i = 0; i < max; i++) {
-      /* Do not allow colour conversions for 8 bit bitmaps, but require it for
-       * all other colour depths.
-       */
-      bits[i] = read_bitmap(pack, depth, depth!=8);
+      bits[i] = read_bitmap(pack, 8, FALSE);
       if (!bits[i]) {
 	 while (i) {
 	    i--;
@@ -466,7 +654,6 @@ static FONT *read_font(PACKFILE *pack)
    FONT *f = NULL;
    int num_ranges = 0;
    int height = 0;
-   int depth;
 
    f = malloc(sizeof(FONT));
    if (!f) {
@@ -478,8 +665,7 @@ static FONT *read_font(PACKFILE *pack)
 
    num_ranges = pack_mgetw(pack);
    while (num_ranges--) {
-      depth = pack_getc(pack);
-      if (depth == 1 || depth == 255) {
+      if (pack_getc(pack)) {
 	 FONT_MONO_DATA *mf = 0, *iter = (FONT_MONO_DATA *)f->data;
 	 
 	 f->vtable = font_vtable_mono;
@@ -499,14 +685,10 @@ static FONT *read_font(PACKFILE *pack)
       } 
       else {
 	 FONT_COLOR_DATA *cf = NULL, *iter = (FONT_COLOR_DATA *)f->data;
-         
-         /* Older versions of Allegro use `0' to indicate a colour font */
-         if (depth == 0)
-            depth = 8;
 
 	 f->vtable = font_vtable_color;
 
-	 cf = read_font_color(pack, &height, depth);
+	 cf = read_font_color(pack, &height);
 	 if (!cf) {
 	    destroy_font(f);
 	    return NULL;
@@ -682,7 +864,7 @@ static RLE_SPRITE *read_rle_sprite(PACKFILE *f, int bits)
    unsigned int eol_marker;
    signed short s16;
    signed short *p16;
-   int32_t *p32;
+   signed long *p32;
 
    if (bits < 0) {
       bits = -bits;
@@ -756,13 +938,13 @@ static RLE_SPRITE *read_rle_sprite(PACKFILE *f, int bits)
       case 24:
       case 32:
 	 /* read truecolor data */
-	 p32 = (int32_t *)s->dat;
+	 p32 = (signed long *)s->dat;
 	 eol_marker = (bits == 24) ? MASK_COLOR_24 : MASK_COLOR_32;
 
 	 for (y=0; y<h; y++) {
 	    c = pack_igetl(f);
 
-	    while ((uint32_t)c != MASK_COLOR_32) {
+	    while ((unsigned long)c != MASK_COLOR_32) {
 	       if (c < 0) {
 		  /* skip count */
 		  *p32 = c;
@@ -873,7 +1055,7 @@ static DATAFILE *read_old_datafile(PACKFILE *f, void (*callback)(DATAFILE *))
    int c;
 
    size = pack_mgetw(f);
-   if (size == EOF) {
+   if (*allegro_errno) {
       pack_fclose(f);
       return NULL;
    }
@@ -892,8 +1074,6 @@ static DATAFILE *read_old_datafile(PACKFILE *f, void (*callback)(DATAFILE *))
       dat[c].prop = NULL;
    }
 
-   *allegro_errno = 0;
-
    for (c=0; c<size; c++) {
 
       type = pack_mgetw(f);
@@ -1149,143 +1329,19 @@ static void unload_midi(MIDI *m)
 
 
 /* load_object:
- *  Helper to load an object from a datafile and store it in 'obj'.
- *  Returns 0 on success and -1 on failure.
+ *  Helper to load an object from a datafile.
  */
-static int load_object(DATAFILE *obj, PACKFILE *f, int type)
+static void *load_object(PACKFILE *f, int type, long size)
 {
-   PACKFILE *ff;
-   int d, i;
+   int i;
 
-   /* load actual data */
-   ff = pack_fopen_chunk(f, FALSE);
+   /* look for a load function */
+   for (i=0; i<MAX_DATAFILE_TYPES; i++)
+      if (_datafile_type[i].type == type)
+	 return _datafile_type[i].load(f, size);
 
-   if (ff) {
-      d = ff->normal.todo;
-
-      /* look for a load function */
-      for (i=0; i<MAX_DATAFILE_TYPES; i++) {
-	 if (_datafile_type[i].type == type) {
-	    obj->dat = _datafile_type[i].load(ff, d);
-	    goto Found;
-	 }
-      }
-
-      /* if not found, load binary data */
-      obj->dat = load_data_object(ff, d);
-
-    Found:
-      pack_fclose_chunk(ff);
-
-      if (!obj->dat)
-	 return -1;
-
-      obj->type = type;
-      obj->size = d;
-      return 0;
-   }
-
-   return -1;
-}
-
-
-
-/* _load_property:
- *  Helper to load a property from a datafile and store it in 'prop'.
- *  Returns 0 on success and -1 on failure.
- */
-int _load_property(DATAFILE_PROPERTY *prop, PACKFILE *f)
-{
-   int type, size;
-   char *p;
-
-   type = pack_mgetl(f);
-   size = pack_mgetl(f);
-
-   prop->type = type;
-   prop->dat = malloc(size+1); /* '1' for end-of-string delimiter */
-   if (!prop->dat) {
-      *allegro_errno = ENOMEM;
-      pack_fseek(f, size);
-      return -1;
-   }
-
-   /* read the property */
-   pack_fread(prop->dat, size, f);
-   prop->dat[size] = 0;
-
-   /* convert to current encoding format if needed */
-   if (need_uconvert(prop->dat, U_UTF8, U_CURRENT)) {
-      int length = uconvert_size(prop->dat, U_UTF8, U_CURRENT);
-      p = malloc(length);
-      if (!p) {
-	 *allegro_errno = ENOMEM;
-	 return -1;
-      }
-
-      do_uconvert(prop->dat, U_UTF8, p, U_CURRENT, length);
-      free(prop->dat);
-      prop->dat = p;
-   }
-
-   return 0;
-}
-
-
-
-/* _add_property:
- *  Helper to add a new property to a property list. Returns 0 on
- *  success or -1 on failure.
- */
-int _add_property(DATAFILE_PROPERTY **list, DATAFILE_PROPERTY *prop)
-{
-   DATAFILE_PROPERTY *iter;
-   int length = 0;
-
-   ASSERT(list);
-   ASSERT(prop);
-
-   /* find the length of the list */
-   if (*list) {
-      iter = *list;
-      while (iter->type != DAT_END) {
-	 length++;
-	 iter++;
-      }
-   }
-
-   /* grow the list */
-   *list = _al_sane_realloc(*list, sizeof(DATAFILE_PROPERTY)*(length+2));
-   if (!(*list)) {
-      *allegro_errno = ENOMEM;
-      return -1;
-   }
-
-   /* copy the new property */
-   (*list)[length] = *prop;
-
-   /* set end-of-array marker */
-   (*list)[length+1].type = DAT_END;
-   (*list)[length+1].dat = NULL;
-
-   return 0;
-}
-
-
-
-/* _destroy_property_list:
- *  Helper to destroy a property list.
- */
-void _destroy_property_list(DATAFILE_PROPERTY *list)
-{
-   int c;
-
-   for (c=0; list[c].type != DAT_END; c++) {
-      if (list[c].dat)
-	 free(list[c].dat);
-   }
-
-   free(list);
+   /* if not found, load binary data */
+   return load_data_object(f, size);
 }
 
 
@@ -1295,9 +1351,13 @@ void _destroy_property_list(DATAFILE_PROPERTY *list)
  */
 static void *load_file_object(PACKFILE *f, long size)
 {
+   #define MAX_PROPERTIES  64
+
+   DATAFILE_PROPERTY prop[MAX_PROPERTIES];
    DATAFILE *dat;
-   DATAFILE_PROPERTY prop, *list;
-   int count, c, type, failed;
+   PACKFILE *ff;
+   int prop_count, count, type, c, d;
+   char *p;
 
    count = pack_mgetl(f);
 
@@ -1307,50 +1367,105 @@ static void *load_file_object(PACKFILE *f, long size)
       return NULL;
    }
 
-   list = NULL;
-   failed = FALSE;
+   for (c=0; c<=count; c++) {
+      dat[c].type = DAT_END;
+      dat[c].dat = NULL;
+      dat[c].size = 0;
+      dat[c].prop = NULL;
+   }
 
-   /* search the packfile for properties or objects */
-   for (c=0; c<count;) {
+   for (c=0; c<MAX_PROPERTIES; c++)
+      prop[c].dat = NULL;
+
+   c = 0;
+   prop_count = 0;
+
+   while (c < count) {
       type = pack_mgetl(f);
 
       if (type == DAT_PROPERTY) {
-	 if ((_load_property(&prop, f) != 0) || (_add_property(&list, &prop) != 0)) {
-	    failed = TRUE;
-	    break;
+	 /* load an object property */
+	 type = pack_mgetl(f);
+	 d = pack_mgetl(f);
+	 if (prop_count < MAX_PROPERTIES) {
+	    prop[prop_count].type = type;
+	    prop[prop_count].dat = malloc(d+1);
+	    if (prop[prop_count].dat != NULL) {
+	       pack_fread(prop[prop_count].dat, d, f);
+	       prop[prop_count].dat[d] = 0;
+	       if (need_uconvert(prop[prop_count].dat, U_UTF8, U_CURRENT)) {
+		  p = malloc(uconvert_size(prop[prop_count].dat, U_UTF8, U_CURRENT));
+		  if (p)
+		     do_uconvert(prop[prop_count].dat, U_UTF8, p, U_CURRENT, -1);
+		  free(prop[prop_count].dat);
+		  prop[prop_count].dat = p;
+	       }
+	       prop_count++;
+	       d = 0;
+	    }
 	 }
+	 while (d-- > 0)
+	    pack_getc(f);
       }
       else {
-	 if (load_object(&dat[c], f, type) != 0) {
-	    failed = TRUE;
-	    break;
+	 /* load actual data */
+	 ff = pack_fopen_chunk(f, FALSE);
+
+	 if (ff) {
+	    d = ff->todo;
+
+	    dat[c].dat = load_object(ff, type, d);
+
+	    if (dat[c].dat) {
+	       dat[c].type = type;
+	       dat[c].size = d;
+
+	       if (prop_count > 0) {
+		  dat[c].prop = malloc(sizeof(DATAFILE_PROPERTY)*(prop_count+1));
+		  if (dat[c].prop != NULL) {
+		     for (d=0; d<prop_count; d++) {
+			dat[c].prop[d].dat = prop[d].dat;
+			dat[c].prop[d].type = prop[d].type;
+			prop[d].dat = NULL;
+		     }
+		     dat[c].prop[d].dat = NULL;
+		     dat[c].prop[d].type = DAT_END;
+		  }
+		  else {
+		     for (d=0; d<prop_count; d++) {
+			free(prop[d].dat);
+			prop[d].dat = NULL;
+		     }
+		  }
+		  prop_count = 0;
+	       }
+	       else
+		  dat[c].prop = NULL;
+	    }
+
+	    f = pack_fclose_chunk(ff);
+
+	    if (datafile_callback)
+	       datafile_callback(dat+c);
+
+	    c++;
 	 }
+      }
 
-	 /* attach the property list to the object */
-	 dat[c].prop = list;
-	 list = NULL;
+      if (*allegro_errno) {
+	 unload_datafile(dat);
 
-	 if (datafile_callback)
-	    datafile_callback(dat+c);
+	 for (c=0; c<MAX_PROPERTIES; c++)
+	    if (prop[c].dat)
+	       free(prop[c].dat);
 
-	 c++;
+	 return NULL;
       }
    }
 
-   /* set end-of-array marker */
-   dat[c].type = DAT_END;
-   dat[c].dat = NULL;
-
-   /* destroy the property list if not assigned to an object */
-   if (list)
-      _destroy_property_list(list);
-
-   /* gracefully handle failure */
-   if (failed) {
-      unload_datafile(dat);
-      free(dat);
-      dat = NULL;
-   }
+   for (c=0; c<MAX_PROPERTIES; c++)
+      if (prop[c].dat)
+	 free(prop[c].dat);
 
    return dat;
 }
@@ -1363,7 +1478,6 @@ static void *load_file_object(PACKFILE *f, long size)
  */
 DATAFILE *load_datafile(AL_CONST char *filename)
 {
-   ASSERT(filename);
    return load_datafile_callback(filename, NULL);
 }
 
@@ -1378,13 +1492,12 @@ DATAFILE *load_datafile_callback(AL_CONST char *filename, void (*callback)(DATAF
    PACKFILE *f;
    DATAFILE *dat;
    int type;
-   ASSERT(filename);
 
    f = pack_fopen(filename, F_READ_PACKED);
    if (!f)
       return NULL;
 
-   if ((f->normal.flags & PACKFILE_FLAG_CHUNK) && (!(f->normal.flags & PACKFILE_FLAG_EXEDAT)))
+   if ((f->flags & PACKFILE_FLAG_CHUNK) && (!(f->flags & PACKFILE_FLAG_EXEDAT)))
       type = (_packfile_type == DAT_FILE) ? DAT_MAGIC : 0;
    else
       type = pack_mgetl(f);
@@ -1413,115 +1526,44 @@ DATAFILE *load_datafile_object(AL_CONST char *filename, AL_CONST char *objectnam
 {
    PACKFILE *f;
    DATAFILE *dat;
-   DATAFILE_PROPERTY prop, *list;
-   char parent[1024], child[1024], tmp[8];
-   char *bufptr, *prevptr, *separator;
-   int count, c, type, size, found;
+   void *object;
+   char buf[1024], tmp[16];
+   int size;
 
-   ASSERT(filename);
-   ASSERT(objectname);
-  
-   /* concatenate to filename#objectname */
-   ustrzcpy(parent, sizeof(parent), filename);
+   ustrzcpy(buf, sizeof(buf), filename);
 
-   if (ustrcmp(parent, uconvert_ascii("#", tmp)) != 0)
-      ustrzcat(parent, sizeof(parent), uconvert_ascii("#", tmp));
+   if (ustrcmp(buf, uconvert_ascii("#", tmp)) != 0)
+      ustrzcat(buf, sizeof(buf), uconvert_ascii("#", tmp));
 
-   ustrzcat(parent, sizeof(parent), objectname);
-   
-   /* split into path and actual objectname (for nested files) */
-   prevptr = bufptr = parent;
-   separator = NULL;
-   while ((c = ugetx(&bufptr)) != 0) {
-      if ((c == '#') || (c == '/') || (c == OTHER_PATH_SEPARATOR))
-	 separator = prevptr;
+   ustrzcat(buf, sizeof(buf), objectname);
 
-      prevptr = bufptr;
-   }
-   
-   ustrzcpy(child, sizeof(child), separator + uwidth (separator));
-
-   if (separator == parent)
-      usetc(separator + uwidth (separator), 0);
-   else
-      usetc(separator, 0);
-
-   /* open the parent datafile */
-   f = pack_fopen(parent, F_READ_PACKED);
+   f = pack_fopen(buf, F_READ_PACKED);
    if (!f)
       return NULL;
 
-   if ((f->normal.flags & PACKFILE_FLAG_CHUNK) && (!(f->normal.flags & PACKFILE_FLAG_EXEDAT)))
-      type = (_packfile_type == DAT_FILE) ? DAT_MAGIC : 0;
-   else
-      type = pack_mgetl(f);
+   size = f->todo;
 
-   /* only support V2 datafile format */
-   if (type != DAT_MAGIC) {
+   dat = malloc(sizeof(DATAFILE));
+   if (!dat) {
       pack_fclose(f);
       return NULL;
    }
 
-   count = pack_mgetl(f);
-
-   dat = NULL;
-   list = NULL;
-   found = FALSE;
-
-   /* search the packfile for properties or objects */
-   for (c=0; c<count;) {
-      type = pack_mgetl(f);
-
-      if (type == DAT_PROPERTY) {
-	 if ((_load_property(&prop, f) != 0) || (_add_property(&list, &prop) != 0))
-	    break;
-
-	 if ((prop.type == DAT_NAME) && (ustricmp(prop.dat, child) == 0))
-	    found = TRUE;
-      }
-      else {
-	 if (found) {
-	    /* we have found the object, load it */
-	    dat = malloc(sizeof(DATAFILE));
-	    if (!dat) {
-	       *allegro_errno = ENOMEM;
-	       break;
-	    }
-
-	    if (load_object(dat, f, type) != 0) {
-	       free(dat);
-	       dat = NULL;
-	       break;
-	    }
-
-	    /* attach the property list to the object */
-	    dat->prop = list;
-	    list = NULL;
-
-	    break;
-	 }
-	 else {
-	    /* skip an unwanted object */
-	    size = pack_mgetl(f);
-	    pack_fseek(f, size+4);  /* '4' for the chunk size */
-
-	    /* destroy the property list */
-	    if (list) {
-	       _destroy_property_list(list);
-	       list = NULL;
-	    }
-
-	    c++;
-	 }
-      }
-   }
-
-   /* destroy the property list if not assigned to an object */
-   if (list)
-      _destroy_property_list(list);
+   object = load_object(f, _packfile_type, size);
 
    pack_fclose(f);
-   return dat;
+
+   if (!object) {
+      free(dat);
+      return NULL;
+   }
+
+   dat->dat = object;
+   dat->type = _packfile_type;
+   dat->size = size;
+   dat->prop = NULL;
+
+   return dat; 
 }
 
 
@@ -1533,9 +1575,14 @@ void _unload_datafile_object(DATAFILE *dat)
 {
    int i;
 
-   /* destroy the property list */
-   if (dat->prop)
-      _destroy_property_list(dat->prop);
+   /* free the property list */
+   if (dat->prop) {
+      for (i=0; dat->prop[i].type != DAT_END; i++)
+	 if (dat->prop[i].dat)
+	    free(dat->prop[i].dat);
+
+      free(dat->prop);
+   }
 
    /* look for a destructor function */
    for (i=0; i<MAX_DATAFILE_TYPES; i++) {
@@ -1595,8 +1642,6 @@ DATAFILE *find_datafile_object(AL_CONST DATAFILE *dat, AL_CONST char *objectname
    char name[512];
    int recurse = FALSE;
    int pos, c;
-   ASSERT(dat);
-   ASSERT(objectname);
 
    /* split up the object name */
    pos = 0;
@@ -1637,10 +1682,8 @@ DATAFILE *find_datafile_object(AL_CONST DATAFILE *dat, AL_CONST char *objectname
  */
 AL_CONST char *get_datafile_property(AL_CONST DATAFILE *dat, int type)
 {
-   DATAFILE_PROPERTY *prop;
-   ASSERT(dat);
+   DATAFILE_PROPERTY *prop = dat->prop;
 
-   prop = dat->prop;
    if (prop) {
       while (prop->type != DAT_END) {
 	 if (prop->type == type)
@@ -1656,9 +1699,8 @@ AL_CONST char *get_datafile_property(AL_CONST DATAFILE *dat, int type)
 
 
 /* fixup_datafile:
- *  For use with compiled datafiles, to initialise them if they are not
- *  constructed and to convert truecolor images into the appropriate
- *  pixel format.
+ *  For use with compiled datafiles, to convert truecolor images into the
+ *  appropriate pixel format.
  */
 void fixup_datafile(DATAFILE *data)
 {
@@ -1666,17 +1708,13 @@ void fixup_datafile(DATAFILE *data)
    RLE_SPRITE *rle;
    int i, c, r, g, b, a, x, y;
    int bpp, depth;
-   unsigned char *p8;
    unsigned short *p16;
-   uint32_t *p32;
+   unsigned long *p32;
    signed short *s16;
-   int32_t *s32;
+   signed long *s32;
    int eol_marker;
-   ASSERT(data);
 
-   /* initialise the datafile if needed */
-   if (!constructed_datafiles)
-      initialise_datafile(data);
+   _construct_datafile(data);
 
    for (i=0; data[i].type != DAT_END; i++) {
 
@@ -1743,8 +1781,6 @@ void fixup_datafile(DATAFILE *data)
 			r = _rgb_scale_5[c & 0x1F];
 			g = _rgb_scale_6[(c >> 5) & 0x3F];
 			b = _rgb_scale_5[(c >> 11) & 0x1F];
-			if (_color_conv & COLORCONV_KEEP_TRANS && depth == 15 &&
-			   c == 0xf83f) g = 8; /* don't end up as mask color */
 			p16[x] = makecol_depth(depth, r, g, b);
 		     }
 		  }
@@ -1758,14 +1794,13 @@ void fixup_datafile(DATAFILE *data)
 	       case 24:
 		  /* fix up a 24 bit truecolor bitmap */
 		  for (y=0; y<bmp->h; y++) {
-		     p8 = bmp->line[y];
-
 		     for (x=0; x<bmp->w; x++) {
-			c = READ3BYTES(p8+x*3);
-			r = (c & 0xFF);
-			g = (c >> 8) & 0xFF;
-			b = (c >> 16) & 0xFF;
-			WRITE3BYTES(p8+x*3, makecol24(r, g, b));
+			r = bmp->line[y][x*3];
+			g = bmp->line[y][x*3+1];
+			b = bmp->line[y][x*3+2];
+			bmp->line[y][x*3+_rgb_r_shift_24/8] = r;
+			bmp->line[y][x*3+_rgb_r_shift_24/8] = g;
+			bmp->line[y][x*3+_rgb_r_shift_24/8] = b;
 		     }
 		  }
 		  break;
@@ -1778,7 +1813,7 @@ void fixup_datafile(DATAFILE *data)
 	       case 32:
 		  /* fix up a 32 bit truecolor bitmap */
 		  for (y=0; y<bmp->h; y++) {
-		     p32 = (uint32_t *)bmp->line[y];
+		     p32 = (unsigned long *)bmp->line[y];
 
 		     for (x=0; x<bmp->w; x++) {
 			c = p32[x];
@@ -1896,10 +1931,10 @@ void fixup_datafile(DATAFILE *data)
 		     eol_marker = MASK_COLOR_24;
 		  }
 
-		  s32 = (int32_t *)rle->dat;
+		  s32 = (signed long *)rle->dat;
 
 		  for (y=0; y<rle->h; y++) {
-		     while ((uint32_t)*s32 != MASK_COLOR_24) {
+		     while ((unsigned long)*s32 != MASK_COLOR_24) {
 			if (*s32 > 0) {
 			   x = *s32;
 			   s32++;
@@ -1937,10 +1972,10 @@ void fixup_datafile(DATAFILE *data)
 		     eol_marker = MASK_COLOR_32;
 		  }
 
-		  s32 = (int32_t *)rle->dat;
+		  s32 = (signed long *)rle->dat;
 
 		  for (y=0; y<rle->h; y++) {
-		     while ((uint32_t)*s32 != MASK_COLOR_32) {
+		     while ((unsigned long)*s32 != MASK_COLOR_32) {
 			if (*s32 > 0) {
 			   x = *s32;
 			   s32++;
@@ -1976,36 +2011,15 @@ void fixup_datafile(DATAFILE *data)
 
 
 
-/* initialise_bitmap:
- *  Helper used by the output from dat2s/c, for fixing up parts
- *  of a BITMAP structure that can't be statically initialised.
+/* _construct_datafile:
+ *  Helper used by the output from dat2s, for fixing up parts of
+ *  the data that can't be statically initialised, such as locking
+ *  samples and MIDI files, and setting the segment selectors in the 
+ *  BITMAP structures.
  */
-static void initialise_bitmap(BITMAP *bmp)
+void _construct_datafile(DATAFILE *data)
 {
-   int i;
-
-   for (i=0; _vtable_list[i].vtable; i++) {
-      if (_vtable_list[i].color_depth == (int)(uintptr_t)bmp->vtable) {
-	 bmp->vtable = _vtable_list[i].vtable;
-	 bmp->write_bank = _stub_bank_switch;
-	 bmp->read_bank = _stub_bank_switch;
-	 bmp->seg = _default_ds();
-	 return;
-      }
-   }
-
-   ASSERT(FALSE); 
-}
-
-
-
-/* initialise_datafile:
- *  Helper used by the output from dat2s/c, for fixing up parts
- *  of the data that can't be statically initialised.
- */
-static void initialise_datafile(DATAFILE *data)
-{
-   int c, c2, color_flag;
+   int c, c2;
    FONT *f;
    SAMPLE *s;
    MIDI *m;
@@ -2015,27 +2029,22 @@ static void initialise_datafile(DATAFILE *data)
       switch (data[c].type) {
 
 	 case DAT_FILE:
-	    initialise_datafile(data[c].dat);
+	    _construct_datafile(data[c].dat);
 	    break;
 
 	 case DAT_BITMAP:
-	    initialise_bitmap((BITMAP *)data[c].dat);
+	    ((BITMAP *)data[c].dat)->seg = _default_ds();
 	    break;
 
 	 case DAT_FONT:
 	    f = data[c].dat;
-	    color_flag = (int)(uintptr_t)f->vtable;
-	    if (color_flag == 1) {
+	    if (f->vtable == font_vtable_color) {
 	       FONT_COLOR_DATA *cf = (FONT_COLOR_DATA *)f->data;
 	       while (cf) {
 		  for (c2 = cf->begin; c2 < cf->end; c2++)
-		     initialise_bitmap((BITMAP *)cf->bitmaps[c2 - cf->begin]);
+		     cf->bitmaps[c2 - cf->begin]->seg = _default_ds();
 		  cf = cf->next;
 	       }
-	       f->vtable = font_vtable_color;
-	    }
-	    else {
-	       f->vtable = font_vtable_mono;
 	    }
 	    break;
 
@@ -2060,27 +2069,14 @@ static void initialise_datafile(DATAFILE *data)
 
 
 
-/* _construct_datafile:
- *  Constructor called by the output from dat2s/c.
- */
-void _construct_datafile(DATAFILE *data)
-{
-   /* record that datafiles are constructed */
-   constructed_datafiles = TRUE;
-
-   initialise_datafile(data);
-}
-
-
-
 /* _initialize_datafile_types:
  *  Register my loader functions with the code in dataregi.c.
  */
-#ifdef ALLEGRO_USE_CONSTRUCTOR
-   CONSTRUCTOR_FUNCTION(void _initialize_datafile_types(void));
+#ifdef CONSTRUCTOR_FUNCTION
+   CONSTRUCTOR_FUNCTION(void _initialize_datafile_types());
 #endif
 
-void _initialize_datafile_types(void)
+void _initialize_datafile_types()
 {
    register_datafile_object(DAT_FILE,         load_file_object,             (void (*)(void *data))unload_datafile        );
    register_datafile_object(DAT_FONT,         load_font_object,             (void (*)(void *data))destroy_font           );
