@@ -97,7 +97,6 @@ static void sys_dos_save_console_state(void);
 static void sys_dos_restore_console_state(void);
 static void sys_dos_read_palette(void);
 static void sys_dos_set_palette(AL_CONST struct RGB *p, int from, int to, int vsync);
-static void sys_dos_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode);
 static void sys_dos_yield_timeslice(void);
 
 #ifdef ALLEGRO_DJGPP
@@ -110,45 +109,23 @@ static void sys_dos_yield_timeslice(void);
 /* the main system driver for running under dos */
 SYSTEM_DRIVER system_dos =
 {
-   SYSTEM_DOS,   /* id */
-   empty_string, /* name */
-   empty_string, /* desc */
-   "DOS",        /* ascii_name */
+   SYSTEM_DOS,
+   empty_string,
+   empty_string,
+   "DOS",
    sys_dos_init,
    sys_dos_exit,
    sys_dos_get_executable_name,
-   NULL, /* find_resource */
-   NULL, /* set_window_title */
-   NULL, /* set_close_button_callback */
-   NULL, /* message */
+   NULL, NULL, NULL, NULL, NULL,
    sys_dos_assert,
    sys_dos_save_console_state,
    sys_dos_restore_console_state,
-   NULL, /* create_bitmap */
-   NULL, /* created_bitmap */
-   NULL, /* create_sub_bitmap */
-   NULL, /* created_sub_bitmap */
-   NULL, /* destroy_bitmap */
+   NULL, NULL, NULL, NULL, NULL,
    sys_dos_read_palette,
    sys_dos_set_palette,
-   NULL, /* get_vtable */
-   NULL, /* set_display_switch_mode */
-   NULL, /* display_switch_lock */
-   NULL, /* desktop_color_depth */
-   NULL, /* get_desktop_resolution */
-   sys_dos_get_gfx_safe_mode,
+   NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    sys_dos_yield_timeslice,
-   NULL, /* create_mutex */
-   NULL, /* destroy_mutex */
-   NULL, /* lock_mutex */
-   NULL, /* unlock_mutex */
-   NULL, /* gfx_drivers */
-   NULL, /* digi_drivers */
-   NULL, /* midi_drivers */
-   NULL, /* keyboard_drivers */
-   NULL, /* mouse_drivers */
-   NULL, /* joystick_drivers */
-   NULL  /* timer_drivers */
+   NULL, NULL, NULL, NULL, NULL, NULL, NULL
 };
 
 
@@ -182,7 +159,7 @@ static void signal_handler(int num)
 /* detect_os:
  *  Operating system autodetection routine.
  */
-static void detect_os(void)
+static void detect_os()
 {
    __dpmi_regs r;
    char buf[16];
@@ -544,19 +521,6 @@ static void sys_dos_set_palette(AL_CONST struct RGB *p, int from, int to, int vs
 
 
 
-/* sys_dos_get_gfx_safe_mode:
- *  Defines the safe graphics mode for this system.
- */
-static void sys_dos_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode)
-{
-   *driver = GFX_VGA;
-   mode->width = 320;
-   mode->height = 200;
-   mode->bpp = 8;
-}
-
-
-
 /* _set_vga_mode:
  *  Helper for the VGA and mode-X drivers to set a video mode.
  */
