@@ -41,7 +41,7 @@ static void vesa_vsync(void);
 static void vesa_set_palette_range(AL_CONST PALETTE p, int from, int to, int vsync);
 static int vesa_request_scroll(int x, int y);
 static int vesa_poll_scroll(void);
-static GFX_MODE_LIST *vesa_fetch_mode_list(void);
+static GFX_MODE_LIST *vesa_fetch_mode_list();
 
 static char vesa_desc[256] = EMPTY_STRING;
 
@@ -64,7 +64,6 @@ GFX_DRIVER gfx_vesa_1 =
    NULL, NULL, NULL, NULL,       /* no hardware cursor */
    NULL,                         /* no drawing mode hook */
    NULL, NULL,                   /* no state saving */
-   NULL,    /* AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a)); */
    vesa_fetch_mode_list,         /* aye! */
    0, 0, FALSE, 0, 0, 0, 0, FALSE
 };
@@ -88,7 +87,6 @@ GFX_DRIVER gfx_vesa_2b =
    NULL, NULL, NULL, NULL,       /* no hardware cursor */
    NULL,                         /* no drawing mode hook */
    NULL, NULL,                   /* no state saving */
-   NULL,
    vesa_fetch_mode_list,         /* aye! */
    0, 0, FALSE, 0, 0, 0, 0, FALSE
 };
@@ -112,7 +110,6 @@ GFX_DRIVER gfx_vesa_2l =
    NULL, NULL, NULL, NULL,       /* no hardware cursor */
    NULL,                         /* no drawing mode hook */
    NULL, NULL,                   /* no state saving */
-   NULL,
    vesa_fetch_mode_list,         /* aye! */
    0, 0, FALSE, 0, 0, 0, 0, FALSE
 };
@@ -138,7 +135,6 @@ GFX_DRIVER gfx_vesa_3 =
    NULL, NULL, NULL, NULL,       /* no hardware cursor */
    NULL,                         /* no drawing mode hook */
    NULL, NULL,                   /* no state saving */
-   NULL,
    vesa_fetch_mode_list,         /* aye! */
    0, 0, FALSE, 0, 0, 0, 0, FALSE
 };
@@ -166,7 +162,7 @@ GFX_DRIVER gfx_vesa_3 =
 
 typedef struct VESA_INFO         /* VESA information block structure */
 { 
-   char           VESASignature[4]     __PACKED__;
+   unsigned char  VESASignature[4]     __PACKED__;
    unsigned short VESAVersion          __PACKED__;
    unsigned long  OEMStringPtr         __PACKED__;
    unsigned char  Capabilities[4]      __PACKED__;
@@ -304,7 +300,7 @@ static int evilness_flag = 0;             /* set if we are doing dodgy things
 /* get_vesa_info:
  *  Retrieves a VESA info block structure, returning 0 for success.
  */
-static int get_vesa_info(void)
+static int get_vesa_info()
 {
    unsigned long addr;
    int c;
@@ -608,14 +604,14 @@ static void setup_vesa_desc(GFX_DRIVER *driver, int vbe_version, int linear)
  *  Generates a list of valid video modes for the VESA drivers.
  *  Returns the mode list on success or NULL on failure.
  */
-static GFX_MODE_LIST *vesa_fetch_mode_list(void)
+static GFX_MODE_LIST *vesa_fetch_mode_list()
 {
    GFX_MODE_LIST  *mode_list;
    unsigned long  mode_ptr;
    unsigned short *vesa_mode;
    int            mode, pos, vesa_list_length;
 
-   mode_list = _AL_MALLOC(sizeof(GFX_MODE_LIST));
+   mode_list = malloc(sizeof(GFX_MODE_LIST));
    if (!mode_list) return NULL;
 
    /* fetch list of VESA modes */
@@ -629,7 +625,7 @@ static GFX_MODE_LIST *vesa_fetch_mode_list(void)
       mode_ptr += sizeof(unsigned short);
 
    /* allocate and fill in temporary vesa mode list */
-   vesa_mode = _AL_MALLOC(sizeof(unsigned short) * mode_list->num_modes);
+   vesa_mode = malloc(sizeof(unsigned short) * mode_list->num_modes);
    if (!vesa_mode) return NULL;
 
    mode_ptr = RM_TO_LINEAR(vesa_info.VideoModePtr);
@@ -650,7 +646,7 @@ static GFX_MODE_LIST *vesa_fetch_mode_list(void)
    }
 
    /* allocate mode list */
-   mode_list->mode = _AL_MALLOC(sizeof(GFX_MODE) * (mode_list->num_modes + 1));
+   mode_list->mode = malloc(sizeof(GFX_MODE) * (mode_list->num_modes + 1));
    if (!mode_list->mode) return NULL;
 
    /* fill in width, height and color depth for each VESA mode */
@@ -670,7 +666,7 @@ static GFX_MODE_LIST *vesa_fetch_mode_list(void)
    mode_list->mode[mode_list->num_modes].bpp    = 0;
 
    /* free up temporary vesa mode list */
-   _AL_FREE(vesa_mode);
+   free(vesa_mode);
 
    return mode_list;
 }
@@ -696,9 +692,9 @@ static int get_pmode_functions(void (**w1)(void), void (**w2)(void))
       return -1;
 
    if (pm_info)
-      _AL_FREE(pm_info);
+      free(pm_info);
 
-   pm_info = _AL_MALLOC(_dpmi_reg.x.cx);         /* copy into our address space */
+   pm_info = malloc(_dpmi_reg.x.cx);         /* copy into our address space */
    LOCK_DATA(pm_info, _dpmi_reg.x.cx);
    dosmemget((_dpmi_reg.x.es*16)+_dpmi_reg.x.di, _dpmi_reg.x.cx, pm_info);
 
@@ -1008,7 +1004,7 @@ static BITMAP *vesa_init(GFX_DRIVER *driver, int linear, int vbe_version, int w,
       if (vclk != 0) {
 	 f0 = (double)vclk / (crtc.HorizontalTotal * crtc.VerticalTotal);
 
-	 _set_current_refresh_rate((int)(f0 + 0.5));
+	 _current_refresh_rate = (int)(f0 + 0.5);
 
 	 crtc.PixelClock  = vclk;
 	 crtc.RefreshRate = _refresh_rate_request * 100;
@@ -1238,7 +1234,7 @@ static BITMAP *vesa_3_init(int w, int h, int v_w, int v_h, int color_depth)
  *  blanking registers. VBE doesn't provide a vsync function, but we 
  *  can emulate it by changing the display start with the vsync flag set.
  */
-static void vesa_vsync(void)
+static void vesa_vsync()
 {
    vesa_scroll(vesa_xscroll, vesa_yscroll);
 }
@@ -1423,7 +1419,7 @@ static int vesa_request_scroll(int x, int y)
 /* vesa_poll_scroll:
  *  VBE 3.0 triple buffering test routine.
  */
-static int vesa_poll_scroll(void)
+static int vesa_poll_scroll()
 {
    _dpmi_reg.x.ax = 0x4F07;
    _dpmi_reg.x.bx = 4;
@@ -1456,7 +1452,7 @@ static void vesa_exit(BITMAP *b)
    _remove_physical_mapping(&mmio_linear, &_mmio_segment);
 
    if (pm_info) {
-      _AL_FREE(pm_info);
+      free(pm_info);
       pm_info = NULL;
    }
 
