@@ -21,14 +21,6 @@
 #include "allegro.h"
 #include "allegro/internal/aintern.h"
 
-extern void blit_end(void);   /* for LOCK_FUNCTION; defined in blit.c */
-
-
-
-#define PREFIX_I                "al-gfx INFO: "
-#define PREFIX_W                "al-gfx WARNING: "
-#define PREFIX_E                "al-gfx ERROR: "
-
 
 
 static int gfx_virgin = TRUE;          /* is the graphics system active? */
@@ -48,9 +40,7 @@ COLOR_MAP *color_map = NULL;           /* translucency/lighting table */
 int _color_depth = 8;                  /* how many bits per pixel? */
 
 int _refresh_rate_request = 0;         /* requested refresh rate */
-static int current_refresh_rate = 0;   /* refresh rate set by the driver */
-
-int _wait_for_vsync = TRUE;            /* vsync when page-flipping? */
+int _current_refresh_rate = 0;         /* refresh rate set by the driver */
 
 int _color_conv = COLORCONV_TOTAL;     /* which formats to auto convert? */
 
@@ -80,28 +70,28 @@ int _blender_col_32 = 0;
 
 int _blender_alpha = 0;                /* for truecolor translucent drawing */
 
-int _rgb_r_shift_15 = DEFAULT_RGB_R_SHIFT_15;     /* truecolor pixel format */
-int _rgb_g_shift_15 = DEFAULT_RGB_G_SHIFT_15;
-int _rgb_b_shift_15 = DEFAULT_RGB_B_SHIFT_15;
-int _rgb_r_shift_16 = DEFAULT_RGB_R_SHIFT_16;
-int _rgb_g_shift_16 = DEFAULT_RGB_G_SHIFT_16;
-int _rgb_b_shift_16 = DEFAULT_RGB_B_SHIFT_16;
-int _rgb_r_shift_24 = DEFAULT_RGB_R_SHIFT_24;
-int _rgb_g_shift_24 = DEFAULT_RGB_G_SHIFT_24;
-int _rgb_b_shift_24 = DEFAULT_RGB_B_SHIFT_24;
-int _rgb_r_shift_32 = DEFAULT_RGB_R_SHIFT_32;
-int _rgb_g_shift_32 = DEFAULT_RGB_G_SHIFT_32;
-int _rgb_b_shift_32 = DEFAULT_RGB_B_SHIFT_32;
-int _rgb_a_shift_32 = DEFAULT_RGB_A_SHIFT_32;
+int _rgb_r_shift_15 = 0;               /* truecolor pixel format */
+int _rgb_g_shift_15 = 5;
+int _rgb_b_shift_15 = 10;
+int _rgb_r_shift_16 = 0;
+int _rgb_g_shift_16 = 5;
+int _rgb_b_shift_16 = 11;
+int _rgb_r_shift_24 = 0;
+int _rgb_g_shift_24 = 8;
+int _rgb_b_shift_24 = 16;
+int _rgb_r_shift_32 = 0;
+int _rgb_g_shift_32 = 8;
+int _rgb_b_shift_32 = 16;
+int _rgb_a_shift_32 = 24;
 
 
 /* lookup table for scaling 5 bit colors up to 8 bits */
 int _rgb_scale_5[32] =
 {
-   0,   8,   16,  24,  33,  41,  49,  57,
-   66,  74,  82,  90,  99,  107, 115, 123,
-   132, 140, 148, 156, 165, 173, 181, 189,
-   198, 206, 214, 222, 231, 239, 247, 255
+   0,   8,   16,  24,  32,  41,  49,  57,
+   65,  74,  82,  90,  98,  106, 115, 123,
+   131, 139, 148, 156, 164, 172, 180, 189,
+   197, 205, 213, 222, 230, 238, 246, 255
 };
 
 
@@ -110,12 +100,12 @@ int _rgb_scale_6[64] =
 {
    0,   4,   8,   12,  16,  20,  24,  28,
    32,  36,  40,  44,  48,  52,  56,  60,
-   65,  69,  73,  77,  81,  85,  89,  93,
+   64,  68,  72,  76,  80,  85,  89,  93,
    97,  101, 105, 109, 113, 117, 121, 125,
-   130, 134, 138, 142, 146, 150, 154, 158,
-   162, 166, 170, 174, 178, 182, 186, 190,
-   195, 199, 203, 207, 211, 215, 219, 223,
-   227, 231, 235, 239, 243, 247, 251, 255
+   129, 133, 137, 141, 145, 149, 153, 157,
+   161, 165, 170, 174, 178, 182, 186, 190,
+   194, 198, 202, 206, 210, 214, 218, 222,
+   226, 230, 234, 238, 242, 246, 250, 255
 };
 
 
@@ -126,25 +116,13 @@ typedef struct VRAM_BITMAP             /* list of video memory bitmaps */
 {
    int x, y, w, h;
    BITMAP *bmp;
-   struct VRAM_BITMAP *next_x, *next_y;
+   struct VRAM_BITMAP *next;
 } VRAM_BITMAP;
 
 
 static VRAM_BITMAP *vram_bitmap_list = NULL;
 
 
-#define BMP_MAX_SIZE  46340   /* sqrt(INT_MAX) */
-
-/* the product of these must fit in an int */
-static int failed_bitmap_w = BMP_MAX_SIZE;
-static int failed_bitmap_h = BMP_MAX_SIZE;
-
-
-
-static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_config);
-static int _set_gfx_mode_safe(int card, int w, int h, int v_w, int v_h);
-
-
 
 /* lock_bitmap:
  *  Locks all the memory used by a bitmap structure.
@@ -166,7 +144,7 @@ void lock_bitmap(BITMAP *bmp)
  */
 void request_refresh_rate(int rate)
 {
-   _refresh_rate_request = rate;
+    _refresh_rate_request = rate;
 }
 
 
@@ -176,25 +154,7 @@ void request_refresh_rate(int rate)
  */
 int get_refresh_rate(void)
 {
-   return current_refresh_rate;
-}
-
-
-
-/* _set_current_refresh_rate:
- *  Sets the current refresh rate.
- *  (This function must be called by the gfx drivers)
- */
-void _set_current_refresh_rate(int rate)
-{
-   /* sanity check to discard bogus values */
-   if ((rate<40) || (rate>200))
-      rate = 0;
-
-   current_refresh_rate = rate;
-
-   /* adjust retrace speed */
-   _vsync_speed = rate ? BPS_TO_TIMER(rate) : BPS_TO_TIMER(70);
+    return _current_refresh_rate;
 }
 
 
@@ -242,9 +202,7 @@ GFX_MODE_LIST *get_gfx_mode_list(int card)
    _DRIVER_INFO *list_entry;
    GFX_DRIVER *drv = NULL;
    GFX_MODE_LIST *gfx_mode_list = NULL;
-
-   ASSERT(system_driver);
-
+   
    /* ask the system driver for a list of graphics hardware drivers */
    if (system_driver->gfx_drivers)
       list_entry = system_driver->gfx_drivers();
@@ -287,9 +245,9 @@ void destroy_gfx_mode_list(GFX_MODE_LIST *gfx_mode_list)
 {
    if (gfx_mode_list) {
       if (gfx_mode_list->mode)
-         _AL_FREE(gfx_mode_list->mode);
+         free(gfx_mode_list->mode);
 
-      _AL_FREE(gfx_mode_list);
+      free(gfx_mode_list);
    }
 }
 
@@ -315,16 +273,6 @@ void set_color_depth(int depth)
 
 
 
-/* get_color_depth:
- *  Returns the current color depth.
- */
-int get_color_depth(void)
-{
-   return _color_depth;
-}
-
-
-
 /* set_color_conversion:
  *  Sets a bit mask specifying which types of color format conversions are
  *  valid when loading data from disk.
@@ -338,17 +286,6 @@ void set_color_conversion(int mode)
 
 
 
-/* get_color_conversion:
- *  Returns the bitmask specifying which types of color format
- *  conversion are valid when loading data from disk.
- */
-int get_color_conversion(void)
-{
-   return _color_conv;
-}
-
-
-
 /* _color_load_depth:
  *  Works out which color depth an image should be loaded as, given the
  *  current conversion mode.
@@ -467,71 +404,16 @@ static void shutdown_gfx(void)
 }
 
 
-#define GFX_DRIVER_FULLSCREEN_FLAG    0x01
-#define GFX_DRIVER_WINDOWED_FLAG      0x02
-
-
-/* gfx_driver_is_valid:
- *  Checks that the graphics driver 'drv' fulfills the condition
- *  expressed by the bitmask 'flags'.
- */
-static int gfx_driver_is_valid(GFX_DRIVER *drv, int flags)
-{
-   if ((flags & GFX_DRIVER_FULLSCREEN_FLAG) && drv->windowed)
-      return FALSE;
-
-   if ((flags & GFX_DRIVER_WINDOWED_FLAG) && !drv->windowed)
-      return FALSE;
-
-   return TRUE;
-}
-
-
-
-/* get_gfx_driver_from_id:
- *  Retrieves a pointer to the graphics driver identified by 'card' from
- *  the list 'driver_list' or NULL if it doesn't exist.
- */
-static GFX_DRIVER *get_gfx_driver_from_id(int card, _DRIVER_INFO *driver_list)
-{
-   int c;
-
-   for (c=0; driver_list[c].driver; c++) {
-      if (driver_list[c].id == card)
-	 return driver_list[c].driver;
-   }
-
-   return NULL;
-}
-
-
-
-/* init_gfx_driver:
- *  Helper function for initializing a graphics driver.
- */
-static BITMAP *init_gfx_driver(GFX_DRIVER *drv, int w, int h, int v_w, int v_h)
-{
-   drv->name = drv->desc = get_config_text(drv->ascii_name);
-
-   /* set gfx_driver so that it is visible when initializing the driver */
-   gfx_driver = drv;
-
-   return drv->init(w, h, v_w, v_h, _color_depth);
-}
-
-
 
 /* get_config_gfx_driver:
- *  Helper function for set_gfx_mode: it reads the gfx_card* config variables and
- *  tries to set the graphics mode if a matching driver is found. Returns TRUE if
- *  at least one matching driver was found, FALSE otherwise.
+ *  Helper function for set_gfx_mode: it reads the gfx_card* config variables
+ *  and sets the gfx_driver variable if the config variable was found
  */
-static int get_config_gfx_driver(char *gfx_card, int w, int h, int v_w, int v_h, int flags, _DRIVER_INFO *driver_list)
+static int get_config_gfx_driver(char *gfx_card, int check_mode, int require_window, _DRIVER_INFO *driver_list, int card, int w, int h, int v_w, int v_h)
 {
+   int tried = FALSE;
    char buf[512], tmp[64];
-   GFX_DRIVER *drv;
-   int found = FALSE;
-   int card, n;
+   int c, n;
 
    /* try the drivers that are listed in the config file */
    for (n=-2; n<255; n++) {
@@ -557,30 +439,39 @@ static int get_config_gfx_driver(char *gfx_card, int w, int h, int v_w, int v_h,
 	    uszprintf(buf, sizeof(buf), uconvert_ascii("%s%d", tmp), gfx_card, n);
 	    break;
       }
+      card = get_config_id(uconvert_ascii("graphics", tmp), buf, GFX_AUTODETECT);
 
-      card = get_config_id(uconvert_ascii("graphics", tmp), buf, 0);
-
-      if (card) {
-	 drv = get_gfx_driver_from_id(card, driver_list);
-
-	 if (drv && gfx_driver_is_valid(drv, flags)) {
-	    found = TRUE;
-	    screen = init_gfx_driver(drv, w, h, v_w, v_h);
-
+      if (card != GFX_AUTODETECT) {
+	 for (c=0; driver_list[c].driver; c++) {
+	    if (driver_list[c].id == card) {
+	       gfx_driver = driver_list[c].driver;
+	       if (check_mode) {
+		  if (((require_window) && (!gfx_driver->windowed)) ||
+		      ((!require_window) && (gfx_driver->windowed))) {
+		     gfx_driver = NULL;
+		     continue;
+		  }
+	       }
+	       break;
+	    }
+	 }
+	 if (gfx_driver) {
+	    tried = TRUE;
+	    gfx_driver->name = gfx_driver->desc = get_config_text(gfx_driver->ascii_name);
+	    screen = gfx_driver->init(w, h, v_w, v_h, _color_depth);
 	    if (screen)
 	       break;
+	    else
+	       gfx_driver = NULL;
 	 }
       }
       else {
-	 /* Stop searching the gfx_card#n (n>0) family at the first missing member,
-	  * except gfx_card1 which could have been identified with gfx_card.
-	  */
 	 if (n > 1)
 	    break;
       }
    }
 
-   return found;
+   return tried;
 }
 
 
@@ -598,53 +489,63 @@ static int get_config_gfx_driver(char *gfx_card, int w, int h, int v_w, int v_h,
  */
 int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
 {
-   TRACE(PREFIX_I "Called set_gfx_mode(%d, %d, %d, %d, %d).\n",
-	 card, w, h, v_w, v_h);
-         
-   /* TODO: shouldn't this be incremented only IF successful? */
+   static int allow_config = TRUE;
+   extern void blit_end();
+   _DRIVER_INFO *driver_list;
+   int tried = FALSE;
+   char buf[ALLEGRO_ERROR_SIZE], tmp[64];
+   int c;
+   int check_mode = FALSE, require_window = FALSE;
+
+
    _gfx_mode_set_count++;
 
+   /* check specific fullscreen/windowed mode detection */
+   if ((card == GFX_AUTODETECT_FULLSCREEN) ||
+       (card == GFX_AUTODETECT_WINDOWED)) {
+      check_mode = TRUE;
+      require_window = (card == GFX_AUTODETECT_WINDOWED ? TRUE : FALSE);
+      card = GFX_AUTODETECT;
+   }
+
    /* special bodge for the GFX_SAFE driver */
-   if (card == GFX_SAFE)
-      return _set_gfx_mode_safe(card, w, h, v_w, v_h);
-   else
-      return _set_gfx_mode(card, w, h, v_w, v_h, TRUE);
-}
+   if (card == GFX_SAFE) {
+      _safe_gfx_mode_change = 1;
+      ustrzcpy(buf, sizeof(buf), allegro_error);
+      if (set_gfx_mode(GFX_AUTODETECT, w, h, 0, 0) != 0) {
+	 set_color_depth(8);
+	 allow_config = FALSE;
+	 _safe_gfx_mode_change = 0;
+      #ifdef GFX_SAFE_ID
+	 if (set_gfx_mode(GFX_SAFE_ID, GFX_SAFE_W, GFX_SAFE_H, 0, 0) != 0) {
+      #else
+	 if (set_gfx_mode(GFX_AUTODETECT, 0, 0, 0, 0) != 0) {
+      #endif
+	    set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
+	    allegro_message(uconvert_ascii("%s\n", tmp), get_config_text("Fatal error: unable to set GFX_SAFE"));
+	    return -1;
+	 }
+	 allow_config = TRUE;
+      }
 
-
-
-/* _set_gfx_mode:
- *  Called by set_gfx_mode(). Separated to make a clear difference between
- *  the virtual GFX_SAFE driver and the rest. The allow_config parameter,
- *  if true, allows the configuration to override the graphics card/driver
- *  when using GFX_AUTODETECT.
- */
-static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_config)
-{
-   _DRIVER_INFO *driver_list;
-   GFX_DRIVER *drv;
-   char tmp1[64], tmp2[64];
-   AL_CONST char *dv;
-   int flags = 0;
-   int c;
-   ASSERT(system_driver);
-   ASSERT(card != GFX_SAFE);
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, buf);
+      _safe_gfx_mode_change = 0;
+      return 0;
+   }
 
    /* remember the current console state */
    if (gfx_virgin) {
-      TRACE(PREFIX_I "First call, remembering console state.\n");
       LOCK_FUNCTION(_stub_bank_switch);
       LOCK_FUNCTION(blit);
 
       if (system_driver->save_console_state)
 	 system_driver->save_console_state();
 
-      _add_exit_func(shutdown_gfx, "shutdown_gfx");
+      _add_exit_func(shutdown_gfx);
 
       gfx_virgin = FALSE;
    }
 
-   /* lock the application in the foreground */
    if (system_driver->display_switch_lock)
       system_driver->display_switch_lock(TRUE, TRUE);
 
@@ -653,8 +554,6 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
 
    /* close down any existing graphics driver */
    if (gfx_driver) {
-      TRACE(PREFIX_I "Closing graphics driver (%p) ", gfx_driver);
-      TRACE("%s.\n", gfx_driver->ascii_name);
       if (_al_linker_mouse)
          _al_linker_mouse->show_mouse(NULL);
 
@@ -678,12 +577,6 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
       gfx_capabilities = 0;
    }
 
-   /* We probably don't want to do this because it makes
-    * Allegro "forget" the color layout of previously set
-    * graphics modes. But it should be retained if bitmaps
-    * created in those modes are to be used in the new mode.
-    */
-#if 0
    /* restore default truecolor pixel format */
    _rgb_r_shift_15 = 0; 
    _rgb_g_shift_15 = 5;
@@ -698,98 +591,78 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
    _rgb_g_shift_32 = 8;
    _rgb_b_shift_32 = 16;
    _rgb_a_shift_32 = 24;
-#endif
 
    gfx_capabilities = 0;
 
-   _set_current_refresh_rate(0);
+   _current_refresh_rate = 0;
 
    /* return to text mode? */
    if (card == GFX_TEXT) {
-      TRACE(PREFIX_I "Closing, restoring original console state.\n");
       if (system_driver->restore_console_state)
 	 system_driver->restore_console_state();
 
       if (_gfx_bank) {
-	 _AL_FREE(_gfx_bank);
+	 free(_gfx_bank);
 	 _gfx_bank = NULL;
       }
 
       if (system_driver->display_switch_lock)
 	 system_driver->display_switch_lock(FALSE, FALSE);
 
-      TRACE(PREFIX_I "Graphic mode closed.\n");
       return 0;
    }
 
-   /* now to the interesting part: let's try to find a graphics driver */
-   usetc(allegro_error, 0);
-
    /* ask the system driver for a list of graphics hardware drivers */
    if (system_driver->gfx_drivers)
       driver_list = system_driver->gfx_drivers();
    else
       driver_list = _gfx_driver_list;
 
-   /* filter specific fullscreen/windowed driver requests */
-   if (card == GFX_AUTODETECT_FULLSCREEN) {
-      flags |= GFX_DRIVER_FULLSCREEN_FLAG;
-      card = GFX_AUTODETECT;
-   }
-   else if (card == GFX_AUTODETECT_WINDOWED) {
-      flags |= GFX_DRIVER_WINDOWED_FLAG;
-      card = GFX_AUTODETECT;
-   }
+   usetc(allegro_error, 0);
 
-   if (card == GFX_AUTODETECT) {
-      /* autodetect the driver */
-      int found = FALSE;
+   /* try windowed mode drivers first if GFX_AUTODETECT_WINDOWED was selected */
+   if ((card == GFX_AUTODETECT) && (allow_config) && (require_window))
+      tried = get_config_gfx_driver(uconvert_ascii("gfx_cardw", tmp), check_mode, require_window, driver_list, card, w, h, v_w, v_h);
 
-      tmp1[0] = '\0';
+   /* check the gfx_card config variable if gfx_cardw wasn't used */
+   if ((card == GFX_AUTODETECT) && (allow_config) && (!gfx_driver))
+      tried = get_config_gfx_driver(uconvert_ascii("gfx_card", tmp), check_mode, require_window, driver_list, card, w, h, v_w, v_h);
 
-      /* first try the config variables */
-      if (allow_config) {
-	 /* try the gfx_card variable if GFX_AUTODETECT or GFX_AUTODETECT_FULLSCREEN was selected */
-	 if (!(flags & GFX_DRIVER_WINDOWED_FLAG))
-	    found = get_config_gfx_driver(uconvert_ascii("gfx_card", tmp1), w, h, v_w, v_h, flags, driver_list);
-
-	 /* try the gfx_cardw variable if GFX_AUTODETECT or GFX_AUTODETECT_WINDOWED was selected */
-	 if (!(flags & GFX_DRIVER_FULLSCREEN_FLAG) && !found)
-	    found = get_config_gfx_driver(uconvert_ascii("gfx_cardw", tmp1), w, h, v_w, v_h, flags, driver_list);
+   if (!tried) {
+      /* search table for the requested driver */
+      for (c=0; driver_list[c].driver; c++) {
+	 if (driver_list[c].id == card) {
+	    gfx_driver = driver_list[c].driver;
+	    break;
+	 }
       }
 
-      /* go through the list of autodetected drivers if none was previously found */
-      if (!found) {
-	 TRACE(PREFIX_I "Autodetecting graphic driver.\n");
+      if (gfx_driver) {                            /* specific driver? */
+	 gfx_driver->name = gfx_driver->desc = get_config_text(gfx_driver->ascii_name);
+	 screen = gfx_driver->init(w, h, v_w, v_h, _color_depth);
+      }
+      else {                                      /* otherwise autodetect */
 	 for (c=0; driver_list[c].driver; c++) {
 	    if (driver_list[c].autodetect) {
-	       drv = driver_list[c].driver;
-
-	       if (gfx_driver_is_valid(drv, flags)) {
-		  screen = init_gfx_driver(drv, w, h, v_w, v_h);
-
-		  if (screen)
-		     break;
+	       gfx_driver = driver_list[c].driver;
+	       if (check_mode) {
+		  if (((require_window) && (!gfx_driver->windowed)) ||
+		      ((!require_window) && (gfx_driver->windowed)))
+		  continue;
 	       }
+	       gfx_driver->name = gfx_driver->desc = get_config_text(gfx_driver->ascii_name);
+	       screen = gfx_driver->init(w, h, v_w, v_h, _color_depth);
+	       if (screen)
+		  break;
 	    }
 	 }
       }
-      else {
-	 TRACE(PREFIX_I "GFX_AUTODETECT overridden through configuration:"
-	       " %s.\n", tmp1);
-      }
-   }
-   else {
-      /* search the list for the requested driver */
-      drv = get_gfx_driver_from_id(card, driver_list);
-
-      if (drv)
-	 screen = init_gfx_driver(drv, w, h, v_w, v_h);
    }
 
-   /* gracefully handle failure */
+
    if (!screen) {
-      gfx_driver = NULL;  /* set by init_gfx_driver() */
+      gfx_driver = NULL;
+      screen = NULL;
 
       if (!ugetc(allegro_error))
 	 ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Unable to find a suitable graphics driver"));
@@ -797,11 +670,9 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
       if (system_driver->display_switch_lock)
 	 system_driver->display_switch_lock(FALSE, FALSE);
 
-      TRACE(PREFIX_E "Failed setting graphic driver %d.\n", card);
       return -1;
    }
 
-   /* set the basic capabilities of the driver */
    if ((VIRTUAL_W > SCREEN_W) || (VIRTUAL_H > SCREEN_H)) {
       if (gfx_driver->scroll)
 	 gfx_capabilities |= GFX_CAN_SCROLL;
@@ -810,26 +681,6 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
 	 gfx_capabilities |= GFX_CAN_TRIPLE_BUFFER;
    }
 
-   /* check whether we are instructed to disable vsync */
-   dv = get_config_string(uconvert_ascii("graphics", tmp1),
-                          uconvert_ascii("disable_vsync", tmp2),
-                          NULL);
-
-   if ((dv) && ((c = ugetc(dv)) != 0) && ((c == 'y') || (c == 'Y') || (c == '1')))
-      _wait_for_vsync = FALSE;
-   else
-      _wait_for_vsync = TRUE;
-
-   TRACE(PREFIX_I "The driver %s wait for vsync.\n",
-	 (_wait_for_vsync) ? "will" : "won't");
-
-   /* Give the gfx driver an opportunity to set the drawing mode */
-   if ((gfx_driver->drawing_mode) && (!_dispsw_status))
-      gfx_driver->drawing_mode();
-
-   clear_bitmap(screen);
-
-   /* set up the default colors */
    for (c=0; c<256; c++)
       _palette_color8[c] = c;
 
@@ -846,6 +697,8 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
       gui_bg_color = makecol(255, 255, 255);
    }
 
+   clear_bitmap(screen);
+
    if (_al_linker_mouse)
       _al_linker_mouse->set_mouse_etc();
 
@@ -856,76 +709,11 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
    if (system_driver->display_switch_lock)
       system_driver->display_switch_lock(FALSE, FALSE);
 
-   TRACE(PREFIX_I "set_gfx_card success for %dx%dx%d.\n",
-	 screen->w, screen->h, bitmap_color_depth(screen));
    return 0;
 } 
 
 
 
-/* _set_gfx_mode_safe:
- *  Special wrapper used when the card parameter of set_gfx_mode()
- *  is GFX_SAFE. In this case the function tries to query the
- *  system driver for a "safe" resolution+driver it knows it will
- *  work, and set it. If the system driver cannot get a "safe"
- *  resolution+driver, it will try the given parameters.
- */
-static int _set_gfx_mode_safe(int card, int w, int h, int v_w, int v_h)
-{
-   char buf[ALLEGRO_ERROR_SIZE], tmp1[64];
-   struct GFX_MODE mode;
-   int ret, driver;
-
-   ASSERT(card == GFX_SAFE);
-   ASSERT(system_driver);
-   TRACE(PREFIX_I "Trying to set a safe graphics mode.\n");
-   
-   if (system_driver->get_gfx_safe_mode) {
-      ustrzcpy(buf, sizeof(buf), allegro_error);
-
-      /* retrieve the safe graphics mode */
-      system_driver->get_gfx_safe_mode(&driver, &mode);
-      TRACE(PREFIX_I "The system driver suggests %dx%dx%d\n",
-	    mode.width, mode.height, mode.bpp);
-
-      /* try using the specified resolution but current depth */
-      if (_set_gfx_mode(driver, w, h, 0, 0, TRUE) == 0)
-         return 0;
-
-      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, buf);
-
-      /* finally use the safe settings */
-      set_color_depth(mode.bpp);
-      if (_set_gfx_mode(driver, mode.width, mode.height, 0, 0, TRUE) == 0)
-         return 0;
-
-      ASSERT(FALSE);  /* the safe graphics mode must always work */
-   }
-   else {
-      TRACE(PREFIX_W "The system driver was unable to get a safe mode, "
-	    "I'll try with the specified parameters...\n");
-      /* no safe graphics mode, try hard-coded autodetected modes with
-       * custom settings */
-      _safe_gfx_mode_change = 1;
-
-      ret = _set_gfx_mode(GFX_AUTODETECT, w, h, 0, 0, TRUE);
-
-      _safe_gfx_mode_change = 0;
-
-      if (ret == 0)
-         return 0;
-   }
-
-   /* failing to set GFX_SAFE is a fatal error */
-   TRACE(PREFIX_E "Bad bad, not even GFX_SAFE works?\n");
-   _set_gfx_mode(GFX_TEXT, 0, 0, 0, 0, TRUE);
-   allegro_message(uconvert_ascii("%s\n", tmp1),
-		   get_config_text("Fatal error: unable to set GFX_SAFE"));
-   return -1;
-}
-
-
-
 /* _sort_out_virtual_width:
  *  Decides how wide the virtual screen really needs to be. That is more 
  *  complicated than it sounds, because the Allegro graphics primitives 
@@ -964,7 +752,7 @@ void _sort_out_virtual_width(int *width, GFX_DRIVER *driver)
  *  switching table using bank size/granularity information from the 
  *  specified graphics driver.
  */
-BITMAP *_make_bitmap(int w, int h, uintptr_t addr, GFX_DRIVER *driver, int color_depth, int bpl)
+BITMAP *_make_bitmap(int w, int h, unsigned long addr, GFX_DRIVER *driver, int color_depth, int bpl)
 {
    GFX_VTABLE *vtable = _get_vtable(color_depth);
    int i, bank, size;
@@ -975,13 +763,13 @@ BITMAP *_make_bitmap(int w, int h, uintptr_t addr, GFX_DRIVER *driver, int color
 
    size = sizeof(BITMAP) + sizeof(char *) * h;
 
-   b = (BITMAP *)_AL_MALLOC(size);
+   b = (BITMAP *)malloc(size);
    if (!b)
       return NULL;
 
-   _gfx_bank = _AL_REALLOC(_gfx_bank, h * sizeof(int));
+   _gfx_bank = realloc(_gfx_bank, h * sizeof(int));
    if (!_gfx_bank) {
-      _AL_FREE(b);
+      free(b);
       return NULL;
    }
 
@@ -1044,14 +832,8 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
 {
    GFX_VTABLE *vtable;
    BITMAP *bitmap;
-   int nr_pointers;
-   int padding;
    int i;
 
-   ASSERT(width >= 0);
-   ASSERT(height > 0);
-   ASSERT(system_driver);
-
    if (system_driver->create_bitmap)
       return system_driver->create_bitmap(color_depth, width, height);
 
@@ -1059,23 +841,13 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
    if (!vtable)
       return NULL;
 
-   /* We need at least two pointers when drawing, otherwise we get crashes with
-    * Electric Fence.  We think some of the assembly code assumes a second line
-    * pointer is always available.
-    */
-   nr_pointers = MAX(2, height);
-   bitmap = _AL_MALLOC(sizeof(BITMAP) + (sizeof(char *) * nr_pointers));
+   bitmap = malloc(sizeof(BITMAP) + (sizeof(char *) * height));
    if (!bitmap)
       return NULL;
 
-   /* This avoids a crash for assembler code accessing the last pixel, as it
-    * read 4 bytes instead of 3.
-    */
-   padding = (color_depth == 24) ? 1 : 0;
-
-   bitmap->dat = _AL_MALLOC_ATOMIC(width * height * BYTES_PER_PIXEL(color_depth) + padding);
+   bitmap->dat = malloc(width * height * BYTES_PER_PIXEL(color_depth));
    if (!bitmap->dat) {
-      _AL_FREE(bitmap);
+      free(bitmap);
       return NULL;
    }
 
@@ -1091,11 +863,9 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
    bitmap->y_ofs = 0;
    bitmap->seg = _default_ds();
 
-   if (height > 0) {
-      bitmap->line[0] = bitmap->dat;
-      for (i=1; i<height; i++)
-         bitmap->line[i] = bitmap->line[i-1] + width * BYTES_PER_PIXEL(color_depth);
-   }
+   bitmap->line[0] = bitmap->dat;
+   for (i=1; i<height; i++)
+      bitmap->line[i] = bitmap->line[i-1] + width * BYTES_PER_PIXEL(color_depth);
 
    if (system_driver->created_bitmap)
       system_driver->created_bitmap(bitmap);
@@ -1110,8 +880,6 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
  */
 BITMAP *create_bitmap(int width, int height)
 {
-   ASSERT(width >= 0);
-   ASSERT(height > 0);
    return create_bitmap_ex(_color_depth, width, height);
 }
 
@@ -1128,13 +896,16 @@ BITMAP *create_bitmap(int width, int height)
 BITMAP *create_sub_bitmap(BITMAP *parent, int x, int y, int width, int height)
 {
    BITMAP *bitmap;
-   int nr_pointers;
    int i;
 
-   ASSERT(parent);
-   ASSERT((x >= 0) && (y >= 0) && (x < parent->w) && (y < parent->h));
-   ASSERT((width > 0) && (height > 0));
-   ASSERT(system_driver);
+   if (!parent) 
+      return NULL;
+
+   if (x < 0) 
+      x = 0; 
+
+   if (y < 0) 
+      y = 0;
 
    if (x+width > parent->w) 
       width = parent->w-x;
@@ -1149,9 +920,7 @@ BITMAP *create_sub_bitmap(BITMAP *parent, int x, int y, int width, int height)
       return system_driver->create_sub_bitmap(parent, x, y, width, height);
 
    /* get memory for structure and line pointers */
-   /* (see create_bitmap for the reason we need at least two) */
-   nr_pointers = MAX(2, height);
-   bitmap = _AL_MALLOC(sizeof(BITMAP) + (sizeof(char *) * nr_pointers));
+   bitmap = malloc(sizeof(BITMAP) + (sizeof(char *) * height));
    if (!bitmap)
       return NULL;
 
@@ -1214,83 +983,60 @@ BITMAP *create_sub_bitmap(BITMAP *parent, int x, int y, int width, int height)
 
 
 
-/* add_vram_block:
- *  Creates a video memory bitmap out of the specified region
- *  of the larger screen surface, returning a pointer to it.
+/* try_vram_location:
+ *  Attempts to create a video memory bitmap out of the specified region
+ *  of the larger screen surface, returning a pointer to it if that area
+ *  is free.
  */
-static BITMAP *add_vram_block(int x, int y, int w, int h)
+static BITMAP *try_vram_location(int x, int y, int w, int h)
 {
-   VRAM_BITMAP *b, *new_b;
-   VRAM_BITMAP **last_p;
+   VRAM_BITMAP *block = vram_bitmap_list;
 
-   new_b = _AL_MALLOC(sizeof(VRAM_BITMAP));
-   if (!new_b)
+   if ((x<0) || (y<0) || (x+w > VIRTUAL_W) || (y+h > VIRTUAL_H))
       return NULL;
 
-   new_b->x = x;
-   new_b->y = y;
-   new_b->w = w;
-   new_b->h = h;
+   while (block) {
+      if (((x < block->x+block->w) && (x+w > block->x)) &&
+	  ((y < block->y+block->h) && (y+h > block->y)))
+	 return NULL;
 
-   new_b->bmp = create_sub_bitmap(screen, x, y, w, h);
-   if (!new_b->bmp) {
-      _AL_FREE(new_b);
+      block = block->next;
+   }
+
+   block = malloc(sizeof(VRAM_BITMAP));
+   if (!block)
+      return NULL;
+
+   block->x = x;
+   block->y = y;
+   block->w = w;
+   block->h = h;
+
+   block->bmp = create_sub_bitmap(screen, x, y, w, h);
+   if (!(block->bmp)) {
+      free(block);
       return NULL;
    }
 
-   /* find sorted y-position */
-   last_p = &vram_bitmap_list;
-   for (b = vram_bitmap_list; b && (b->y < new_b->y); b = b->next_y)
-      last_p = &b->next_y;
+   block->next = vram_bitmap_list;
+   vram_bitmap_list = block;
 
-   /* insert */
-   *last_p = new_b;
-   new_b->next_y = b;
-
-   return new_b->bmp;
+   return block->bmp;
 }
 
 
 
 /* create_video_bitmap:
  *  Attempts to make a bitmap object for accessing offscreen video memory.
- *
- *  The algorithm is similar to algorithms for drawing polygons. Think of
- *  a horizontal stripe whose height is equal to that of the new bitmap.
- *  It is initially aligned to the top of video memory and then it moves
- *  downwards, stopping each time its top coincides with the bottom of a
- *  video bitmap. For each stop, create a list of video bitmaps intersecting
- *  the stripe, sorted by the left coordinate, from left to right, in the
- *  next_x linked list. We look through this list and stop as soon as the
- *  gap between the rightmost right edge seen so far and the current left
- *  edge is big enough for the new video bitmap. In that case, we are done.
- *  If we don't find such a gap, we move the stripe further down.
- *
- *  To make it efficient to find new bitmaps intersecting the stripe, the
- *  list of all bitmaps is always sorted by top, from top to bottom, in
- *  the next_y linked list. The list of bitmaps intersecting the stripe is
- *  merely updated, not recalculated from scratch, when we move the stripe.
- *  So every bitmap gets bubbled to its correct sorted x-position at most
- *  once. Bubbling a bitmap takes at most as many steps as the number of
- *  video bitmaps. So the algorithm behaves at most quadratically with
- *  regard to the number of video bitmaps. I think that a linear behaviour
- *  is more typical in practical applications (this happens, for instance,
- *  if you allocate many bitmaps of the same height).
- *
- *  There's one more optimization: if a call fails, we cache the size of the
- *  requested bitmap, so that next time we can detect very quickly that this
- *  size is too large (this needs to be maintained when destroying bitmaps).
  */
 BITMAP *create_video_bitmap(int width, int height)
 {
-   VRAM_BITMAP *active_list, *b, *vram_bitmap;
-   VRAM_BITMAP **last_p;
+   VRAM_BITMAP origin_vram_bitmap;
+   VRAM_BITMAP *blockx;
+   VRAM_BITMAP *blocky;
+   VRAM_BITMAP *block;
    BITMAP *bmp;
-   int x = 0, y = 0;
 
-   ASSERT(width >= 0);
-   ASSERT(height > 0);
-   
    if (_dispsw_status)
       return NULL;
 
@@ -1300,111 +1046,32 @@ BITMAP *create_video_bitmap(int width, int height)
       if (!bmp)
 	 return NULL;
 
-      b = _AL_MALLOC(sizeof(VRAM_BITMAP));
-      b->x = -1;
-      b->y = -1;
-      b->w = 0;
-      b->h = 0;
-      b->bmp = bmp;
-      b->next_y = vram_bitmap_list;
-      vram_bitmap_list = b;
+      block = malloc(sizeof(VRAM_BITMAP));
+      block->x = -1;
+      block->y = -1;
+      block->w = 0;
+      block->h = 0;
+      block->bmp = bmp;
+      block->next = vram_bitmap_list;
+      vram_bitmap_list = block;
 
       return bmp;
    }
 
-   /* check bad args */
-   if ((width > VIRTUAL_W) || (height > VIRTUAL_H) ||
-       (width < 0) || (height < 0))
-      return NULL;
+   /* otherwise fall back on subdividing the normal screen surface:
+    *  it is sufficient to test the nodes of the grid made up of the X-axis, the Y-axis
+    *  and the straight lines carrying the right edge or the bottom edge of the bitmaps
+    */
+   memset(&origin_vram_bitmap, 0, sizeof(VRAM_BITMAP));  /* brings X-axis and Y-axis */
+   origin_vram_bitmap.next = vram_bitmap_list;
 
-   /* check cached bitmap size */
-   if ((width >= failed_bitmap_w) && (height >= failed_bitmap_h))
-      return NULL;
+   /* do the search in O(n^3) time */
+   for (blocky = &origin_vram_bitmap; blocky; blocky = blocky->next)
+      for (blockx = &origin_vram_bitmap; blockx; blockx = blockx->next)
+	 if ((bmp = try_vram_location((blockx->x+blockx->w+15)&~15, blocky->y+blocky->h, width, height)) != NULL)
+	    return bmp;
 
-   vram_bitmap = vram_bitmap_list;
-   active_list = NULL;
-   y = 0;
-
-   while (TRUE) {
-      /* Move the blocks from the VRAM_BITMAP_LIST that intersect the
-       * stripe to the ACTIVE_LIST: look through the VRAM_BITMAP_LIST
-       * following the next_y link, grow the ACTIVE_LIST following
-       * the next_x link and sorting by x-position.
-       * (this loop runs in amortized quadratic time)
-       */
-      while (vram_bitmap && (vram_bitmap->y < y+height)) {
-	 /* find sorted x-position */
-	 last_p = &active_list;
-	 for (b = active_list; b && (vram_bitmap->x > b->x); b = b->next_x)
-	    last_p = &b->next_x;
-
-	 /* insert */
-	 *last_p = vram_bitmap;
-	 vram_bitmap->next_x = b;
-
-	 /* next video bitmap */
-	 vram_bitmap = vram_bitmap->next_y;
-      }
-
-      x = 0;
-
-      /* Look for holes to put our bitmap in.
-       * (this loop runs in quadratic time)
-       */
-      for (b = active_list; b; b = b->next_x) {
-	 if (x+width <= b->x)  /* hole is big enough? */
-	    return add_vram_block(x, y, width, height);
-
-         /* Move search x-position to the right edge of the
-          * skipped bitmap if we are not already past it.
-          * And check there is enough room before continuing.
-          */
-	 if (x < b->x + b->w) {
-	    x = (b->x + b->w + 15) & ~15;
-	    if (x+width > VIRTUAL_W)
-	       break;
-	 }
-      }
-
-      /* If the whole ACTIVE_LIST was scanned, there is a big
-       * enough hole to the right of the rightmost bitmap.
-       */
-      if (b == NULL)
-	 return add_vram_block(x, y, width, height);
-
-      /* Move search y-position to the topmost bottom edge
-       * of the bitmaps intersecting the stripe.
-       * (this loop runs in quadratic time)
-       */
-      y = active_list->y + active_list->h;
-      for (b = active_list->next_x; b; b = b->next_x) {
-	 if (y > b->y + b->h)
-	    y = b->y + b->h;
-      }
-
-      if (y+height > VIRTUAL_H) {  /* too close to bottom? */
-	 /* Before failing, cache this bitmap size so that later calls can
-	  * learn from it. Use area as a measure to sort the bitmap sizes.
-          */
-	 if (width * height < failed_bitmap_w * failed_bitmap_h) {
-	    failed_bitmap_w = width;
-	    failed_bitmap_h = height;
-	 }
-
-	 return NULL;
-      }
-
-      /* Remove the blocks that don't intersect the new stripe from ACTIVE_LIST.
-       * (this loop runs in quadratic time)
-       */
-      last_p = &active_list;
-      for (b = active_list; b; b = b->next_x) {
-	 if (y >= b->y + b->h)
-	    *last_p = b->next_x;
-	 else
-	    last_p = &b->next_x;
-      }
-   }
+   return NULL;
 }
 
 
@@ -1416,8 +1083,6 @@ BITMAP *create_system_bitmap(int width, int height)
 {
    BITMAP *bmp;
 
-   ASSERT(width >= 0);
-   ASSERT(height > 0);
    ASSERT(gfx_driver != NULL);
 
    if (gfx_driver->create_system_bitmap)
@@ -1449,35 +1114,23 @@ void destroy_bitmap(BITMAP *bitmap)
 	 while (pos) {
 	    if (pos->bmp == bitmap) {
 	       if (prev)
-		  prev->next_y = pos->next_y;
+		  prev->next = pos->next;
 	       else
-		  vram_bitmap_list = pos->next_y;
+		  vram_bitmap_list = pos->next;
 
 	       if (pos->x < 0) {
 		  /* the driver is in charge of this object */
 		  gfx_driver->destroy_video_bitmap(bitmap);
-		  _AL_FREE(pos);
+		  free(pos);
 		  return;
 	       } 
 
-	       /* Update cached bitmap size using worst case scenario:
-		* the bitmap lies between two holes whose size is the cached
-		* size on each axis respectively.
-		*/
-	       failed_bitmap_w = failed_bitmap_w * 2 + ((bitmap->w + 15) & ~15);
-	       if (failed_bitmap_w > BMP_MAX_SIZE)
-		  failed_bitmap_w = BMP_MAX_SIZE;
-
-	       failed_bitmap_h = failed_bitmap_h * 2 + bitmap->h;
-	       if (failed_bitmap_h > BMP_MAX_SIZE)
-		  failed_bitmap_h = BMP_MAX_SIZE;
-
-	       _AL_FREE(pos);
+	       free(pos);
 	       break;
 	    }
 
 	    prev = pos;
-	    pos = pos->next_y;
+	    pos = pos->next;
 	 }
 
 	 _unregister_switch_bitmap(bitmap);
@@ -1499,61 +1152,14 @@ void destroy_bitmap(BITMAP *bitmap)
       }
 
       if (bitmap->dat)
-	 _AL_FREE(bitmap->dat);
+	 free(bitmap->dat);
 
-      _AL_FREE(bitmap);
+      free(bitmap);
    }
 }
 
 
 
-/* set_clip_rect:
- *  Sets the two opposite corners of the clipping rectangle to be used when
- *  drawing to the bitmap. Nothing will be drawn to positions outside of this 
- *  rectangle. When a new bitmap is created the clipping rectangle will be 
- *  set to the full area of the bitmap.
- */
-void set_clip_rect(BITMAP *bitmap, int x1, int y1, int x2, int y2)
-{
-   ASSERT(bitmap);
-
-   /* internal clipping is inclusive-exclusive */
-   x2++;
-   y2++;
-
-   bitmap->cl = MID(0, x1, bitmap->w-1);
-   bitmap->ct = MID(0, y1, bitmap->h-1);
-   bitmap->cr = MID(0, x2, bitmap->w);
-   bitmap->cb = MID(0, y2, bitmap->h);
-
-   if (bitmap->vtable->set_clip)
-      bitmap->vtable->set_clip(bitmap);
-}
-
-
-
-/* add_clip_rect:
- *  Makes the new clipping rectangle the intersection between the given
- *  rectangle and the current one.
- */
-void add_clip_rect(BITMAP *bitmap, int x1, int y1, int x2, int y2)
-{
-   int cx1, cy1, cx2, cy2;
-
-   ASSERT(bitmap);
-
-   get_clip_rect(bitmap, &cx1, &cy1, &cx2, &cy2);
-
-   x1 = MAX(x1, cx1);
-   y1 = MAX(y1, cy1);
-   x2 = MIN(x2, cx2);
-   y2 = MIN(y2, cy2);
-
-   set_clip_rect(bitmap, x1, y1, x2, y2);
-}
-
-
-
 /* set_clip:
  *  Sets the two opposite corners of the clipping rectangle to be used when
  *  drawing to the bitmap. Nothing will be drawn to positions outside of this 
@@ -1570,8 +1176,14 @@ void set_clip(BITMAP *bitmap, int x1, int y1, int x2, int y2)
    ASSERT(bitmap);
 
    if ((!x1) && (!y1) && (!x2) && (!y2)) {
-      set_clip_rect(bitmap, 0, 0, bitmap->w-1, bitmap->h-1);
-      set_clip_state(bitmap, FALSE);
+      bitmap->clip = FALSE;
+      bitmap->cl = bitmap->ct = 0;
+      bitmap->cr = bitmap->w;
+      bitmap->cb = bitmap->h;
+
+      if (bitmap->vtable->set_clip)
+	 bitmap->vtable->set_clip(bitmap);
+
       return;
    }
 
@@ -1587,8 +1199,17 @@ void set_clip(BITMAP *bitmap, int x1, int y1, int x2, int y2)
       y2 = t;
    }
 
-   set_clip_rect(bitmap, x1, y1, x2, y2);
-   set_clip_state(bitmap, TRUE);
+   x2++;
+   y2++;
+
+   bitmap->clip = TRUE;
+   bitmap->cl = MID(0, x1, bitmap->w-1);
+   bitmap->ct = MID(0, y1, bitmap->h-1);
+   bitmap->cr = MID(0, x2, bitmap->w);
+   bitmap->cb = MID(0, y2, bitmap->h);
+
+   if (bitmap->vtable->set_clip)
+      bitmap->vtable->set_clip(bitmap);
 }
 
 
@@ -1690,7 +1311,7 @@ int request_scroll(int x, int y)
 /* poll_scroll:
  *  Checks whether a requested triple buffer flip has actually taken place.
  */
-int poll_scroll(void)
+int poll_scroll()
 {
    if ((!gfx_driver->poll_scroll) || (_dispsw_status))
       return FALSE;
@@ -1742,7 +1363,7 @@ int request_video_bitmap(BITMAP *bitmap)
  *  Asks a driver to turn on triple buffering mode, if it is capable
  *  of that.
  */
-int enable_triple_buffer(void)
+int enable_triple_buffer()
 {
    if (gfx_capabilities & GFX_CAN_TRIPLE_BUFFER)
       return 0;
