@@ -1400,7 +1400,7 @@ static char* split_around_tab(const char *s, char **tok1, char **tok2)
    char *buf, *last;
    char tmp[16];
 
-   buf = ustrdup(s);
+   buf = _al_ustrdup(s);
    *tok1 = ustrtok_r(buf, uconvert_ascii("\t", tmp), &last);
    *tok2 = ustrtok_r(NULL, empty_string, &last);
 
@@ -1788,8 +1788,8 @@ static MENU_PLAYER *init_single_menu(MENU *menu, MENU_PLAYER *parent, DIALOG *di
    layout_menu(player, menu, bar, x, y, minw, minh);
 
    if (repos) {
-      player->x = MID(0, player->x, SCREEN_W-player->w-1);
-      player->y = MID(0, player->y, SCREEN_H-player->h-1);
+      player->x = CLAMP(0, player->x, SCREEN_W-player->w-1);
+      player->y = CLAMP(0, player->y, SCREEN_H-player->h-1);
    }
 
    if (scare)
