@@ -28,22 +28,18 @@
 #include "allegro/internal/aintern.h"
 
 
-#ifdef ALLEGRO_WINDOWS
-/* exported address of d_clear_proc */
-int (*_d_clear_proc)(int, DIALOG *, int) = NULL;
-#endif
 
 /* typedef for the listbox callback functions */
 typedef char *(*getfuncptr)(int, int *);
 
 
 
-/* gui_textout_ex:
+/* gui_textout:
  *  Wrapper function for drawing text to the screen, which interprets the
  *  & character as an underbar for displaying keyboard shortcuts. Returns
  *  the width of the output string in pixels.
  */
-int gui_textout_ex(BITMAP *bmp, AL_CONST char *s, int x, int y, int color, int bg, int centre)
+int gui_textout(BITMAP *bmp, AL_CONST char *s, int x, int y, int color, int centre)
 {
    char tmp[1024];
    int hline_pos = -1;
@@ -51,7 +47,6 @@ int gui_textout_ex(BITMAP *bmp, AL_CONST char *s, int x, int y, int color, int b
    int in_pos = 0;
    int out_pos = 0;
    int pix_len, c;
-   ASSERT(s);
 
    while (((c = ugetc(s+in_pos)) != 0) && (out_pos<(int)(sizeof(tmp)-ucwidth(0)))) {
       if (c == '&') {
@@ -79,7 +74,7 @@ int gui_textout_ex(BITMAP *bmp, AL_CONST char *s, int x, int y, int color, int b
       x -= pix_len / 2;
 
    if (bmp) {
-      textout_ex(bmp, font, tmp, x, y, color, bg);
+      textout(bmp, font, tmp, x, y, color);
 
       if (hline_pos >= 0) {
 	 c = ugetat(tmp, hline_pos);
@@ -102,8 +97,7 @@ int gui_textout_ex(BITMAP *bmp, AL_CONST char *s, int x, int y, int color, int b
  */
 int gui_strlen(AL_CONST char *s)
 {
-   ASSERT(s);
-   return gui_textout_ex(NULL, s, 0, 0, 0, 0, 0);
+   return gui_textout(NULL, s, 0, 0, 0, 0);
 }
 
 
@@ -150,14 +144,6 @@ int d_yield_proc(int msg, DIALOG *d, int c)
  */
 int d_clear_proc(int msg, DIALOG *d, int c)
 {
-   ASSERT(d);
-
-#ifdef ALLEGRO_WINDOWS
-   /* kludge to get the exported address of d_clear_proc */
-   if (!_d_clear_proc)
-      _d_clear_proc = d->proc;
-#endif
-
    if (msg == MSG_DRAW) {
       set_clip(screen, 0, 0, SCREEN_W-1, SCREEN_H-1);
       clear_to_color(screen, d->bg);
@@ -173,7 +159,6 @@ int d_clear_proc(int msg, DIALOG *d, int c)
  */
 int d_box_proc(int msg, DIALOG *d, int c)
 {
-   ASSERT(d);
    if (msg==MSG_DRAW) {
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       rectfill(screen, d->x+1, d->y+1, d->x+d->w-2, d->y+d->h-2, d->bg);
@@ -190,7 +175,6 @@ int d_box_proc(int msg, DIALOG *d, int c)
  */
 int d_shadow_box_proc(int msg, DIALOG *d, int c)
 {
-   ASSERT(d);
    if (msg==MSG_DRAW) {
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       int black = makecol(0,0,0);
@@ -210,10 +194,8 @@ int d_shadow_box_proc(int msg, DIALOG *d, int c)
  */
 int d_bitmap_proc(int msg, DIALOG *d, int c)
 {
-   BITMAP *b;
-   ASSERT(d);
+   BITMAP *b = (BITMAP *)d->dp;
 
-   b = (BITMAP *)d->dp;
    if (msg==MSG_DRAW)
       blit(b, screen, 0, 0, d->x, d->y, d->w, d->h);
 
@@ -227,15 +209,17 @@ int d_bitmap_proc(int msg, DIALOG *d, int c)
  */
 int d_text_proc(int msg, DIALOG *d, int c)
 {
-   ASSERT(d);
    if (msg==MSG_DRAW) {
+      int rtm;
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       FONT *oldfont = font;
 
       if (d->dp2)
 	 font = d->dp2;
 
-      gui_textout_ex(screen, d->dp, d->x, d->y, fg, d->bg, FALSE);
+      rtm = text_mode(d->bg);
+      gui_textout(screen, d->dp, d->x, d->y, fg, FALSE);
+      text_mode(rtm);
 
       font = oldfont;
    }
@@ -251,15 +235,17 @@ int d_text_proc(int msg, DIALOG *d, int c)
  */
 int d_ctext_proc(int msg, DIALOG *d, int c)
 {
-   ASSERT(d);
    if (msg==MSG_DRAW) {
+      int rtm;
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       FONT *oldfont = font;
 
       if (d->dp2)
 	 font = d->dp2;
 
-      gui_textout_ex(screen, d->dp, d->x + d->w/2, d->y, fg, d->bg, TRUE);
+      rtm = text_mode(d->bg);
+      gui_textout(screen, d->dp, d->x, d->y, fg, TRUE);
+      text_mode(rtm);
 
       font = oldfont;
    }
@@ -275,15 +261,17 @@ int d_ctext_proc(int msg, DIALOG *d, int c)
  */
 int d_rtext_proc(int msg, DIALOG *d, int c)
 {
-   ASSERT(d);
    if (msg==MSG_DRAW) {
+      int rtm;
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       FONT *oldfont = font;
 
       if (d->dp2)
 	 font = d->dp2;
 
-      gui_textout_ex(screen, d->dp, d->x + d->w - gui_strlen(d->dp), d->y, fg, d->bg, FALSE);
+      rtm = text_mode(d->bg);
+      gui_textout(screen, d->dp, d->x + d->w - gui_strlen(d->dp), d->y, fg, FALSE);
+      text_mode(rtm);
 
       font = oldfont;
    }
@@ -305,7 +293,7 @@ int d_button_proc(int msg, DIALOG *d, int c)
    int black;
    int swap;
    int g;
-   ASSERT(d);
+   int rtm;
 
    switch (msg) {
 
@@ -316,14 +304,16 @@ int d_button_proc(int msg, DIALOG *d, int c)
 	    state2 = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
 	 }
 	 else {
-	    g = 0;
+	    g = 0; 
 	    state1 = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
 	    state2 = d->bg;
 	 }
 
 	 rectfill(screen, d->x+1+g, d->y+1+g, d->x+d->w-3+g, d->y+d->h-3+g, state2);
 	 rect(screen, d->x+g, d->y+g, d->x+d->w-2+g, d->y+d->h-2+g, state1);
-	 gui_textout_ex(screen, d->dp, d->x+d->w/2+g, d->y+d->h/2-text_height(font)/2+g, state1, -1, TRUE);
+	 rtm = text_mode(-1);
+	 gui_textout(screen, d->dp, d->x+d->w/2+g, d->y+d->h/2-text_height(font)/2+g, state1, TRUE);
+	 text_mode(rtm);
 
 	 if (d->flags & D_SELECTED) {
 	    vline(screen, d->x, d->y, d->y+d->h-2, d->bg);
@@ -350,7 +340,9 @@ int d_button_proc(int msg, DIALOG *d, int c)
 
 	 /* or just toggle */
 	 d->flags ^= D_SELECTED;
+	 scare_mouse();
 	 object_message(d, MSG_DRAW, 0);
+	 unscare_mouse();
 	 break;
 
       case MSG_CLICK:
@@ -372,7 +364,9 @@ int d_button_proc(int msg, DIALOG *d, int c)
 	    if (((state1) && (!state2)) || ((state2) && (!state1))) {
 	       d->flags ^= D_SELECTED;
 	       state1 = d->flags & D_SELECTED;
+	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
+	       unscare_mouse();
 	    }
 
 	    /* let other objects continue to animate */
@@ -400,17 +394,20 @@ int d_check_proc(int msg, DIALOG *d, int c)
 {
    int x;
    int fg, bg;
-   ASSERT(d);
 
    if (msg==MSG_DRAW) {
+      int rtm;
+
       fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       bg = (d->bg < 0) ? gui_bg_color : d->bg;
 
-      x = d->x + ((d->d1) ? 0 : gui_textout_ex(screen, d->dp, d->x, d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, d->bg, FALSE) + text_height(font)/2);
+      rtm = text_mode(d->bg);
+
+      x = d->x + ((d->d1) ? 0 : gui_textout(screen, d->dp, d->x, d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, FALSE) + text_height(font)/2);
       rectfill(screen, x+1, d->y+1, x+d->h-2, d->y+d->h-2, bg);
       rect(screen, x, d->y, x+d->h-1, d->y+d->h-1, fg);
       if (d->d1)
-	 gui_textout_ex(screen, d->dp, x+d->h-1+text_height(font)/2, d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, d->bg, FALSE);
+	 gui_textout(screen, d->dp, x+d->h-1+text_height(font)/2, d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, FALSE);
       if (d->flags & D_SELECTED) {
 	 line(screen, x, d->y, x+d->h-1, d->y+d->h-1, fg);
 	 line(screen, x, d->y+d->h-1, x+d->h-1, d->y, fg);
@@ -418,6 +415,8 @@ int d_check_proc(int msg, DIALOG *d, int c)
       if (d->flags & D_GOTFOCUS)
 	 dotted_rect(x+1, d->y+1, x+d->h-2, d->y+d->h-2, fg, bg);
 
+      text_mode(rtm);
+
       return D_O_K;
    } 
 
@@ -434,7 +433,7 @@ int d_check_proc(int msg, DIALOG *d, int c)
 int d_radio_proc(int msg, DIALOG *d, int c)
 {
    int x, center, r, ret, fg, bg;
-   ASSERT(d);
+   int rtm;
 
    switch(msg) {
 
@@ -442,7 +441,9 @@ int d_radio_proc(int msg, DIALOG *d, int c)
 	 fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
 	 bg = (d->bg < 0) ? gui_bg_color : d->bg;
 
-	 gui_textout_ex(screen, d->dp, d->x+d->h-1+text_height(font), d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, d->bg, FALSE);
+	 rtm = text_mode(d->bg);
+	 gui_textout(screen, d->dp, d->x+d->h-1+text_height(font), d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, FALSE);
+	 text_mode(rtm);
 
 	 x = d->x;
 	 r = d->h/2;
@@ -484,7 +485,9 @@ int d_radio_proc(int msg, DIALOG *d, int c)
       case MSG_RADIO:
 	 if ((c == d->d1) && (d->flags & D_SELECTED)) {
 	    d->flags &= ~D_SELECTED;
+	    scare_mouse();
 	    object_message(d, MSG_DRAW, 0);
+	    unscare_mouse();
 	 }
 	 break;
    }
@@ -520,15 +523,13 @@ int d_radio_proc(int msg, DIALOG *d, int c)
  */
 int d_icon_proc(int msg, DIALOG *d, int c)
 {
-   BITMAP *butimage;
+   BITMAP *butimage = (BITMAP *)d->dp;
    int butx;
    int buty;
    int index;
    int indent;
    int depth;
-   ASSERT(d);
 
-   butimage = (BITMAP *)d->dp;
    if ((msg == MSG_DRAW) && (!(d->flags & D_HIDDEN))) {
       depth = 0;
       if ((d->dp2 == NULL) && (d->flags & D_SELECTED)) {
@@ -588,9 +589,8 @@ int d_icon_proc(int msg, DIALOG *d, int c)
  */
 int d_keyboard_proc(int msg, DIALOG *d, int c)
 {
-   int (*proc)(void);
+   int (*proc)();
    int ret = D_O_K;
-   ASSERT(d);
 
    switch (msg) {
 
@@ -630,7 +630,7 @@ int d_edit_proc(int msg, DIALOG *d, int c)
    int f, l, p, w, x, fg, b, scroll;
    char buf[16];
    char *s, *t;
-   ASSERT(d);
+   int rtm;
 
    s = d->dp;
    l = ustrlen(s);
@@ -685,10 +685,12 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	    f = ugetat(s, p);
 	    usetc(buf+usetc(buf, (f) ? f : ' '), 0);
 	    w = text_length(font, buf);
-	    if (x+w > d->w)
+	    if (x+w > d->w) 
 	       break;
 	    f = ((p == d->d2) && (d->flags & D_GOTFOCUS));
-	    textout_ex(screen, font, buf, d->x+x, d->y, (f) ? d->bg : fg, (f) ? fg : d->bg);
+	    rtm = text_mode((f) ? fg : d->bg);
+	    textout(screen, font, buf, d->x+x, d->y, (f) ? d->bg : fg);
+	    text_mode(rtm);
 	    x += w;
 	 }
 	 if (x < d->w)
@@ -712,7 +714,9 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	       break;
 	 }
 	 d->d2 = MID(0, p, l);
+	 scare_mouse();
 	 object_message(d, MSG_DRAW, 0);
+	 unscare_mouse();
 	 break;
 
       case MSG_WANTFOCUS:
@@ -775,7 +779,9 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	 }
 	 else if ((c >> 8) == KEY_ENTER) {
 	    if (d->flags & D_EXIT) {
+	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
+	       unscare_mouse();
 	       return D_CLOSE;
 	    }
 	    else
@@ -789,7 +795,9 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	    /* don't process regular keys here: MSG_UCHAR will do that */
 	    break;
 	 }
+	 scare_mouse();
 	 object_message(d, MSG_DRAW, 0);
+	 unscare_mouse();
 	 return D_USED_CHAR;
 
       case MSG_UCHAR:
@@ -798,7 +806,9 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	       uinsert(s, d->d2, c);
 	       d->d2++;
 
+	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
+	       unscare_mouse();
 	    }
 	    return D_USED_CHAR;
 	 }
@@ -834,7 +844,9 @@ void _handle_scrollable_scroll_click(DIALOG *d, int listsize, int *offset, int h
 
 	    if (yy != *offset) {
 	       *offset = yy;
+	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
+	       unscare_mouse();
 	    }
 
 	    /* let other objects continue to animate */
@@ -855,7 +867,9 @@ void _handle_scrollable_scroll_click(DIALOG *d, int listsize, int *offset, int h
 
 	 if (yy != *offset) {
 	    *offset = yy;
+	    scare_mouse();
 	    object_message(d, MSG_DRAW, 0);
+	    unscare_mouse();
 	 }
       }
 
@@ -1046,6 +1060,7 @@ void _draw_listbox(DIALOG *d)
    int fg_color, fg, bg;
    char *sel = d->dp2;
    char s[1024];
+   int rtm;
 
    (*(getfuncptr)d->dp)(-1, &listsize);
    height = (d->h-4) / text_height(font);
@@ -1071,14 +1086,16 @@ void _draw_listbox(DIALOG *d)
 	 ustrzcpy(s, sizeof(s), (*(getfuncptr)d->dp)(i+d->d2, NULL));
 	 x = d->x + 2;
 	 y = d->y + 2 + i*text_height(font);
-	 rectfill(screen, x, y, x+7, y+text_height(font)-1, bg);
+	 rtm = text_mode(bg);
+	 rectfill(screen, x, y, x+7, y+text_height(font)-1, bg); 
 	 x += 8;
 	 len = ustrlen(s);
 	 while (text_length(font, s) >= MAX(d->w - 1 - (bar ? 22 : 10), 1)) {
 	    len--;
 	    usetat(s, len, 0);
 	 }
-	 textout_ex(screen, font, s, x, y, fg, bg);
+	 textout(screen, font, s, x, y, fg);
+	 text_mode(rtm);
 	 x += text_length(font, s);
 	 if (x <= d->x+w) 
 	    rectfill(screen, x, y, d->x+w, y+text_height(font)-1, bg);
@@ -1118,7 +1135,6 @@ int d_list_proc(int msg, DIALOG *d, int c)
    int listsize, i, bottom, height, bar, orig;
    char *sel = d->dp2;
    int redraw = FALSE;
-   ASSERT(d);
 
    switch (msg) {
 
@@ -1143,8 +1159,11 @@ int d_list_proc(int msg, DIALOG *d, int c)
 		     sel[i] = FALSE;
 		  }
 	       }
-	       if (redraw)
+	       if (redraw) {
+		  scare_mouse();
 		  object_message(d, MSG_DRAW, 0);
+		  unscare_mouse();
+	       }
 	    }
 	    _handle_listbox_click(d);
 	    while (gui_mouse_b()) {
@@ -1186,7 +1205,9 @@ int d_list_proc(int msg, DIALOG *d, int c)
 	       i = MIN(listsize-height, d->d2+delta);
 	    if (i != d->d2) {
 	       d->d2 = i;
+	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
+	       unscare_mouse(); 
 	    }
 	 }
 	 break;
@@ -1273,7 +1294,6 @@ int d_text_list_proc(int msg, DIALOG *d, int c)
    int listsize, i, a, failure;
    char *selected, *thisitem;
    char *sel = d->dp2;
-   ASSERT(d);
 
    switch (msg) {
 
@@ -1303,17 +1323,17 @@ int d_text_list_proc(int msg, DIALOG *d, int c)
 		  thisitem = (*(getfuncptr)d->dp)(i, NULL);
 		  failure = FALSE;
 
-		  if ((int)((unsigned long)d->dp3) < ustrlen(thisitem)) {
-		     for (a=0; a < (int)((unsigned long)d->dp3); a++) {
+		  if ((int)d->dp3 < ustrlen(thisitem)) {
+		     for (a=0; a<(int)d->dp3; a++) {
 			if (utolower(ugetat(thisitem, a)) != utolower(ugetat(selected, a))) {
 			   failure = TRUE;
 			   break;
 			}
 		     }
 
-		     if ((!failure) && (utolower(ugetat(thisitem, (int)(unsigned long)d->dp3)) == utolower(c))) {
+		     if ((!failure) && (utolower(ugetat(thisitem, (int)d->dp3)) == utolower(c))) {
 			d->d1 = i;
-			d->dp3 = (void *)((unsigned long)d->dp3 + 1);
+			d->dp3 = (void *)((int)d->dp3 + 1);
 
 			if (sel) {
 			   for (i=0; i<listsize; i++)
@@ -1321,7 +1341,9 @@ int d_text_list_proc(int msg, DIALOG *d, int c)
 			}
 
 			_handle_scrollable_scroll(d, listsize, &d->d1, &d->d2);
+			scare_mouse();
 			object_message(d, MSG_DRAW, 0);
+			unscare_mouse();
 			return D_USED_CHAR;
 		     }
 		  }
@@ -1370,6 +1392,7 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
    int line = 0;
    int i = 0;
    int noignore;
+   int rtm;
 
    usetc(s+usetc(s, '.'), 0);
    usetc(text+usetc(text, ' '), 0);
@@ -1388,9 +1411,11 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
    }
 
    /* choose the text color */
-   if (disabled)
+   if (disabled) 
       fg = disable;
 
+   rtm = text_mode(deselect);
+
    /* loop over the entire string */
    while (1) {
       width = 0;
@@ -1485,7 +1510,7 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
 	       case '\t':
 		  for (i=0; i<tabsize; i++) {
 		     usetc(s+usetc(s, ' '), 0);
-		     textout_ex(screen, font, s, x1, y1, fg, deselect);
+		     textout(screen, font, s, x1, y1, fg);
 		     x1 += text_length(font, s);
 		  }
 		  break;
@@ -1494,7 +1519,7 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
 	       default:
 		  if (printed != ignore) {
 		     usetc(s+usetc(s, ugetc(printed)), 0);
-		     textout_ex(screen, font, s, x1, y1, fg, deselect);
+		     textout(screen, font, s, x1, y1, fg);
 		     x1 += text_length(font, s);
 		  }
 	    }
@@ -1517,15 +1542,17 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
       /* check if we are at the end of the string */
       if (!ugetc(printed)) {
 	 /* the under blank bit */
-	 if (draw)
+	 if (draw) 
 	    rectfill(screen, x+1, y1, x+w-3, y+h-1, deselect);
 
 	 /* tell how many lines we found */
 	 *listsize = line;
+	 text_mode(rtm);
 	 return;
       }
    }
 
+   text_mode(rtm);
 }
 
 
@@ -1545,10 +1572,8 @@ int d_textbox_proc(int msg, DIALOG *d, int c)
    int height, bar, ret = D_O_K;
    int start, top, bottom, l;
    int used, delta;
-   int fg_color;
-   ASSERT(d);
+   int fg_color = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
 
-   fg_color = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
    /* calculate the actual height */
    height = (d->h-8) / text_height(font);
 
@@ -1719,7 +1744,6 @@ int d_slider_proc(int msg, DIALOG *d, int c)
    fixed slratio, slmax, slpos;
    int (*proc)(void *cbpointer, int d2value);
    int oldval;
-   ASSERT(d);
 
    /* check for slider direction */
    if (d->h < d->w)
@@ -1870,7 +1894,9 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 		  retval |= (*proc)(d->dp3, d->d2);
 	       }
 
+	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
+	       unscare_mouse();
 	    }
 	 }
 	 break;
@@ -1885,7 +1911,9 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 	       retval |= (*proc)(d->dp3, d->d2);
 	    }
 
+	    scare_mouse();
 	    object_message(d, MSG_DRAW, 0);
+	    unscare_mouse();
 	 }
 	 break;
 
@@ -1917,7 +1945,9 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 		  retval |= (*proc)(d->dp3, d->d2);
 	       }
 
+	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
+	       unscare_mouse();
 	    }
 
 	    /* let other objects continue to animate */
