@@ -43,9 +43,8 @@ BITMAP *load_lbm(AL_CONST char *filename, RGB *pal)
    int w, h, i, x, y, bpl, ppl, pbm_mode;
    char ch, cmp_type, bit_plane, color_depth;
    unsigned char uc, check_flags, bit_mask, *line_buf;
-   long data_id, len, l;
+   long id, len, l;
    int dest_depth = _color_load_depth(8, FALSE);
-   ASSERT(filename);
 
    if (!pal)
       pal = tmppal;
@@ -54,26 +53,26 @@ BITMAP *load_lbm(AL_CONST char *filename, RGB *pal)
    if (!f)
       return NULL;
 
-   data_id = pack_igetl(f);         /* read file header    */
-   if (data_id != IFF_FORM) {       /* check for 'FORM' id */
+   id = pack_igetl(f);              /* read file header    */
+   if (id != IFF_FORM) {            /* check for 'FORM' id */
       pack_fclose(f);
       return NULL;
    }
 
    pack_igetl(f);                   /* skip FORM length    */
 
-   data_id = pack_igetl(f);         /* read id             */
+   id = pack_igetl(f);              /* read id             */
 
    /* check image type ('ILBM' or 'PBM ') */
-   if ((data_id != IFF_ILBM) && (data_id != IFF_PBM)) {
+   if ((id != IFF_ILBM) && (id != IFF_PBM)) {
       pack_fclose(f);
       return NULL;
    }
 
-   pbm_mode = data_id == IFF_PBM;
+   pbm_mode = id == IFF_PBM;
 
-   data_id = pack_igetl(f);         /* read id               */
-   if (data_id != IFF_BMHD) {       /* check for header      */
+   id = pack_igetl(f);              /* read id               */
+   if (id != IFF_BMHD) {            /* check for header      */
       pack_fclose(f);
       return NULL;
    }
@@ -105,8 +104,6 @@ BITMAP *load_lbm(AL_CONST char *filename, RGB *pal)
       return NULL;
    }
 
-   *allegro_errno = 0;
-
    pack_getc(f);                    /* skip unused field        */
    pack_igetw(f);                   /* skip transparent color   */
    pack_getc(f);                    /* skip x aspect ratio      */
@@ -118,9 +115,9 @@ BITMAP *load_lbm(AL_CONST char *filename, RGB *pal)
 
    do {  /* We'll use cycle to skip possible junk      */
 	 /*  chunks: ANNO, CAMG, GRAB, DEST, TEXT etc. */
-      data_id = pack_igetl(f);
+      id = pack_igetl(f);
 
-      switch(data_id) {
+      switch(id) {
 
 	 case IFF_CMAP:
 	    memset(pal, 0, 256 * 3);
@@ -288,8 +285,21 @@ BITMAP *load_lbm(AL_CONST char *filename, RGB *pal)
       return FALSE;
    }
 
-   if (dest_depth != 8)
-      b = _fixup_loaded_bitmap(b, pal, dest_depth);
+   if (dest_depth != 8) {
+      BITMAP *b2 = b;
+
+      b = create_bitmap_ex(dest_depth, b2->w, b2->h);
+      if (!b) {
+	 destroy_bitmap(b2);
+	 return NULL;
+      }
+
+      select_palette(pal);
+      blit(b2, b, 0, 0, 0, 0, b->w, b->h);
+      unselect_palette();
+
+      destroy_bitmap(b2);
+   }
 
    return b;
 }
