@@ -62,7 +62,7 @@ int _key_accent4_flag = 0;
 
 int _key_standard_kb = TRUE;
 
-char *_keyboard_layout = NULL;
+
 
 /* lookup table for converting hardware scancodes into Allegro format */
 static unsigned char hw_to_mycode[128] =
@@ -98,7 +98,7 @@ static unsigned char hw_to_mycode[128] =
    /* 0x70 */  KEY_KANA,       0,              0,              KEY_ABNT_C1, 
    /* 0x74 */  0,              0,              0,              0,
    /* 0x78 */  0,              KEY_CONVERT,    0,              KEY_NOCONVERT, 
-   /* 0x7C */  0,              KEY_YEN,        0,              0
+   /* 0x7C */  0,              KEY_YEN,       0,              0
 };
 
 
@@ -173,8 +173,7 @@ static unsigned short standard_key_ascii_table[KEY_MAX] =
    /* misc chars */  27, '`', '-', '=', 8, 9, '[', ']', 13, ';', '\'', '\\', '\\', ',', '.', '/', ' ',
    /* controls */    0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF,
    /* numpad */      '/', '*', '-', '+', '.', 13,
-   /* others */      0, 0, 0, 0, 0, 0, 0, 0, 0, '\'', 0, 0, 0, 0, 0,
-   /* modifiers */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+   /* modifiers */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 };
 
 
@@ -190,8 +189,7 @@ static unsigned short standard_key_capslock_table[KEY_MAX] =
    /* misc chars */  27, '`', '-', '=', 8, 9, '[', ']', 13, ';', '\'', '\\', '\\', ',', '.', '/', ' ',
    /* controls */    0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF,
    /* numpad */      '/', '*', '-', '+', '.', 13,
-   /* others */      0, 0, 0, 0, 0, 0, 0, 0, 0, '\'', 0, 0, 0, 0, 0,
-   /* modifiers */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+   /* modifiers */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 };
 
 
@@ -207,8 +205,7 @@ static unsigned short standard_key_shift_table[KEY_MAX] =
    /* misc chars */  27, '~', '_', '+', 8, 9, '{', '}', 13, ':', '"', '|', '|', '<', '>', '?', ' ',
    /* controls */    0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF,
    /* numpad */      '/', '*', '-', '+', '.', 13,
-   /* others */      0, 0, 0, 0, 0, 0, 0, 0, 0, '"', 0, 0, 0, 0, 0,
-   /* modifiers */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+   /* modifiers */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 };
 
 
@@ -224,8 +221,7 @@ static unsigned short standard_key_control_table[KEY_MAX] =
    /* misc chars */  27, 2, 2, 2, 127, 127, 2, 2, 10, 2, 2, 2, 2, 2, 2, 2, 2,
    /* controls */    0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF,
    /* numpad */      2, 2, 2, 2, 2, 10,
-   /* others */      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-   /* modifiers */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+   /* modifiers */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 };
 
 
@@ -270,15 +266,10 @@ unsigned short *_key_accent4_upper_table = standard_key_empty_table;
 
 
 
-/* Table with key descriptions. */
-char *_pckeys_names[KEY_MAX];
-
-
-
 /* set_standard_keyboard:
  *  Sets up pointers ready to use the standard US keyboard mapping.
  */
-static INLINE void set_standard_keyboard(void)
+static INLINE void set_standard_keyboard()
 {
    _key_ascii_table         = standard_key_ascii_table;
    _key_capslock_table      = standard_key_capslock_table;
@@ -303,7 +294,7 @@ static INLINE void set_standard_keyboard(void)
 /* set_custom_keyboard:
  *  Sets up pointers ready to use the custom keyboard mapping.
  */
-static INLINE void set_custom_keyboard(void)
+static INLINE void set_custom_keyboard()
 {
    _key_ascii_table         = custom_key_ascii_table;
    _key_capslock_table      = custom_key_capslock_table;
@@ -345,20 +336,6 @@ int _pckey_scancode_to_ascii(int scancode)
 
 
 
-/* _pckey_scancode_to_name:
- *  Like above, but returns a static string and also works for things like
- *  modifier keys.
- */
-AL_CONST char *_pckey_scancode_to_name(int scancode)
-{
-   if ((scancode < 0) || (scancode >= KEY_MAX))
-      return NULL;
-
-   return _pckeys_names[scancode];
-}
-
-
-
 /* _handle_pckey:
  *  Handles PC keyboard input, in the same format it comes from the
  *  keyboard controller hardware (raw scancodes, top bit set means the
@@ -638,28 +615,6 @@ static void read_key_table(unsigned short *out, unsigned short *in, char *sectio
 
 
 
-/* Updates the key descriptions from the ASCII values. */
-static void update_key_descriptions(void)
-{
-   char str[64];
-   int i;
-   for (i = 0; i < KEY_MAX; i++)
-   {
-      int ascii = scancode_to_ascii (i);
-      if (_pckeys_names[i])
-            free(_pckeys_names[i]);
-      if (ascii > ' ') {
-         uszprintf(str, sizeof str, "%c", ascii);
-         _pckeys_names[i] = strdup (str);
-      }
-      else {
-         _pckeys_names[i] = strdup(_keyboard_common_names[i]);
-      }
-   }
-}
-
-
-
 /* read_keyboard_config:
  *  Reads in the keyboard config tables.
  */
@@ -668,7 +623,7 @@ static void read_keyboard_config(void)
    char filename[1024], tmp1[128], tmp2[128], *ext, *datafile;
    AL_CONST char* name;
 
-   name = get_config_string(uconvert_ascii("system", tmp1), uconvert_ascii("keyboard", tmp2), _keyboard_layout);
+   name = get_config_string(uconvert_ascii("system", tmp1), uconvert_ascii("keyboard", tmp2), NULL);
 
    if ((!name) || (!ugetc(name)))
       return;
@@ -716,8 +671,6 @@ static void read_keyboard_config(void)
    pop_config_state();
 
    set_custom_keyboard();
-
-   update_key_descriptions();
 }
 
 
@@ -726,7 +679,7 @@ static void read_keyboard_config(void)
  *  Initialises the scancode translation routines, loading mapping tables
  *  from keyboard.dat.
  */
-void _pckeys_init(void)
+void _pckeys_init()
 {
    int i;
 
@@ -763,8 +716,6 @@ void _pckeys_init(void)
    memcpy(custom_key_accent4_lower_table,    standard_key_empty_table,     sizeof(custom_key_accent2_lower_table));
    memcpy(custom_key_accent4_upper_table,    standard_key_empty_table,     sizeof(custom_key_accent2_upper_table));
 
-   update_key_descriptions();
-
    LOCK_VARIABLE(hw_to_mycode);
    LOCK_VARIABLE(hw_to_mycode_ex);
    LOCK_VARIABLE(modifier_table);
