@@ -31,20 +31,22 @@
 #include <string.h>
 
 #include "allegro.h"
-#include "allegro/aintern.h"
+#include "allegro/internal/aintern.h"
 
 #ifdef ALLEGRO_INTERNAL_HEADER
    #include ALLEGRO_INTERNAL_HEADER
 #endif
 
-#ifdef ALLEGRO_DOS
-   #include <dos.h>
-   #include <conio.h>
-#endif
+#ifndef SCAN_DEPEND
+   #ifdef ALLEGRO_DOS
+      #include <dos.h>
+      #include <conio.h>
+   #endif
 
-#ifdef ALLEGRO_DJGPP
-   #include <crt0.h>
-   #include <sys/nearptr.h>
+   #ifdef ALLEGRO_DJGPP
+      #include <crt0.h>
+      #include <sys/nearptr.h>
+   #endif
 #endif
 
 
