@@ -23,9 +23,6 @@
 #include <string.h>
 
 #include "allegro.h"
-
-#ifdef GFX_HAS_VGA
-
 #include "allegro/internal/aintern.h"
 #include "allegro/internal/aintvga.h"
 
@@ -33,6 +30,7 @@
    #include ALLEGRO_INTERNAL_HEADER
 #endif
 
+#ifdef GFX_VGA
 #if (!defined ALLEGRO_LINUX) || ((defined ALLEGRO_LINUX_VGA) && ((!defined ALLEGRO_WITH_MODULES) || (defined ALLEGRO_MODULE)))
 
 
@@ -40,7 +38,7 @@
 static BITMAP *vga_init(int w, int h, int v_w, int v_h, int color_depth);
 static void vga_exit(BITMAP *b);
 static int vga_scroll(int x, int y);
-static GFX_MODE_LIST *vga_fetch_mode_list(void);
+static GFX_MODE_LIST *vga_fetch_mode_list();
 
 
 GFX_DRIVER gfx_vga = 
@@ -61,7 +59,6 @@ GFX_DRIVER gfx_vga =
    NULL,                         /* no drawing mode hook */
    _save_vga_mode,
    _restore_vga_mode,
-   NULL,                         /* AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a)); */
    vga_fetch_mode_list,
    320, 200,
    TRUE,
@@ -73,7 +70,7 @@ GFX_DRIVER gfx_vga =
 
 
 
-static GFX_MODE vga_gfx_modes[] = {
+GFX_MODE vga_gfx_modes[] = {
    { 80,  80,  8 },
    { 160, 120, 8 },
    { 256, 256, 8 },
@@ -325,8 +322,7 @@ static int vga_scroll(int x, int y)
 
    DISABLE();
 
-   if (_wait_for_vsync)
-      _vsync_out_h();
+   _vsync_out_h();
 
    /* write to VGA address registers */
    _write_vga_register(_crtc, 0x0D, a & 0xFF);
@@ -334,8 +330,7 @@ static int vga_scroll(int x, int y)
 
    ENABLE();
 
-   if (_wait_for_vsync)
-      _vsync_in();
+   _vsync_in();
 
    return 0;
 }
@@ -345,7 +340,7 @@ static int vga_scroll(int x, int y)
 /* vga_fetch_mode_list:
  *  Creates a list of of currently implemented VGA modes.
  */
-static GFX_MODE_LIST *vga_fetch_mode_list(void)
+static GFX_MODE_LIST *vga_fetch_mode_list()
 {
    GFX_MODE_LIST *mode_list;
 
@@ -383,4 +378,4 @@ void _module_init(int system_driver)
 
 
 #endif      /* (!defined ALLEGRO_LINUX) || ((defined ALLEGRO_LINUX_VGA) && ...) */
-#endif      /* ifdef GFX_HAS_VGA */
+#endif      /* ifdef GFX_VGA */
