@@ -25,7 +25,7 @@
 
 
 
-static DIGI_DRIVER digi_none;
+extern DIGI_DRIVER digi_none;
 
 
 
@@ -135,7 +135,7 @@ int _sound_input_installed = FALSE;
 static int digi_reserve = -1;             /* how many voices to reserve */
 static int midi_reserve = -1;
 
-static VOICE virt_voice[VIRTUAL_VOICES];  /* list of active samples */
+VOICE _voice[VIRTUAL_VOICES];             /* list of active samples */
 
 PHYS_VOICE _phys_voice[DIGI_VOICES];      /* physical -> virtual voice map */
 
@@ -143,7 +143,7 @@ int _digi_volume = -1;                    /* current volume settings */
 int _midi_volume = -1;
 
 int _sound_flip_pan = FALSE;              /* reverse l/r sample panning? */
-int _sound_hq = 2;                        /* mixer speed vs. quality */
+int _sound_hq = 1;                        /* mixer speed vs. quality */
 
 int _sound_freq = -1;                     /* common hardware parameters */
 int _sound_stereo = -1;
@@ -162,7 +162,7 @@ static void sound_lock_mem(void);
 /* read_sound_config:
  *  Helper for reading the sound hardware configuration data.
  */
-static void read_sound_config(void)
+static void read_sound_config()
 {
    char tmp1[64], tmp2[64];
    char *sound = uconvert_ascii("sound", tmp1);
@@ -306,8 +306,8 @@ int install_sound(int digi, int midi, AL_CONST char *cfg_path)
       return 0;
 
    for (c=0; c<VIRTUAL_VOICES; c++) {
-      virt_voice[c].sample = NULL;
-      virt_voice[c].num = -1;
+      _voice[c].sample = NULL;
+      _voice[c].num = -1;
    }
 
    for (c=0; c<DIGI_VOICES; c++)
@@ -525,7 +525,7 @@ int install_sound(int digi, int midi, AL_CONST char *cfg_path)
       midi_driver->voices = midi_voices;
 
       for (c=0; c<midi_voices; c++) {
-	 virt_voice[midi_driver->basevoice+c].num = digi_driver->voices+c;
+	 _voice[midi_driver->basevoice+c].num = digi_driver->voices+c;
 	 _phys_voice[digi_driver->voices+c].num = midi_driver->basevoice+c;
       }
    }
@@ -540,7 +540,7 @@ int install_sound(int digi, int midi, AL_CONST char *cfg_path)
    if ((_digi_volume >= 0) || (_midi_volume >= 0))
       set_volume(_digi_volume, _midi_volume);
 
-   _add_exit_func(remove_sound, "remove_sound");
+   _add_exit_func(remove_sound);
    _sound_installed = TRUE;
    return 0;
 }
@@ -688,7 +688,7 @@ void remove_sound(void)
       remove_int(update_sweeps);
 
       for (c=0; c<VIRTUAL_VOICES; c++)
-	 if (virt_voice[c].sample)
+	 if (_voice[c].sample)
 	    deallocate_voice(c);
 
       if (_al_linker_midi)
@@ -710,7 +710,7 @@ void remove_sound(void)
 /* remove_sound_input:
  *  Sound input module cleanup routine.
  */
-void remove_sound_input(void)
+void remove_sound_input()
 {
    if (_sound_input_installed) {
       digi_input_driver->exit(TRUE);
@@ -730,57 +730,31 @@ void remove_sound_input(void)
 
 /* set_volume:
  *  Alters the global sound output volume. Specify volumes for both digital
- *  samples and MIDI playback, as integers from 0 to 255. This routine will
- *  not alter the volume of the hardware mixer if it exists.
+ *  samples and MIDI playback, as integers from 0 to 255. If possible this
+ *  routine will use a hardware mixer to control the volume, otherwise it
+ *  will tell the sample mixer and MIDI player to simulate a mixer in
+ *  software.
  */
 void set_volume(int digi_volume, int midi_volume)
-{
-   int *voice_vol;
-   int i;
-
-   if (digi_volume >= 0) {
-      voice_vol = malloc(sizeof(int)*VIRTUAL_VOICES);
-
-      /* Retrieve the (relative) volume of each voice. */
-      for (i=0; i<VIRTUAL_VOICES; i++)
-	 voice_vol[i] = voice_get_volume(i);
-
-      _digi_volume = MID(0, digi_volume, 255);
-
-      /* Set the new (relative) volume for each voice. */
-      for (i=0; i<VIRTUAL_VOICES; i++) {
-	 if (voice_vol[i] >= 0)
-	    voice_set_volume(i, voice_vol[i]);
-      }
-
-      free(voice_vol);
-   }
-
-   if (midi_volume >= 0)
-      _midi_volume = MID(0, midi_volume, 255);
-}
-
-
-
-/* set_hardware_volume:
- *  Alters the hardware sound output volume. Specify volumes for both digital
- *  samples and MIDI playback, as integers from 0 to 255. This routine will
- *  use the hardware mixer to control the volume if it exists, or do nothing.
- */
-void set_hardware_volume(int digi_volume, int midi_volume)
 {
    if (digi_volume >= 0) {
       digi_volume = MID(0, digi_volume, 255);
 
-      if (digi_driver->mixer_volume)
-	 digi_driver->mixer_volume(digi_volume);
+      if ((digi_driver->mixer_volume) && 
+	  (digi_driver->mixer_volume(digi_volume) == 0))
+	 _digi_volume = -1;
+      else
+	 _digi_volume = digi_volume;
    }
 
    if (midi_volume >= 0) {
       midi_volume = MID(0, midi_volume, 255);
 
-      if (midi_driver->mixer_volume)
-	 midi_driver->mixer_volume(midi_volume);
+      if ((midi_driver->mixer_volume) && 
+	  (midi_driver->mixer_volume(midi_volume) == 0))
+	 _midi_volume = -1;
+      else
+	 _midi_volume = midi_volume;
    }
 }
 
@@ -792,13 +766,29 @@ void set_hardware_volume(int digi_volume, int midi_volume)
  */
 void lock_sample(SAMPLE *spl)
 {
-   ASSERT(spl);
    LOCK_DATA(spl, sizeof(SAMPLE));
    LOCK_DATA(spl->data, spl->len * ((spl->bits==8) ? 1 : sizeof(short)) * ((spl->stereo) ? 2 : 1));
 }
 
 
 
+/* load_sample:
+ *  Loads a sample from disk.
+ */
+SAMPLE *load_sample(AL_CONST char *filename)
+{
+   char tmp[32];
+
+   if (ustricmp(get_extension(filename), uconvert_ascii("wav", tmp)) == 0)
+      return load_wav(filename);
+   else if (ustricmp(get_extension(filename), uconvert_ascii("voc", tmp)) == 0)
+      return load_voc(filename);
+   else
+      return NULL;
+}
+
+
+
 /* load_voc:
  *  Reads a mono VOC format sample file, returning a SAMPLE structure, 
  *  or NULL on error.
@@ -806,42 +796,17 @@ void lock_sample(SAMPLE *spl)
 SAMPLE *load_voc(AL_CONST char *filename)
 {
    PACKFILE *f;
-   SAMPLE *spl;
-   ASSERT(filename);
-
-   f = pack_fopen(filename, F_READ);
-   if (!f) 
-      return NULL;
-
-   spl = load_voc_pf(f);
-
-   pack_fclose(f);
-
-   return spl;
-}
-
-
-
-/* load_voc_pf:
- *  Reads a mono VOC format sample from the packfile given, returning a
- *  SAMPLE structure, or NULL on error. If successful the offset into the
- *  file will be left just after the sample data. If unsuccessful the offset
- *  into the file is unspecified, i.e. you must either reset the offset to
- *  some known place or close the packfile. The packfile is not closed by
- *  this function.
- */
-SAMPLE *load_voc_pf(PACKFILE *f)
-{
    char buffer[30];
    int freq = 22050;
    int bits = 8;
    SAMPLE *spl = NULL;
    int len;
    int x, ver;
-   int s;
-   ASSERT(f);
+   signed short s;
 
-   memset(buffer, 0, sizeof buffer);
+   f = pack_fopen(filename, F_READ);
+   if (!f) 
+      return NULL;
 
    pack_fread(buffer, 0x16, f);
 
@@ -875,7 +840,8 @@ SAMPLE *load_voc_pf(PACKFILE *f)
       spl = create_sample(8, FALSE, freq, len);
 
       if (spl) {
-	 if (pack_fread(spl->data, len, f) < len) {
+	 pack_fread(spl->data, len, f);
+	 if (*allegro_errno) {
 	    destroy_sample(spl);
 	    spl = NULL;
 	 }
@@ -901,26 +867,24 @@ SAMPLE *load_voc_pf(PACKFILE *f)
 
       if (spl) {
 	 if (bits == 8) {
-	    if (pack_fread(spl->data, len, f) < len) {
-	       destroy_sample(spl);
-	       spl = NULL;
-	    }
+	    pack_fread(spl->data, len, f);
 	 }
 	 else {
 	    len /= 2;
 	    for (x=0; x<len; x++) {
-	       if ((s = pack_igetw(f)) == EOF) {
-		  destroy_sample(spl);
-		  spl = NULL;
-		  break;
-	       }
-	       ((signed short *)spl->data)[x] = (signed short)s^0x8000;
+	       s = pack_igetw(f);
+	       ((signed short *)spl->data)[x] = s^0x8000;
 	    }
 	 }
+	 if (*allegro_errno) {
+	    destroy_sample(spl);
+	    spl = NULL;
+	 }
       }
    }
 
    getout: 
+   pack_fclose(f);
    return spl;
 }
 
@@ -933,54 +897,24 @@ SAMPLE *load_voc_pf(PACKFILE *f)
 SAMPLE *load_wav(AL_CONST char *filename)
 {
    PACKFILE *f;
-   SAMPLE *spl;
-   ASSERT(filename);
-
-   f = pack_fopen(filename, F_READ);
-   if (!f)
-      return NULL;
-
-   spl = load_wav_pf(f);
-
-   pack_fclose(f);
-
-   return spl;
-}
-
-
-
-/* load_wav_pf:
- *  Reads a RIFF WAV format sample from the packfile given, returning a
- *  SAMPLE structure, or NULL on error.
- *
- *  Note that the loader does not stop reading bytes from the PACKFILE until
- *  it sees the EOF. If you want to embed a WAV file into another file, you
- *  will have to implement your own chunking system that stops reading at the
- *  end of the chunk.
- *
- *  If unsuccessful the offset into the file is unspecified, i.e. you must
- *  either reset the offset to some known place or close the packfile. The
- *  packfile is not closed by this function.
- */
-SAMPLE *load_wav_pf(PACKFILE *f)
-{
    char buffer[25];
    int i;
    int length, len;
    int freq = 22050;
    int bits = 8;
    int channels = 1;
-   int s;
+   signed short s;
    SAMPLE *spl = NULL;
-   ASSERT(f);
 
-   memset(buffer, 0, sizeof buffer);
+   f = pack_fopen(filename, F_READ);
+   if (!f)
+      return NULL;
 
    pack_fread(buffer, 12, f);          /* check RIFF header */
    if (memcmp(buffer, "RIFF", 4) || memcmp(buffer+8, "WAVE", 4))
       goto getout;
 
-   while (TRUE) {
+   while (!pack_feof(f)) {
       if (pack_fread(buffer, 4, f) != 4)
 	 break;
 
@@ -1017,25 +951,23 @@ SAMPLE *load_wav_pf(PACKFILE *f)
 
 	 spl = create_sample(bits, ((channels == 2) ? TRUE : FALSE), freq, len);
 
-	 if (spl) {
+	 if (spl) { 
 	    if (bits == 8) {
-	       if (pack_fread(spl->data, length, f) < length) {
-		  destroy_sample(spl);
-		  spl = NULL;
-	       }
+	       pack_fread(spl->data, length, f);
 	    }
 	    else {
 	       for (i=0; i<len*channels; i++) {
-		  if ((s = pack_igetw(f)) == EOF) {
-		     destroy_sample(spl);
-		     spl = NULL;
-		     break;
-		  }
-		  ((signed short *)spl->data)[i] = (signed short)s^0x8000;
+		  s = pack_igetw(f);
+		  ((signed short *)spl->data)[i] = s^0x8000;
 	       }
 	    }
 
 	    length = 0;
+
+	    if (*allegro_errno) {
+	       destroy_sample(spl);
+	       spl = NULL;
+	    }
 	 }
       }
 
@@ -1048,6 +980,7 @@ SAMPLE *load_wav_pf(PACKFILE *f)
    }
 
    getout:
+   pack_fclose(f);
    return spl;
 }
 
@@ -1059,8 +992,6 @@ SAMPLE *load_wav_pf(PACKFILE *f)
 SAMPLE *create_sample(int bits, int stereo, int freq, int len)
 {
    SAMPLE *spl;
-   ASSERT(freq > 0);
-   ASSERT(len > 0);
 
    spl = malloc(sizeof(SAMPLE)); 
    if (!spl)
@@ -1081,6 +1012,8 @@ SAMPLE *create_sample(int bits, int stereo, int freq, int len)
       return NULL;
    }
 
+   memset(spl->data, 0, len * ((bits==8) ? 1 : sizeof(short)) * ((stereo) ? 2 : 1));
+
    lock_sample(spl);
    return spl;
 }
@@ -1114,7 +1047,6 @@ void destroy_sample(SAMPLE *spl)
  */
 static INLINE int absolute_freq(int freq, AL_CONST SAMPLE *spl)
 {
-   ASSERT(spl);
    if (freq == 1000)
       return spl->freq;
    else
@@ -1135,13 +1067,8 @@ static INLINE int absolute_freq(int freq, AL_CONST SAMPLE *spl)
  */
 int play_sample(AL_CONST SAMPLE *spl, int vol, int pan, int freq, int loop)
 {
-   int voice;
-   ASSERT(spl);
-   ASSERT(vol >= 0 && vol <= 255);
-   ASSERT(pan >= 0 && vol <= 255);
-   ASSERT(freq > 0);
+   int voice = allocate_voice(spl);
 
-   voice = allocate_voice(spl);
    if (voice >= 0) {
       voice_set_volume(voice, vol);
       voice_set_pan(voice, pan);
@@ -1170,10 +1097,9 @@ END_OF_FUNCTION(play_sample);
 void adjust_sample(AL_CONST SAMPLE *spl, int vol, int pan, int freq, int loop)
 {
    int c;
-   ASSERT(spl);
 
    for (c=0; c<VIRTUAL_VOICES; c++) { 
-      if (virt_voice[c].sample == spl) {
+      if (_voice[c].sample == spl) {
 	 voice_set_volume(c, vol);
 	 voice_set_pan(c, pan);
 	 voice_set_frequency(c, absolute_freq(freq, spl));
@@ -1195,10 +1121,9 @@ END_OF_FUNCTION(adjust_sample);
 void stop_sample(AL_CONST SAMPLE *spl)
 {
    int c;
-   ASSERT(spl);
 
    for (c=0; c<VIRTUAL_VOICES; c++)
-      if (virt_voice[c].sample == spl)
+      if (_voice[c].sample == spl)
 	 deallocate_voice(c);
 }
 
@@ -1225,7 +1150,7 @@ static INLINE int allocate_physical_voice(int priority)
 
    /* look for an autokill voice that has stopped */
    for (c=0; c<digi_driver->voices; c++) {
-      voice = virt_voice + _phys_voice[c].num;
+      voice = _voice + _phys_voice[c].num;
       if ((voice->autokill) && (digi_driver->get_position(c) < 0)) {
 	 digi_driver->release_voice(c);
 	 voice->sample = NULL;
@@ -1237,7 +1162,7 @@ static INLINE int allocate_physical_voice(int priority)
 
    /* ok, we're going to have to get rid of something to make room... */
    for (c=0; c<digi_driver->voices; c++) {
-      voice = virt_voice + _phys_voice[c].num;
+      voice = _voice + _phys_voice[c].num;
 
       /* sort by voice priorities */
       if (voice->priority <= priority) {
@@ -1261,7 +1186,7 @@ static INLINE int allocate_physical_voice(int priority)
       /* kill off the old voice */
       digi_driver->stop_voice(best);
       digi_driver->release_voice(best);
-      virt_voice[_phys_voice[best].num].num = -1;
+      _voice[_phys_voice[best].num].num = -1;
       _phys_voice[best].num = -1;
       return best;
    }
@@ -1276,32 +1201,32 @@ static INLINE int allocate_physical_voice(int priority)
  *  others to make room, as we allow up to 256 virtual voices to be used
  *  simultaneously.
  */
-static INLINE int allocate_virtual_voice(void)
+static INLINE int allocate_virtual_voice()
 {
-   int num_virt_voices, c;
+   int virt_voices, c;
 
-   num_virt_voices = VIRTUAL_VOICES;
+   virt_voices = VIRTUAL_VOICES;
    if (midi_driver->max_voices < 0)
-      num_virt_voices -= midi_driver->voices;
+      virt_voices -= midi_driver->voices;
 
    /* look for a free voice */
-   for (c=0; c<num_virt_voices; c++)
-      if (!virt_voice[c].sample)
+   for (c=0; c<virt_voices; c++)
+      if (!_voice[c].sample)
 	 return c;
 
    /* look for a stopped autokill voice */
-   for (c=0; c<num_virt_voices; c++) {
-      if (virt_voice[c].autokill) {
-	 if (virt_voice[c].num < 0) {
-	    virt_voice[c].sample = NULL;
+   for (c=0; c<virt_voices; c++) {
+      if (_voice[c].autokill) {
+	 if (_voice[c].num < 0) {
+	    _voice[c].sample = NULL;
 	    return c;
 	 }
 	 else {
-	    if (digi_driver->get_position(virt_voice[c].num) < 0) {
-	       digi_driver->release_voice(virt_voice[c].num);
-	       _phys_voice[virt_voice[c].num].num = -1;
-	       virt_voice[c].sample = NULL;
-	       virt_voice[c].num = -1;
+	    if (digi_driver->get_position(_voice[c].num) < 0) {
+	       digi_driver->release_voice(_voice[c].num);
+	       _phys_voice[_voice[c].num].num = -1;
+	       _voice[c].sample = NULL;
+	       _voice[c].num = -1;
 	       return c;
 	    }
 	 }
@@ -1324,18 +1249,15 @@ static INLINE int allocate_virtual_voice(void)
  */
 int allocate_voice(AL_CONST SAMPLE *spl)
 {
-   int phys, virt;
-   ASSERT(spl);
-   
-   phys = allocate_physical_voice(spl->priority);
-   virt = allocate_virtual_voice();
+   int phys = allocate_physical_voice(spl->priority);
+   int virt = allocate_virtual_voice();
 
    if (virt >= 0) {
-      virt_voice[virt].sample = spl;
-      virt_voice[virt].num = phys;
-      virt_voice[virt].autokill = FALSE;
-      virt_voice[virt].time = retrace_count;
-      virt_voice[virt].priority = spl->priority;
+      _voice[virt].sample = spl;
+      _voice[virt].num = phys;
+      _voice[virt].autokill = FALSE;
+      _voice[virt].time = retrace_count;
+      _voice[virt].priority = spl->priority;
 
       if (phys >= 0) {
 	 _phys_voice[phys].num = virt;
@@ -1363,16 +1285,14 @@ END_OF_FUNCTION(allocate_voice);
  */
 void deallocate_voice(int voice)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   
-   if (virt_voice[voice].num >= 0) {
-      digi_driver->stop_voice(virt_voice[voice].num);
-      digi_driver->release_voice(virt_voice[voice].num);
-      _phys_voice[virt_voice[voice].num].num = -1;
-      virt_voice[voice].num = -1;
+   if (_voice[voice].num >= 0) {
+      digi_driver->stop_voice(_voice[voice].num);
+      digi_driver->release_voice(_voice[voice].num);
+      _phys_voice[_voice[voice].num].num = -1;
+      _voice[voice].num = -1;
    }
 
-   virt_voice[voice].sample = NULL;
+   _voice[voice].sample = NULL;
 }
 
 END_OF_FUNCTION(deallocate_voice);
@@ -1384,21 +1304,17 @@ END_OF_FUNCTION(deallocate_voice);
  */
 void reallocate_voice(int voice, AL_CONST SAMPLE *spl)
 {
-   int phys;
-   ASSERT(spl);
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-
-   phys =  virt_voice[voice].num;
+   int phys = _voice[voice].num;
 
    if (phys >= 0) {
       digi_driver->stop_voice(phys);
       digi_driver->release_voice(phys);
    }
 
-   virt_voice[voice].sample = spl;
-   virt_voice[voice].autokill = FALSE;
-   virt_voice[voice].time = retrace_count;
-   virt_voice[voice].priority = spl->priority;
+   _voice[voice].sample = spl;
+   _voice[voice].autokill = FALSE;
+   _voice[voice].time = retrace_count;
+   _voice[voice].priority = spl->priority;
 
    if (phys >= 0) {
       _phys_voice[phys].playmode = 0;
@@ -1423,8 +1339,7 @@ END_OF_FUNCTION(reallocate_voice);
  */
 void release_voice(int voice)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   virt_voice[voice].autokill = TRUE;
+   _voice[voice].autokill = TRUE;
 }
 
 END_OF_FUNCTION(release_voice);
@@ -1436,11 +1351,10 @@ END_OF_FUNCTION(release_voice);
  */
 void voice_start(int voice)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if (virt_voice[voice].num >= 0)
-      digi_driver->start_voice(virt_voice[voice].num);
+   if (_voice[voice].num >= 0)
+      digi_driver->start_voice(_voice[voice].num);
 
-   virt_voice[voice].time = retrace_count;
+   _voice[voice].time = retrace_count;
 }
 
 END_OF_FUNCTION(voice_start);
@@ -1452,9 +1366,8 @@ END_OF_FUNCTION(voice_start);
  */
 void voice_stop(int voice)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if (virt_voice[voice].num >= 0)
-      digi_driver->stop_voice(virt_voice[voice].num);
+   if (_voice[voice].num >= 0)
+      digi_driver->stop_voice(_voice[voice].num);
 }
 
 END_OF_FUNCTION(voice_stop);
@@ -1466,9 +1379,7 @@ END_OF_FUNCTION(voice_stop);
  */
 void voice_set_priority(int voice, int priority)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   ASSERT(priority >= 0 && priority <= 255);
-   virt_voice[voice].priority = priority;
+   _voice[voice].priority = priority;
 }
 
 END_OF_FUNCTION(voice_set_priority);
@@ -1481,16 +1392,15 @@ END_OF_FUNCTION(voice_set_priority);
  */
 SAMPLE *voice_check(int voice)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if (virt_voice[voice].sample) {
-      if (virt_voice[voice].num < 0)
+   if (_voice[voice].sample) {
+      if (_voice[voice].num < 0)
 	 return NULL;
 
-      if (virt_voice[voice].autokill)
+      if (_voice[voice].autokill)
 	 if (voice_get_position(voice) < 0)
 	    return NULL;
 
-      return (SAMPLE*)virt_voice[voice].sample;
+      return (SAMPLE*)_voice[voice].sample;
    }
    else
       return NULL;
@@ -1507,9 +1417,8 @@ END_OF_FUNCTION(voice_check);
  */
 int voice_get_position(int voice)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if (virt_voice[voice].num >= 0)
-      return digi_driver->get_position(virt_voice[voice].num);
+   if (_voice[voice].num >= 0)
+      return digi_driver->get_position(_voice[voice].num);
    else
       return -1;
 }
@@ -1523,9 +1432,8 @@ END_OF_FUNCTION(voice_get_position);
  */
 void voice_set_position(int voice, int position)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if (virt_voice[voice].num >= 0)
-      digi_driver->set_position(virt_voice[voice].num, position);
+   if (_voice[voice].num >= 0)
+      digi_driver->set_position(_voice[voice].num, position);
 }
 
 END_OF_FUNCTION(voice_set_position);
@@ -1537,13 +1445,12 @@ END_OF_FUNCTION(voice_set_position);
  */
 void voice_set_playmode(int voice, int playmode)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if (virt_voice[voice].num >= 0) {
-      _phys_voice[virt_voice[voice].num].playmode = playmode;
-      digi_driver->loop_voice(virt_voice[voice].num, playmode);
+   if (_voice[voice].num >= 0) {
+      _phys_voice[_voice[voice].num].playmode = playmode;
+      digi_driver->loop_voice(_voice[voice].num, playmode);
 
       if (playmode & PLAYMODE_BACKWARD)
-	 digi_driver->set_position(virt_voice[voice].num, virt_voice[voice].sample->len-1);
+	 digi_driver->set_position(_voice[voice].num, _voice[voice].sample->len-1);
    }
 }
 
@@ -1559,10 +1466,9 @@ END_OF_FUNCTION(voice_set_playmode);
 int voice_get_volume(int voice)
 {
    int vol;
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
 
-   if (virt_voice[voice].num >= 0)
-      vol = digi_driver->get_volume(virt_voice[voice].num);
+   if (_voice[voice].num >= 0)
+      vol = digi_driver->get_volume(_voice[voice].num);
    else
       vol = -1;
 
@@ -1585,17 +1491,14 @@ END_OF_FUNCTION(voice_get_volume);
  */
 void voice_set_volume(int voice, int volume)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   ASSERT(volume >= 0 && volume <= 255);
-
    if (_digi_volume >= 0)
       volume = (volume * _digi_volume) / 255;
 
-   if (virt_voice[voice].num >= 0) {
-      _phys_voice[virt_voice[voice].num].vol = volume << 12;
-      _phys_voice[virt_voice[voice].num].dvol = 0;
+   if (_voice[voice].num >= 0) {
+      _phys_voice[_voice[voice].num].vol = volume << 12;
+      _phys_voice[_voice[voice].num].dvol = 0;
 
-      digi_driver->set_volume(virt_voice[voice].num, volume);
+      digi_driver->set_volume(_voice[voice].num, volume);
    }
 }
 
@@ -1608,22 +1511,18 @@ END_OF_FUNCTION(voice_set_volume);
  */
 void voice_ramp_volume(int voice, int time, int endvol)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   ASSERT(endvol >= 0 && endvol <= 255);
-   ASSERT(time >= 0);
-   
    if (_digi_volume >= 0)
       endvol = (endvol * _digi_volume) / 255;
 
-   if (virt_voice[voice].num >= 0) {
+   if (_voice[voice].num >= 0) {
       if (digi_driver->ramp_volume) {
-	 digi_driver->ramp_volume(virt_voice[voice].num, time, endvol);
+	 digi_driver->ramp_volume(_voice[voice].num, time, endvol);
       }
       else {
-	 int d = (endvol << 12) - _phys_voice[virt_voice[voice].num].vol;
+	 int d = (endvol << 12) - _phys_voice[_voice[voice].num].vol;
 	 time = MAX(time * SWEEP_FREQ / 1000, 1);
-	 _phys_voice[virt_voice[voice].num].target_vol = endvol << 12;
-	 _phys_voice[virt_voice[voice].num].dvol = d / time;
+	 _phys_voice[_voice[voice].num].target_vol = endvol << 12;
+	 _phys_voice[_voice[voice].num].dvol = d / time;
       }
    }
 }
@@ -1637,12 +1536,11 @@ END_OF_FUNCTION(voice_ramp_volume);
  */
 void voice_stop_volumeramp(int voice)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if (virt_voice[voice].num >= 0) {
-      _phys_voice[virt_voice[voice].num].dvol = 0;
+   if (_voice[voice].num >= 0) {
+      _phys_voice[_voice[voice].num].dvol = 0;
 
       if (digi_driver->stop_volume_ramp)
-	 digi_driver->stop_volume_ramp(virt_voice[voice].num);
+	 digi_driver->stop_volume_ramp(_voice[voice].num);
    }
 }
 
@@ -1657,9 +1555,8 @@ END_OF_FUNCTION(voice_stop_volumeramp);
  */
 int voice_get_frequency(int voice)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if (virt_voice[voice].num >= 0)
-      return digi_driver->get_frequency(virt_voice[voice].num);
+   if (_voice[voice].num >= 0)
+      return digi_driver->get_frequency(_voice[voice].num);
    else
       return -1;
 }
@@ -1673,14 +1570,11 @@ END_OF_FUNCTION(voice_get_frequency);
  */
 void voice_set_frequency(int voice, int frequency)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   ASSERT(frequency > 0);
-   
-   if (virt_voice[voice].num >= 0) {
-      _phys_voice[virt_voice[voice].num].freq = frequency << 12;
-      _phys_voice[virt_voice[voice].num].dfreq = 0;
+   if (_voice[voice].num >= 0) {
+      _phys_voice[_voice[voice].num].freq = frequency << 12;
+      _phys_voice[_voice[voice].num].dfreq = 0;
 
-      digi_driver->set_frequency(virt_voice[voice].num, frequency);
+      digi_driver->set_frequency(_voice[voice].num, frequency);
    }
 }
 
@@ -1693,19 +1587,15 @@ END_OF_FUNCTION(voice_set_frequency);
  */
 void voice_sweep_frequency(int voice, int time, int endfreq)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   ASSERT(endfreq > 0);
-   ASSERT(time >= 0);
-   
-   if (virt_voice[voice].num >= 0) {
+   if (_voice[voice].num >= 0) {
       if (digi_driver->sweep_frequency) {
-	 digi_driver->sweep_frequency(virt_voice[voice].num, time, endfreq);
+	 digi_driver->sweep_frequency(_voice[voice].num, time, endfreq);
       }
       else {
-	 int d = (endfreq << 12) - _phys_voice[virt_voice[voice].num].freq;
+	 int d = (endfreq << 12) - _phys_voice[_voice[voice].num].freq;
 	 time = MAX(time * SWEEP_FREQ / 1000, 1);
-	 _phys_voice[virt_voice[voice].num].target_freq = endfreq << 12;
-	 _phys_voice[virt_voice[voice].num].dfreq = d / time;
+	 _phys_voice[_voice[voice].num].target_freq = endfreq << 12;
+	 _phys_voice[_voice[voice].num].dfreq = d / time;
       }
    }
 }
@@ -1719,12 +1609,11 @@ END_OF_FUNCTION(voice_sweep_frequency);
  */
 void voice_stop_frequency_sweep(int voice)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if (virt_voice[voice].num >= 0) {
-      _phys_voice[virt_voice[voice].num].dfreq = 0;
+   if (_voice[voice].num >= 0) {
+      _phys_voice[_voice[voice].num].dfreq = 0;
 
       if (digi_driver->stop_frequency_sweep)
-	 digi_driver->stop_frequency_sweep(virt_voice[voice].num);
+	 digi_driver->stop_frequency_sweep(_voice[voice].num);
    }
 }
 
@@ -1740,10 +1629,9 @@ END_OF_FUNCTION(voice_stop_frequency_sweep);
 int voice_get_pan(int voice)
 {
    int pan;
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
 
-   if (virt_voice[voice].num >= 0)
-      pan = digi_driver->get_pan(virt_voice[voice].num);
+   if (_voice[voice].num >= 0)
+      pan = digi_driver->get_pan(_voice[voice].num);
    else
       pan = -1;
 
@@ -1762,16 +1650,14 @@ END_OF_FUNCTION(voice_get_pan);
  */
 void voice_set_pan(int voice, int pan)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   ASSERT(pan >= 0 && pan <= 255);
    if (_sound_flip_pan)
       pan = 255 - pan;
 
-   if (virt_voice[voice].num >= 0) {
-      _phys_voice[virt_voice[voice].num].pan = pan << 12;
-      _phys_voice[virt_voice[voice].num].dpan = 0;
+   if (_voice[voice].num >= 0) {
+      _phys_voice[_voice[voice].num].pan = pan << 12;
+      _phys_voice[_voice[voice].num].dpan = 0;
 
-      digi_driver->set_pan(virt_voice[voice].num, pan);
+      digi_driver->set_pan(_voice[voice].num, pan);
    }
 }
 
@@ -1784,22 +1670,18 @@ END_OF_FUNCTION(voice_set_pan);
  */
 void voice_sweep_pan(int voice, int time, int endpan)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   ASSERT(endpan >= 0 && endpan <= 255);
-   ASSERT(time >= 0);
-   
    if (_sound_flip_pan)
       endpan = 255 - endpan;
 
-   if (virt_voice[voice].num >= 0) {
+   if (_voice[voice].num >= 0) {
       if (digi_driver->sweep_pan) {
-	 digi_driver->sweep_pan(virt_voice[voice].num, time, endpan);
+	 digi_driver->sweep_pan(_voice[voice].num, time, endpan);
       }
       else {
-	 int d = (endpan << 12) - _phys_voice[virt_voice[voice].num].pan;
+	 int d = (endpan << 12) - _phys_voice[_voice[voice].num].pan;
 	 time = MAX(time * SWEEP_FREQ / 1000, 1);
-	 _phys_voice[virt_voice[voice].num].target_pan = endpan << 12;
-	 _phys_voice[virt_voice[voice].num].dpan = d / time;
+	 _phys_voice[_voice[voice].num].target_pan = endpan << 12;
+	 _phys_voice[_voice[voice].num].dpan = d / time;
       }
    }
 }
@@ -1813,12 +1695,11 @@ END_OF_FUNCTION(voice_sweep_pan);
  */
 void voice_stop_pan_sweep(int voice)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if (virt_voice[voice].num >= 0) {
-      _phys_voice[virt_voice[voice].num].dpan = 0;
+   if (_voice[voice].num >= 0) {
+      _phys_voice[_voice[voice].num].dpan = 0;
 
       if (digi_driver->stop_pan_sweep)
-	 digi_driver->stop_pan_sweep(virt_voice[voice].num);
+	 digi_driver->stop_pan_sweep(_voice[voice].num);
    }
 }
 
@@ -1831,9 +1712,8 @@ END_OF_FUNCTION(voice_stop_pan_sweep);
  */
 void voice_set_echo(int voice, int strength, int delay)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if ((virt_voice[voice].num >= 0) && (digi_driver->set_echo))
-      digi_driver->set_echo(virt_voice[voice].num, strength, delay);
+   if ((_voice[voice].num >= 0) && (digi_driver->set_echo))
+      digi_driver->set_echo(_voice[voice].num, strength, delay);
 }
 
 END_OF_FUNCTION(voice_set_echo);
@@ -1845,9 +1725,8 @@ END_OF_FUNCTION(voice_set_echo);
  */
 void voice_set_tremolo(int voice, int rate, int depth)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if ((virt_voice[voice].num >= 0) && (digi_driver->set_tremolo))
-      digi_driver->set_tremolo(virt_voice[voice].num, rate, depth);
+   if ((_voice[voice].num >= 0) && (digi_driver->set_tremolo))
+      digi_driver->set_tremolo(_voice[voice].num, rate, depth);
 }
 
 END_OF_FUNCTION(voice_set_tremolo);
@@ -1859,9 +1738,8 @@ END_OF_FUNCTION(voice_set_tremolo);
  */
 void voice_set_vibrato(int voice, int rate, int depth)
 {
-   ASSERT(voice >= 0 && voice < VIRTUAL_VOICES);
-   if ((virt_voice[voice].num >= 0) && (digi_driver->set_vibrato))
-      digi_driver->set_vibrato(virt_voice[voice].num, rate, depth);
+   if ((_voice[voice].num >= 0) && (digi_driver->set_vibrato))
+      digi_driver->set_vibrato(_voice[voice].num, rate, depth);
 }
 
 END_OF_FUNCTION(voice_set_vibrato);
@@ -1872,7 +1750,7 @@ END_OF_FUNCTION(voice_set_vibrato);
  *  Timer callback routine used to implement volume/frequency/pan sweep 
  *  effects, for those drivers that can't do them directly.
  */
-static void update_sweeps(void)
+static void update_sweeps()
 {
    int phys_voices, i;
 
@@ -1931,7 +1809,7 @@ END_OF_STATIC_FUNCTION(update_sweeps);
 /* get_sound_input_cap_bits:
  *  Recording capabilities: number of bits
  */
-int get_sound_input_cap_bits(void)
+int get_sound_input_cap_bits()
 {
    return digi_input_driver->rec_cap_bits;
 }
@@ -1941,7 +1819,7 @@ int get_sound_input_cap_bits(void)
 /* get_sound_input_cap_stereo:
  *  Recording capabilities: stereo
  */
-int get_sound_input_cap_stereo(void)
+int get_sound_input_cap_stereo()
 {
    return digi_input_driver->rec_cap_stereo;
 }
@@ -2003,7 +1881,7 @@ int start_sound_input(int rate, int bits, int stereo)
 /* stop_sound_input:
  *  Ends recording.
  */
-void stop_sound_input(void)
+void stop_sound_input()
 {
    if (digi_input_driver->rec_stop)
       digi_input_driver->rec_stop();
@@ -2029,7 +1907,7 @@ END_OF_FUNCTION(read_sound_input);
 /* sound_lock_mem:
  *  Locks memory used by the functions in this file.
  */
-static void sound_lock_mem(void)
+static void sound_lock_mem()
 {
    LOCK_VARIABLE(digi_none);
    LOCK_VARIABLE(_midi_none);
@@ -2039,7 +1917,7 @@ static void sound_lock_mem(void)
    LOCK_VARIABLE(midi_driver);
    LOCK_VARIABLE(digi_recorder);
    LOCK_VARIABLE(midi_recorder);
-   LOCK_VARIABLE(virt_voice);
+   LOCK_VARIABLE(_voice);
    LOCK_VARIABLE(_phys_voice);
    LOCK_VARIABLE(_digi_volume);
    LOCK_VARIABLE(_midi_volume);
