@@ -21,15 +21,6 @@
 
 
 BEGIN_DIGI_DRIVER_LIST
-#if (defined DIGI_JACK) && (!defined ALLEGRO_WITH_MODULES)
-   DIGI_DRIVER_JACK
-#endif
-#if (defined DIGI_SGIAL) && (!defined ALLEGRO_WITH_MODULES)
-   DIGI_DRIVER_SGIAL
-#endif
-#if (defined DIGI_ARTS) && (!defined ALLEGRO_WITH_MODULES)
-   DIGI_DRIVER_ARTS
-#endif
 #if (defined DIGI_ESD) && (!defined ALLEGRO_WITH_MODULES)
    DIGI_DRIVER_ESD
 #endif
