@@ -27,8 +27,8 @@
 #include "allegro/internal/aintern.h"
 #include "allegro/platform/aintunix.h"
 
-#if defined(ALLEGRO_USE_SCHED_YIELD) && defined(_POSIX_PRIORITY_SCHEDULING)
-   /* ALLEGRO_USE_SCHED_YIELD is set by configure */
+#if defined(ALLEGRO_HAVE_SCHED_YIELD) && defined(_POSIX_PRIORITY_SCHEDULING)
+   /* ALLEGRO_HAVE_SCHED_YIELD is set by configure */
    /* Manpages say systems providing sched_yield() define
     * _POSIX_PRIORITY_SCHEDULING in unistd.h
     */
@@ -37,11 +37,11 @@
    #include <sys/time.h>
 #endif
 
-#ifdef HAVE_SYS_UTSNAME_H
+#ifdef ALLEGRO_HAVE_SYS_UTSNAME_H
    #include <sys/utsname.h>
 #endif
 
-#ifdef ALLEGRO_HAVE_SV_PROCFS
+#ifdef ALLEGRO_HAVE_SV_PROCFS_H
    #include <sys/procfs.h>
    #include <sys/ioctl.h>
    #include <fcntl.h>
@@ -157,7 +157,7 @@ int _unix_find_resource(char *dest, AL_CONST char *resource, int size)
  */
 void _unix_read_os_type(void)
 {
-   #ifdef HAVE_SYS_UTSNAME_H
+   #ifdef ALLEGRO_HAVE_SYS_UTSNAME_H
 
       struct utsname utsn;
       char *tmpstr, *tmpstr2;
@@ -166,7 +166,7 @@ void _unix_read_os_type(void)
       uname(&utsn);
 
       /* fetch OS version and revision */
-      tmpstr = malloc(strlen(utsn.release)+1);
+      tmpstr = _AL_MALLOC_ATOMIC(strlen(utsn.release)+1);
       _al_sane_strncpy(tmpstr, utsn.release, strlen(utsn.release)+1);
       tmpstr2 = NULL;
 
@@ -181,7 +181,7 @@ void _unix_read_os_type(void)
       os_version = atoi(tmpstr);
       os_revision = atoi(tmpstr2);
 
-      free(tmpstr);
+      _AL_FREE(tmpstr);
 
       /* try to detect Unix systems we know of */
       if (!strcmp(utsn.sysname, "Linux")) {
@@ -196,6 +196,9 @@ void _unix_read_os_type(void)
       else if (!strcmp(utsn.sysname, "NetBSD")) {
 	 os_type = OSTYPE_NETBSD;
       }
+      else if (!strcmp(utsn.sysname, "OpenBSD")) {
+	 os_type = OSTYPE_OPENBSD;
+      }
       else if ((!strcmp(utsn.sysname, "IRIX"))
 	       || (!strcmp(utsn.sysname, "IRIX64"))) {
 	 os_type = OSTYPE_IRIX;
@@ -228,7 +231,7 @@ void _unix_read_os_type(void)
  */
 void _unix_yield_timeslice(void)
 {
-   #if defined(ALLEGRO_USE_SCHED_YIELD) && defined(_POSIX_PRIORITY_SCHEDULING)
+   #if defined(ALLEGRO_HAVE_SCHED_YIELD) && defined(_POSIX_PRIORITY_SCHEDULING)
 
       sched_yield();
 
@@ -287,7 +290,7 @@ static int _find_executable_file(const char *filename, char *output, int size)
 	 if (!end) end = strchr (start, '\0');
 
 	 /* Resize `buffer' for path component, slash, filename and a '\0' */
-	 temp = realloc (buffer, end - start + 1 + strlen (filename) + 1);
+	 temp = _AL_REALLOC (buffer, end - start + 1 + strlen (filename) + 1);
 	 if (temp) {
 	    buffer = temp;
 
@@ -297,7 +300,7 @@ static int _find_executable_file(const char *filename, char *output, int size)
 
 	    if ((stat(buffer, &finfo)==0) && (!S_ISDIR (finfo.st_mode))) {
 	       do_uconvert (buffer, U_ASCII, output, U_CURRENT, size);
-	       free (buffer);
+	       _AL_FREE (buffer);
 	       return 1;
 	    }
 	 } /* else... ignore the failure; `buffer' is still valid anyway. */
@@ -305,7 +308,7 @@ static int _find_executable_file(const char *filename, char *output, int size)
 	 start = end + 1;
       }
       /* Path search failed */
-      free (buffer);
+      _AL_FREE (buffer);
    }
    
    return 0;
@@ -316,13 +319,10 @@ static int _find_executable_file(const char *filename, char *output, int size)
  */
 void _unix_get_executable_name(char *output, int size)
 {
-   #ifdef ALLEGRO_HAVE_SV_PROCFS
+   #ifdef ALLEGRO_HAVE_SV_PROCFS_H
       struct prpsinfo psinfo;
       int fd;
    #endif
-   #if defined ALLEGRO_HAVE_SV_PROCFS || defined ALLEGRO_SYS_GETEXECNAME
-      char *s;
-   #endif
    char linkname[1024];
    char filename[1024];
    struct stat finfo;
@@ -331,7 +331,8 @@ void _unix_get_executable_name(char *output, int size)
    int len;
    
    #ifdef ALLEGRO_HAVE_GETEXECNAME
-      s = getexecname();
+   {
+      const char *s = getexecname();
       if (s) {
          if (s[0] == '/') {   /* Absolute path */
             do_uconvert (s, U_ASCII, output, U_CURRENT, size);
@@ -342,7 +343,7 @@ void _unix_get_executable_name(char *output, int size)
                return;
          }
       }
-      s = NULL;
+   }
    #endif
 
    /* We need the PID in order to query procfs */
@@ -350,7 +351,7 @@ void _unix_get_executable_name(char *output, int size)
 
    /* Try a Linux-like procfs */   
    /* get symolic link to executable from proc fs */
-   sprintf (linkname, "/proc/%d/exe", pid);
+   sprintf (linkname, "/proc/%d/exe", (int)pid);
    if (stat (linkname, &finfo) == 0) {
       len = readlink (linkname, filename, sizeof(filename)-1);
       if (len>-1) {
@@ -362,8 +363,8 @@ void _unix_get_executable_name(char *output, int size)
    }
    
    /* Use System V procfs calls if available */
-   #ifdef ALLEGRO_HAVE_SV_PROCFS
-      sprintf (linkname, "/proc/%d/exe", pid);
+   #ifdef ALLEGRO_HAVE_SV_PROCFS_H
+      sprintf (linkname, "/proc/%d/exe", (int)pid);
       fd = open(linkname, O_RDONLY);
       if (!fd == -1) {
          ioctl(fd, PIOCPSINFO, &psinfo);
@@ -385,7 +386,7 @@ void _unix_get_executable_name(char *output, int size)
 	     */
 	 
 	    /* Skip other args */
-	    s = strchr(psinfo.pr_psargs, ' ');
+	    char *s = strchr(psinfo.pr_psargs, ' ');
 	    if (s) s[0] = '\0';
 	    if (_find_executable_file(psinfo.pr_psargs, output, size))
 	       return;
@@ -399,7 +400,7 @@ void _unix_get_executable_name(char *output, int size)
    
    /* Last resort: try using the output of the ps command to at least find */
    /* the name of the file if not the full path */
-   uszprintf (linkname, sizeof(linkname), "ps -p %d", pid);
+   uszprintf (linkname, sizeof(linkname), "ps -p %d", (int)pid);
    do_uconvert (linkname, U_CURRENT, filename, U_ASCII, size);
    pipe = popen(filename, "r");
    if (pipe) {
@@ -421,7 +422,7 @@ void _unix_get_executable_name(char *output, int size)
 
       if (linkname[len] == '[' && linkname[strlen(linkname)] == ']') {
          len++;
-         linkname[strlen(linkname)] == '\0';
+         linkname[strlen(linkname)] = '\0';
       }         
       
       /* Now, the filename should be in the last column */
@@ -447,3 +448,18 @@ void _unix_get_executable_name(char *output, int size)
 
 #endif
 
+
+
+/* _unix_get_page_size:
+ *  Get size of a memory page in bytes.  If we can't do it, we make a guess.
+ */
+size_t _unix_get_page_size(void)
+{
+#if defined(ALLEGRO_HAVE_SYSCONF) && defined(_SC_PAGESIZE)
+   long page_size = sysconf(_SC_PAGESIZE); 
+#else
+   long page_size = -1;
+#endif
+
+   return (page_size == -1) ? 4096 : page_size;
+}
