@@ -22,13 +22,9 @@
 
 #include "wddraw.h"
 
-#define PREFIX_I                "al-wddmode INFO: "
-#define PREFIX_W                "al-wddmode WARNING: "
-#define PREFIX_E                "al-wddmode ERROR: "
 
-
-int _win_desktop_depth;
-RGB_MAP _win_desktop_rgb_map;  /* for 8-bit desktops */
+int desktop_depth;
+RGB_MAP desktop_rgb_map;  /* for 8-bit desktops */
 
 
 int mode_supported;
@@ -70,12 +66,11 @@ static int _wnd_width, _wnd_height, _wnd_depth, _wnd_refresh_rate, _wnd_flags;
 static int wnd_set_video_mode(void)
 {
    HRESULT hr;
-   HWND allegro_wnd = win_get_window();
 
    /* set the cooperative level to allow fullscreen access */
    hr = IDirectDraw2_SetCooperativeLevel(directdraw, allegro_wnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN);
    if (FAILED(hr)) {
-      _TRACE(PREFIX_E "SetCooperativeLevel() failed (%x)\n", hr);
+      _TRACE("SetCooperativeLevel() failed (%x)\n", hr);
       return -1;
    }
 
@@ -83,7 +78,7 @@ static int wnd_set_video_mode(void)
    hr = IDirectDraw2_SetDisplayMode(directdraw, _wnd_width, _wnd_height, _wnd_depth,
                                                 _wnd_refresh_rate, _wnd_flags);
    if (FAILED(hr)) {
-      _TRACE(PREFIX_E "SetDisplayMode(%u, %u, %u, %u, %u) failed (%x)\n", _wnd_width, _wnd_height, _wnd_depth,
+      _TRACE("SetDisplayMode(%u, %u, %u, %u, %u) failed (%x)\n", _wnd_width, _wnd_height, _wnd_depth,
                                                                  _wnd_refresh_rate, _wnd_flags, hr);
       return -1;
    }
@@ -128,10 +123,10 @@ void build_desktop_rgb_map(void)
       pal[i].b = shift_gamma(system_palette[i].peBlue);
    }
 
-   create_rgb_table(&_win_desktop_rgb_map, pal, NULL);
+   create_rgb_table(&desktop_rgb_map, pal, NULL);
 
    /* create_rgb_table() never maps RGB triplets to index 0 */
-   _win_desktop_rgb_map.data[pal[0].r>>1][pal[0].g>>1][pal[0].b>>1] = 0;
+   desktop_rgb_map.data[pal[0].r>>1][pal[0].g>>1][pal[0].b>>1] = 0;
 }
 
 
@@ -184,19 +179,19 @@ int gfx_directx_compare_color_depth(int color_depth)
    surf_desc.dwSize = sizeof(surf_desc);
    hr = IDirectDraw2_GetDisplayMode(directdraw, &surf_desc);
    if (FAILED(hr)) {
-      _TRACE(PREFIX_E "Can't get color format.\n");
+      _TRACE("Can't get color format.\n");
       return -1;
    }
 
    /* get the *real* color depth of the desktop */
-   _win_desktop_depth = surf_desc.ddpfPixelFormat.dwRGBBitCount;
-   if (_win_desktop_depth == 16) /* sure? */
-      _win_desktop_depth = get_color_bits(surf_desc.ddpfPixelFormat.dwRBitMask) +
+   desktop_depth = surf_desc.ddpfPixelFormat.dwRGBBitCount;
+   if (desktop_depth == 16) /* sure? */
+      desktop_depth = get_color_bits(surf_desc.ddpfPixelFormat.dwRBitMask) +
                       get_color_bits(surf_desc.ddpfPixelFormat.dwGBitMask) +
                       get_color_bits(surf_desc.ddpfPixelFormat.dwBBitMask);
 
-   if (color_depth == _win_desktop_depth) {
-      ddpixel_format = NULL;
+   if (color_depth == desktop_depth) {
+      dd_pixelformat = NULL;
       return 0;
    }
    else {
@@ -205,8 +200,8 @@ int gfx_directx_compare_color_depth(int color_depth)
          if ((pixel_realdepth[i] == color_depth) &&
             ((surf_desc.ddpfPixelFormat.dwRBitMask & pixel_format[i].dwRBitMask) ||
                 (surf_desc.ddpfPixelFormat.dwBBitMask & pixel_format[i].dwBBitMask) ||
-                   (_win_desktop_depth == 8) || (color_depth == 8))) {
-                      ddpixel_format = &pixel_format[i];
+                   (desktop_depth == 8) || (color_depth == 8))) {
+                      dd_pixelformat = &pixel_format[i];
                       break;
          }
       }
@@ -218,9 +213,9 @@ int gfx_directx_compare_color_depth(int color_depth)
 
 
 /* gfx_directx_update_color_format:
- *  Sets the _rgb* variables for correct color format.
+ *  Sets the _rgb variables for correct color format.
  */
-int gfx_directx_update_color_format(DDRAW_SURFACE *surf, int color_depth)
+int gfx_directx_update_color_format(LPDIRECTDRAWSURFACE2 surf, int color_depth)
 {
    DDPIXELFORMAT pixel_format;
    HRESULT hr;
@@ -228,9 +223,9 @@ int gfx_directx_update_color_format(DDRAW_SURFACE *surf, int color_depth)
 
    /* get pixel format */
    pixel_format.dwSize = sizeof(DDPIXELFORMAT);
-   hr = IDirectDrawSurface2_GetPixelFormat(surf->id, &pixel_format);
+   hr = IDirectDrawSurface2_GetPixelFormat(surf, &pixel_format);
    if (FAILED(hr)) {
-      _TRACE(PREFIX_E "Can't get color format.\n");
+      _TRACE("Can't get color format.\n");
       return -1;
    }
 
@@ -271,6 +266,18 @@ int gfx_directx_update_color_format(DDRAW_SURFACE *surf, int color_depth)
 
 
 
+/* check_mode_availability:
+ *  Callback function for graphics mode enumeration.
+ */ 
+static HRESULT CALLBACK check_mode_availability(LPDDSURFACEDESC lpDDSurfaceDesc, LPVOID unused)
+{
+   mode_supported = TRUE;
+
+   return DDENUMRET_OK;
+}
+
+
+
 /* EnumModesCallback:
  *  Callback function for enumerating the graphics modes.
  */ 
@@ -304,7 +311,7 @@ static HRESULT CALLBACK EnumModesCallback(LPDDSURFACEDESC lpDDSurfaceDesc, LPVOI
 
 /* gfx_directx_fetch_mode_list:
  *  Creates a list of available video modes.
- *  Returns the list on success, NULL on failure.
+ *  Returns number of video modes on success and -1 on failure.
  */
 GFX_MODE_LIST *gfx_directx_fetch_mode_list(void)
 {
@@ -327,7 +334,7 @@ GFX_MODE_LIST *gfx_directx_fetch_mode_list(void)
    }
 
    /* start enumeration */
-   gfx_mode_list = _AL_MALLOC(sizeof(GFX_MODE_LIST));
+   gfx_mode_list = malloc(sizeof(GFX_MODE_LIST));
    if (!gfx_mode_list)
       goto Error;
 
@@ -336,7 +343,7 @@ GFX_MODE_LIST *gfx_directx_fetch_mode_list(void)
 
    hr = IDirectDraw2_EnumDisplayModes(directdraw, enum_flags, NULL, gfx_mode_list, EnumModesCallback);
    if (FAILED(hr)) {
-      _AL_FREE(gfx_mode_list);
+      free(gfx_mode_list);
       goto Error;
    }
 
@@ -368,48 +375,77 @@ GFX_MODE_LIST *gfx_directx_fetch_mode_list(void)
  */
 int set_video_mode(int w, int h, int v_w, int v_h, int color_depth)
 {
-   HWND allegro_wnd = win_get_window();
-   _wnd_width = w;
-   _wnd_height = h;
-   _wnd_depth = (color_depth == 15 ? 16 : color_depth);
-   _wnd_refresh_rate = _refresh_rate_request;
+   DDSURFACEDESC surf_desc;
+   int flags, i;
+   HRESULT hr;
 
-   /* use VGA Mode 13h under DirectX 5 or greater */
-   if ((w == 320) && (h == 200) && (color_depth == 8) && (_dx_ver >= 0x500))
-      _wnd_flags = DDSDM_STANDARDVGAMODE;
+   surf_desc.dwSize   = sizeof(DDSURFACEDESC);
+   surf_desc.dwFlags  = DDSD_HEIGHT | DDSD_WIDTH | DDSD_PIXELFORMAT;
+   surf_desc.dwHeight = h;
+   surf_desc.dwWidth  = w;
+
+   /* enumerate VGA Mode 13h under DirectX 5 or greater */
+   if (_dx_ver >= 0x500)
+      flags = DDEDM_STANDARDVGAMODES;
    else
-      _wnd_flags = 0;
+      flags = 0;
 
-   while (TRUE) {
-      /* let the window thread do the hard work */ 
-      _TRACE(PREFIX_I "setting display mode(%u, %u, %u, %u)\n",
-             _wnd_width, _wnd_height, _wnd_depth, _wnd_refresh_rate);
+   for (i=0 ; pixel_realdepth[i] ; i++)
+      if (pixel_realdepth[i] == color_depth) {
+         surf_desc.ddpfPixelFormat = pixel_format[i];
+         mode_supported = FALSE;
 
-      if (wnd_call_proc(wnd_set_video_mode) == 0) {
-         /* check that the requested mode was properly set by the driver */
-         if (gfx_directx_compare_color_depth(color_depth) == 0)
-            goto Done;
-      }
+         /* check whether the requested mode is supported */
+         if (_refresh_rate_request) {
+            _TRACE("refresh rate requested: %d Hz\n", _refresh_rate_request);
+            surf_desc.dwFlags |= DDSD_REFRESHRATE;
+            surf_desc.dwRefreshRate = _refresh_rate_request;
+            hr = IDirectDraw2_EnumDisplayModes(directdraw, flags | DDEDM_REFRESHRATES, &surf_desc,
+                                               NULL, check_mode_availability);
+            if (FAILED(hr))
+               break;
+         }
 
-      /* try with no refresh rate request */
-      if (_wnd_refresh_rate > 0)
-         _wnd_refresh_rate = 0;
-      else
-         break;
-   }
+         if (!mode_supported) {
+            _TRACE("no refresh rate requested\n");
+            surf_desc.dwFlags &= ~DDSD_REFRESHRATE;
+            surf_desc.dwRefreshRate = 0;
+            hr = IDirectDraw2_EnumDisplayModes(directdraw, flags, &surf_desc,
+                                               NULL, check_mode_availability);
+            if (FAILED(hr))
+               break;
+         }
+
+         if (mode_supported) {
+            _wnd_width        = surf_desc.dwWidth;
+            _wnd_height       = surf_desc.dwHeight;
+            _wnd_depth        = pixel_format[i].dwRGBBitCount; /* not color_depth */
+            _wnd_refresh_rate = surf_desc.dwRefreshRate;
+
+            if ((_wnd_width == 320) && (_wnd_height == 200) && (_wnd_depth == 8) &&
+                                                               (flags & DDEDM_STANDARDVGAMODES))
+               _wnd_flags = DDSDM_STANDARDVGAMODE;
+            else
+               _wnd_flags = 0;
+
+            /* let the window thread do the hard work */ 
+            _TRACE("setting display mode(%u, %u, %u, %u)\n", w, h, color_depth, _wnd_refresh_rate);
+            if (wnd_call_proc(wnd_set_video_mode) != 0)
+               break;
+
+            /* check whether the requested mode was properly set by the driver */
+            if (gfx_directx_compare_color_depth(color_depth) == 0)
+               goto ModeSet;
+         }
+      }  /* end of 'if (pixel_realdepth[i] == color_depth)' */ 
       
-   _TRACE(PREFIX_E "Unable to set any display mode.\n");
+   _TRACE("Unable to set any display mode.\n");
    return -1;
 
- Done:
-   /* remove the window controls */
+ ModeSet:
+   /* remove window controls */
    SetWindowLong(allegro_wnd, GWL_STYLE, WS_POPUP);
-
-   /* maximize the window */
    ShowWindow(allegro_wnd, SW_MAXIMIZE);
 
-   /* put it in the foreground */
-   SetForegroundWindow(allegro_wnd);
-
    return 0;
 }
