@@ -47,10 +47,6 @@
 #error something is wrong with the makefile
 #endif
 
-#define PREFIX_I                "al-dsound INFO: "
-#define PREFIX_W                "al-dsound WARNING: "
-#define PREFIX_E                "al-dsound ERROR: "
-
 
 static int digi_directsound_detect(int input);
 static int digi_directsound_init(int input, int voices);
@@ -170,11 +166,11 @@ struct DIRECTSOUND_VOICE {
    int loop_offset;
    int loop_len;
    int looping;
-   void *lock_buf_a;
-   long lock_size_a;
+   void *lock_buf_a, *lock_buf_b;
+   long lock_size_a, lock_size_b;
+   int lock_bytes;
    LPDIRECTSOUNDBUFFER ds_buffer;
    LPDIRECTSOUNDBUFFER ds_loop_buffer;
-   LPDIRECTSOUNDBUFFER ds_locked_buffer;
 };
 
 static struct DIRECTSOUND_VOICE *ds_voices;
@@ -195,14 +191,17 @@ static LPGUID driver_guids[MAX_DRIVERS];
  */
 static BOOL CALLBACK DSEnumCallback(LPGUID lpGuid, LPCSTR lpcstrDescription, LPCSTR lpcstrModule, LPVOID lpContext)
 {
-   if (lpGuid) {
-      driver_guids[num_drivers] = lpGuid;
-      driver_names[num_drivers] = malloc(strlen(lpcstrDescription)+1);
-      if(driver_names[num_drivers])
-      {
-         _al_sane_strncpy(driver_names[num_drivers], lpcstrDescription, strlen(lpcstrDescription)+1);
-         num_drivers++;
+   if (num_drivers < MAX_DRIVERS) {
+      if (lpGuid) {
+         driver_guids[num_drivers] = malloc(sizeof(GUID));
+         memcpy(driver_guids[num_drivers], lpGuid, sizeof(GUID));
       }
+      else
+         driver_guids[num_drivers] = NULL;
+
+      driver_names[num_drivers] = malloc(strlen(lpcstrDescription)+1);
+      strcpy(driver_names[num_drivers], lpcstrDescription);
+      num_drivers++;
    }
 
    return (num_drivers < MAX_DRIVERS);
@@ -218,32 +217,29 @@ static BOOL CALLBACK DSEnumCallback(LPGUID lpGuid, LPCSTR lpcstrDescription, LPC
 _DRIVER_INFO *_get_win_digi_driver_list(void)
 {
    DIGI_DRIVER *driver;
-   HRESULT hr;
    int i;
 
    if (!driver_list) {
+      /* enumerate the DirectSound drivers */
+      DirectSoundEnumerate(DSEnumCallback, NULL);
+
       driver_list = _create_driver_list();
 
-      /* enumerate the DirectSound drivers */
-      hr = DirectSoundEnumerate(DSEnumCallback, NULL);
+      /* pure DirectSound drivers */
+      for (i=0; i<num_drivers; i++) {
+         driver = malloc(sizeof(DIGI_DRIVER));
+         memcpy(driver, &digi_directsound, sizeof(DIGI_DRIVER));
+         driver->id = DIGI_DIRECTX(i);
+         driver->ascii_name = driver_names[i];
 
-      if (hr == DS_OK) {
-         /* Allegro mixer to DirectSound drivers */
-         for (i=0; i<num_drivers; i++) {
-            driver = _get_dsalmix_driver(driver_names[i], driver_guids[i], i);
+         _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
+      }
 
-            _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
-         }
+      /* Allegro mixer to DirectSound drivers */
+      for (i=0; i<num_drivers; i++) {
+         driver = _get_dsalmix_driver(driver_names[i], driver_guids[i], i);
 
-         /* pure DirectSound drivers */
-         for (i=0; i<num_drivers; i++) {
-            driver = malloc(sizeof(DIGI_DRIVER));
-            memcpy(driver, &digi_directsound, sizeof(DIGI_DRIVER));
-            driver->id = DIGI_DIRECTX(i);
-            driver->ascii_name = driver_names[i];
-
-            _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
-         }
+         _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
       }
 
       /* Allegro mixer to WaveOut drivers */
@@ -285,44 +281,44 @@ void _free_win_digi_driver_list(void)
 #ifdef DEBUGMODE
 static char *ds_err(long err)
 {
-   static char err_str[64];
+   static char err_str[32];
 
    switch (err) {
 
       case DS_OK:
-         _al_sane_strncpy(err_str, "DS_OK", sizeof(err_str));
+         strcpy(err_str, "DS_OK");
          break;
 
       case DSERR_ALLOCATED:
-         _al_sane_strncpy(err_str, "DSERR_ALLOCATED", sizeof(err_str));
+         strcpy(err_str, "DSERR_ALLOCATED");
          break;
 
       case DSERR_BADFORMAT:
-         _al_sane_strncpy(err_str, "DSERR_BADFORMAT", sizeof(err_str));
+         strcpy(err_str, "DSERR_BADFORMAT");
          break;
 
       case DSERR_INVALIDPARAM:
-         _al_sane_strncpy(err_str, "DSERR_INVALIDPARAM", sizeof(err_str));
+         strcpy(err_str, "DSERR_INVALIDPARAM");
          break;
 
       case DSERR_NOAGGREGATION:
-         _al_sane_strncpy(err_str, "DSERR_NOAGGREGATION", sizeof(err_str));
+         strcpy(err_str, "DSERR_NOAGGREGATION");
          break;
 
       case DSERR_OUTOFMEMORY:
-         _al_sane_strncpy(err_str, "DSERR_OUTOFMEMORY", sizeof(err_str));
+         strcpy(err_str, "DSERR_OUTOFMEMORY");
          break;
 
       case DSERR_UNINITIALIZED:
-         _al_sane_strncpy(err_str, "DSERR_UNINITIALIZED", sizeof(err_str));
+         strcpy(err_str, "DSERR_UNINITIALIZED");
          break;
 
       case DSERR_UNSUPPORTED:
-         _al_sane_strncpy(err_str, "DSERR_UNSUPPORTED", sizeof(err_str));
+         strcpy(err_str, "DSERR_UNSUPPORTED");
          break;
 
       default:
-         _al_sane_strncpy(err_str, "DSERR_UNKNOWN", sizeof(err_str));
+         strcpy(err_str, "DSERR_UNKNOWN");
          break;
    }
 
@@ -351,11 +347,11 @@ static int digi_directsound_detect(int input)
       /* initialize DirectSound interface */
       hr = DirectSoundCreate(driver_guids[id], &directsound, NULL);
       if (FAILED(hr)) {
-         _TRACE(PREFIX_E "DirectSound interface creation failed during detect (%s).\n", ds_err(hr));
+         _TRACE("DirectSound interface creation failed during detect (%s).\n", ds_err(hr));
          return 0;
       }
 
-      _TRACE(PREFIX_I "DirectSound interface successfully created.\n");
+      _TRACE("DirectSound interface successfully created.\n");
 
       /* release DirectSound interface */
       IDirectSound_Release(directsound);
@@ -376,8 +372,7 @@ static int digi_directsound_init(int input, int voices)
    DSBUFFERDESC desc;
    WAVEFORMATEX format;
    int v, id;
-   HWND allegro_wnd = win_get_window();
-   
+
    /* deduce our device number from the driver ID code */
    id = ((digi_driver->id >> 8) & 0xFF) - 'A';
 
@@ -389,42 +384,41 @@ static int digi_directsound_init(int input, int voices)
    /* initialize DirectSound interface */
    hr = DirectSoundCreate(driver_guids[id], &directsound, NULL);
    if (FAILED(hr)) { 
-      _TRACE(PREFIX_E "Can't create DirectSound interface (%s).\n", ds_err(hr));
+      _TRACE("Can't create DirectSound interface (%s).\n", ds_err(hr)); 
       goto Error; 
    }
 
    /* set cooperative level */
    hr = IDirectSound_SetCooperativeLevel(directsound, allegro_wnd, DSSCL_PRIORITY);
    if (FAILED(hr))
-      _TRACE(PREFIX_W "Can't set DirectSound cooperative level (%s).\n", ds_err(hr));
+      _TRACE("Can't set DirectSound cooperative level (%s).\n", ds_err(hr));
    else
-      _TRACE(PREFIX_I "DirectSound cooperation level set to PRIORITY.\n");
+      _TRACE("DirectSound cooperation level set to PRIORITY.\n");
 
    /* get hardware capabilities */
    dscaps.dwSize = sizeof(DSCAPS);
    hr = IDirectSound_GetCaps(directsound, &dscaps);
    if (FAILED(hr)) { 
-      _TRACE(PREFIX_E "Can't get DirectSound caps (%s).\n", ds_err(hr));
+      _TRACE("Can't get DirectSound caps (%s).\n", ds_err(hr)); 
       goto Error; 
    }
 
-   if ((dscaps.dwFlags & DSCAPS_PRIMARY16BIT) &&
-       ((_sound_bits >= 16) || (_sound_bits <= 0)))
+   if (dscaps.dwFlags & DSCAPS_PRIMARY16BIT)
       _bits = 16;
    else
       _bits = 8;
 
-   if ((dscaps.dwFlags & DSCAPS_PRIMARYSTEREO) && _sound_stereo)
+   if (dscaps.dwFlags & DSCAPS_PRIMARYSTEREO)
       _stereo = 1;
    else
       _stereo = 0;
 
-   if ((dscaps.dwMaxSecondarySampleRate > (DWORD)_sound_freq) && (_sound_freq > 0))
-      _freq = _sound_freq;
+   if (dscaps.dwMaxSecondarySampleRate > 44000)
+      _freq = 44100;
    else
-      _freq = dscaps.dwMaxSecondarySampleRate;
+      _freq = 22050;
 
-   _TRACE(PREFIX_I "DirectSound caps: %u bits, %s, %uHz\n", _bits, _stereo ? "stereo" : "mono", _freq);
+   _TRACE("DirectSound caps: %u bits, %s, %uHz\n", _bits, _stereo ? "stereo" : "mono", _freq);
 
    /* create primary buffer */
    memset(&desc, 0, sizeof(DSBUFFERDESC));
@@ -433,14 +427,14 @@ static int digi_directsound_init(int input, int voices)
 
    hr = IDirectSound_CreateSoundBuffer(directsound, &desc, &prim_buf, NULL);
    if (hr != DS_OK) { 
-      _TRACE(PREFIX_W "Can't create primary buffer (%s)\nGlobal volume control won't be available.\n", ds_err(hr));
+      _TRACE("Can't create primary buffer (%s)\nGlobal volume control won't be available.\n", ds_err(hr)); 
    }
 
    /* get current format */
    if (prim_buf) {
       hr = IDirectSoundBuffer_GetFormat(prim_buf, &format, sizeof(format), NULL);
       if (FAILED(hr)) {
-         _TRACE(PREFIX_W "Can't get primary buffer format (%s).\n", ds_err(hr));
+         _TRACE("Can't get primary buffer format (%s).\n", ds_err(hr));
       }
       else {
          format.nChannels = _stereo ? 2 : 1;
@@ -451,24 +445,21 @@ static int digi_directsound_init(int input, int voices)
 
          hr = IDirectSoundBuffer_SetFormat(prim_buf, &format);
          if (FAILED(hr)) {
-            _TRACE(PREFIX_W "Can't set primary buffer format (%s).\n", ds_err(hr));
+            _TRACE("Can't set primary buffer format (%s).\n", ds_err(hr));
          }
 
          /* output primary format */
          hr = IDirectSoundBuffer_GetFormat(prim_buf, &format, sizeof(format), NULL);
          if (FAILED(hr)) {
-            _TRACE(PREFIX_W "Can't get primary buffer format (%s).\n", ds_err(hr));
+            _TRACE("Can't get primary buffer format (%s).\n", ds_err(hr));
          }
          else {
-            _TRACE(PREFIX_I "primary format:\n");
-            _TRACE(PREFIX_I "  %u channels\n  %u Hz\n  %u AvgBytesPerSec\n  %u BlockAlign\n  %u bits\n  %u size\n",
+            _TRACE("primary format:\n");
+            _TRACE("  %u channels\n  %u Hz\n  %u AvgBytesPerSec\n  %u BlockAlign\n  %u bits\n  %u size\n",
             format.nChannels, format.nSamplesPerSec, format.nAvgBytesPerSec,
             format.nBlockAlign, format.wBitsPerSample, format.cbSize);
          }
       }
-
-      /* get primary buffer (global) volume */
-      IDirectSoundBuffer_GetVolume(prim_buf, &initial_volume);
    }
 
    /* initialize physical voices */
@@ -476,7 +467,6 @@ static int digi_directsound_init(int input, int voices)
    for (v = 0; v < digi_driver->voices; v++) {
       ds_voices[v].ds_buffer = NULL;
       ds_voices[v].ds_loop_buffer = NULL;
-      ds_voices[v].ds_locked_buffer = NULL;
    }
 
    /* setup volume lookup table */
@@ -493,10 +483,13 @@ static int digi_directsound_init(int input, int voices)
    for (v = 128; v < 255; v++)
       alleg_to_dsound_pan[v] = MIN(DSBPAN_RIGHT, DSBPAN_CENTER - 2000.0*log10((255.0-v)/127.0));
 
+   /* get primary buffer (global) volume */
+   IDirectSoundBuffer_GetVolume(prim_buf, &initial_volume); 
+
    return 0;
 
  Error:
-   _TRACE(PREFIX_E "digi_directsound_init() failed.\n");
+   _TRACE("digi_directsound_init() failed.\n");
    digi_directsound_exit(FALSE);
    return -1;
 }
@@ -563,22 +556,21 @@ static int digi_directsound_mixer_volume(int volume)
 /* create_dsound_buffer:
  *  Worker function for creating a DirectSound buffer.
  */
-static LPDIRECTSOUNDBUFFER create_dsound_buffer(int len, int freq, int bits, int stereo, int vol, int pan)
+static LPDIRECTSOUNDBUFFER create_dsound_buffer(int len, int freq, int bits, int stereo)
 {
    LPDIRECTSOUNDBUFFER snd_buf;
-   WAVEFORMATEX wf;
+   PCMWAVEFORMAT pcmwf;
    DSBUFFERDESC dsbdesc;
    HRESULT hr;
-   int switch_mode;
 
    /* setup wave format structure */
-   memset(&wf, 0, sizeof(WAVEFORMATEX));
-   wf.wFormatTag = WAVE_FORMAT_PCM;
-   wf.nChannels = stereo ? 2 : 1;
-   wf.nSamplesPerSec = freq;
-   wf.wBitsPerSample = bits;
-   wf.nBlockAlign = bits * (stereo ? 2 : 1) / 8;
-   wf.nAvgBytesPerSec = wf.nSamplesPerSec * wf.nBlockAlign;
+   memset(&pcmwf, 0, sizeof(PCMWAVEFORMAT));
+   pcmwf.wf.wFormatTag = WAVE_FORMAT_PCM;
+   pcmwf.wf.nChannels = stereo ? 2 : 1;
+   pcmwf.wf.nSamplesPerSec = freq;
+   pcmwf.wBitsPerSample = bits;
+   pcmwf.wf.nBlockAlign = bits * (stereo ? 2 : 1) / 8;
+   pcmwf.wf.nAvgBytesPerSec = pcmwf.wf.nSamplesPerSec * pcmwf.wf.nBlockAlign;
 
    /* setup DSBUFFERDESC structure */
    memset(&dsbdesc, 0, sizeof(DSBUFFERDESC));
@@ -586,28 +578,25 @@ static LPDIRECTSOUNDBUFFER create_dsound_buffer(int len, int freq, int bits, int
 
    /* need default controls (pan, volume, frequency) */
    dsbdesc.dwFlags = DSBCAPS_CTRLPAN | DSBCAPS_CTRLVOLUME | DSBCAPS_CTRLFREQUENCY;
+   switch (get_display_switch_mode()) {
 
-   switch_mode = get_display_switch_mode();
-   if ((switch_mode == SWITCH_BACKAMNESIA) || (switch_mode == SWITCH_BACKGROUND))
-      dsbdesc.dwFlags |= DSBCAPS_GLOBALFOCUS;
+      case SWITCH_BACKAMNESIA:
+      case SWITCH_BACKGROUND:
+         dsbdesc.dwFlags |= DSBCAPS_STICKYFOCUS;
+         break;
+   }
 
    dsbdesc.dwBufferBytes = len * (bits / 8) * (stereo ? 2 : 1);
-   dsbdesc.lpwfxFormat = &wf;
+   dsbdesc.lpwfxFormat = (LPWAVEFORMATEX)&pcmwf;
 
    /* create buffer */
    hr = IDirectSound_CreateSoundBuffer(directsound, &dsbdesc, &snd_buf, NULL);
    if (FAILED(hr)) {
-      _TRACE(PREFIX_E "create_directsound_buffer() failed (%s).\n", ds_err(hr));
-      _TRACE(PREFIX_E " - %d Hz, %s, %d bits\n", freq, stereo ? "stereo" : "mono", bits);
+      _TRACE("create_directsound_buffer() failed (%s).\n", ds_err(hr));
+      _TRACE(" - %d Hz, %s, %d bits\n", freq, stereo ? "stereo" : "mono", bits);
       return NULL;
    }
 
-   /* set volume */
-   IDirectSoundBuffer_SetVolume(snd_buf, alleg_to_dsound_volume[MID(0, vol, 255)]);
-
-   /* set pan */
-   IDirectSoundBuffer_SetPan(snd_buf, alleg_to_dsound_pan[MID(0, pan, 255)]);
-
    return snd_buf;
 }
 
@@ -618,8 +607,8 @@ static LPDIRECTSOUNDBUFFER create_dsound_buffer(int len, int freq, int bits, int
  */
 static int fill_dsound_buffer(LPDIRECTSOUNDBUFFER snd_buf, int offset, int len, int bits, int stereo, int reversed, char *data)
 {
-   void *buf_a;
-   long int size, size_a;
+   void *buf_a, *buf_b;
+   long int size, size_a, size_b;
    HRESULT hr;
 
    /* transform from samples to bytes */
@@ -627,15 +616,15 @@ static int fill_dsound_buffer(LPDIRECTSOUNDBUFFER snd_buf, int offset, int len,
    offset = offset * (bits / 8) * (stereo ? 2 : 1);
 
    /* lock the buffer portion */
-   hr = IDirectSoundBuffer_Lock(snd_buf, offset, size, &buf_a, &size_a, NULL, NULL, 0);
+   hr = IDirectSoundBuffer_Lock(snd_buf, offset, size, &buf_a, &size_a, &buf_b, &size_b, 0);
 
    /* if buffer lost, restore and retry lock */
    if (hr == DSERR_BUFFERLOST) {
       IDirectSoundBuffer_Restore(snd_buf);
 
-      hr = IDirectSoundBuffer_Lock(snd_buf, offset, size, &buf_a, &size_a, NULL, NULL, 0);
+      hr = IDirectSoundBuffer_Lock(snd_buf, offset, size, &buf_a, &size_a, &buf_b, &size_b, 0);
       if (FAILED(hr)) {
-         _TRACE(PREFIX_E "fill_dsound_buffer() failed (%s).\n", ds_err(hr));
+         _TRACE("fill_dsound_buffer() failed (%s).\n", ds_err(hr));
          return -1;
       }
    }
@@ -696,7 +685,7 @@ static int fill_dsound_buffer(LPDIRECTSOUNDBUFFER snd_buf, int offset, int len,
    }
 
    /* unlock the buffer */
-   IDirectSoundBuffer_Unlock(snd_buf, buf_a, size_a, NULL, 0);
+   IDirectSoundBuffer_Unlock(snd_buf, buf_a, size_a, buf_b, size_b);
 
    return 0;
 }
@@ -720,16 +709,11 @@ static void digi_directsound_init_voice(int voice, AL_CONST SAMPLE *sample)
    ds_voices[voice].loop_offset = sample->loop_start;
    ds_voices[voice].loop_len = sample->loop_end - sample->loop_start;
    ds_voices[voice].looping = FALSE;
-   ds_voices[voice].lock_buf_a = NULL;
-   ds_voices[voice].lock_size_a = 0;
-   ds_voices[voice].ds_locked_buffer = NULL;
    ds_voices[voice].ds_loop_buffer = NULL;
    ds_voices[voice].ds_buffer = create_dsound_buffer(ds_voices[voice].len,
                                                      ds_voices[voice].freq,
                                                      ds_voices[voice].bits,
-                                                     ds_voices[voice].stereo,
-                                                     ds_voices[voice].vol,
-                                                     ds_voices[voice].pan);
+                                                     ds_voices[voice].stereo);
    if (!ds_voices[voice].ds_buffer)
       return;
 
@@ -748,9 +732,6 @@ static void digi_directsound_init_voice(int voice, AL_CONST SAMPLE *sample)
  */
 static void digi_directsound_release_voice(int voice)
 {
-   /* just in case */
-   digi_directsound_unlock_voice(voice);
-
    if (ds_voices[voice].ds_buffer) {
       IDirectSoundBuffer_Release(ds_voices[voice].ds_buffer);
       ds_voices[voice].ds_buffer = NULL;
@@ -816,8 +797,6 @@ static void update_voice_buffers(int voice, int reversed, int bidir, int loop)
 
       if (!loop && ds_voices[voice].ds_loop_buffer) {
          /* we don't need a loop buffer anymore, destroy it */
-         if (ds_voices[voice].ds_locked_buffer == ds_voices[voice].ds_loop_buffer)
-            digi_directsound_unlock_voice(voice);
          IDirectSoundBuffer_Release(ds_voices[voice].ds_loop_buffer);
          ds_voices[voice].ds_loop_buffer = NULL;
       }
@@ -826,22 +805,16 @@ static void update_voice_buffers(int voice, int reversed, int bidir, int loop)
          ds_voices[voice].ds_loop_buffer = create_dsound_buffer(ds_voices[voice].loop_len * (bidir ? 2 : 1),
                                                                 ds_voices[voice].freq,
                                                                 ds_voices[voice].bits,
-                                                                ds_voices[voice].stereo,
-                                                                ds_voices[voice].vol,
-                                                                ds_voices[voice].pan);
+                                                                ds_voices[voice].stereo);
          update_loop_buffer = TRUE;
       }
       else if (update_bidir) {
          /* we need to resize the loop buffer */
-         if (ds_voices[voice].ds_locked_buffer == ds_voices[voice].ds_loop_buffer)
-            digi_directsound_unlock_voice(voice);
          IDirectSoundBuffer_Release(ds_voices[voice].ds_loop_buffer);
          ds_voices[voice].ds_loop_buffer = create_dsound_buffer(ds_voices[voice].loop_len * (bidir ? 2 : 1),
                                                                 ds_voices[voice].freq,
                                                                 ds_voices[voice].bits,
-                                                                ds_voices[voice].stereo,
-                                                                ds_voices[voice].vol,
-                                                                ds_voices[voice].pan);
+                                                                ds_voices[voice].stereo);
          update_loop_buffer = TRUE;
       }
    }
@@ -910,13 +883,10 @@ static void digi_directsound_loop_voice(int voice, int playmode)
 static void *digi_directsound_lock_voice(int voice, int start, int end)
 {
    LPDIRECTSOUNDBUFFER ds_locked_buffer;
-   long size_a;
-   void *buf_a;
+   long size_a, size_b;
+   void *buf_a, *buf_b;
    HRESULT hr;
 
-   /* just in case */
-   digi_directsound_unlock_voice(voice);
-
    if (ds_voices[voice].looping && ds_voices[voice].ds_loop_buffer)
       ds_locked_buffer = ds_voices[voice].ds_loop_buffer;
    else
@@ -925,22 +895,24 @@ static void *digi_directsound_lock_voice(int voice, int start, int end)
    start = start * ds_voices[voice].bytes_per_sample;
    end = end * ds_voices[voice].bytes_per_sample;
 
-   hr = IDirectSoundBuffer_Lock(ds_locked_buffer, start, end - start, &buf_a, &size_a, NULL, NULL, 0);
+   hr = IDirectSoundBuffer_Lock(ds_locked_buffer,
+                                start, end - start, &buf_a, &size_a, &buf_b, &size_b, 0);
 
-   /* if buffer lost, restore and retry lock */
-   if (hr == DSERR_BUFFERLOST) {
-      IDirectSoundBuffer_Restore(ds_locked_buffer);
-
-      hr = IDirectSoundBuffer_Lock(ds_locked_buffer, start, end - start, &buf_a, &size_a, NULL, NULL, 0);
-      if (FAILED(hr)) {
-         _TRACE(PREFIX_E "digi_directsound_lock_voice() failed (%s).\n", ds_err(hr));
-         return NULL;
-      }
+   if (FAILED(hr)) {
+      _TRACE("digi_directsound_lock_voice() failed (%s).\n", ds_err(hr));
+      return NULL;
    }
 
-   ds_voices[voice].ds_locked_buffer = ds_locked_buffer;
    ds_voices[voice].lock_buf_a = buf_a;
    ds_voices[voice].lock_size_a = size_a;
+   ds_voices[voice].lock_buf_b = buf_b;
+   ds_voices[voice].lock_size_b = size_b;
+   ds_voices[voice].lock_bytes = end - start;
+
+   if (buf_b != NULL) {
+      digi_directsound_unlock_voice(voice);
+      return NULL;
+   }
 
    return buf_a;
 }
@@ -951,33 +923,33 @@ static void *digi_directsound_lock_voice(int voice, int start, int end)
  */
 static void digi_directsound_unlock_voice(int voice)
 {
+   LPDIRECTSOUNDBUFFER ds_locked_buffer;
    HRESULT hr;
-   int lock_bytes;
 
-   if (ds_voices[voice].ds_locked_buffer && ds_voices[voice].lock_buf_a) {
+   if (ds_voices[voice].ds_buffer && ds_voices[voice].lock_buf_a) {
+      if (ds_voices[voice].looping && ds_voices[voice].ds_loop_buffer)
+         ds_locked_buffer = ds_voices[voice].ds_loop_buffer;
+      else
+         ds_locked_buffer = ds_voices[voice].ds_buffer;
 
       if (ds_voices[voice].bits == 16) {
          unsigned short *p = (unsigned short *)ds_voices[voice].lock_buf_a;
 
-         lock_bytes = ds_voices[voice].lock_size_a >> 1;
+         ds_voices[voice].lock_bytes >>= 1;
 
-         while (lock_bytes--)
+         while (ds_voices[voice].lock_bytes--)
             *p++ ^= 0x8000;
       }
 
-      hr = IDirectSoundBuffer_Unlock(ds_voices[voice].ds_locked_buffer,
+      hr = IDirectSoundBuffer_Unlock(ds_locked_buffer,
                                      ds_voices[voice].lock_buf_a,
                                      ds_voices[voice].lock_size_a,
-                                     NULL,
-                                     0);
+                                     ds_voices[voice].lock_buf_b,
+                                     ds_voices[voice].lock_size_b);
 
       if (FAILED(hr)) {
-         _TRACE(PREFIX_E "digi_directsound_unlock_voice() failed (%s).\n", ds_err(hr));
+         _TRACE("digi_directsound_unlock_voice() failed (%s).\n", ds_err(hr));
       }
-
-      ds_voices[voice].ds_locked_buffer = NULL;
-      ds_voices[voice].lock_buf_a = NULL;
-      ds_voices[voice].lock_size_a = 0;
    }
 }
 
@@ -1002,7 +974,7 @@ static int digi_directsound_get_position(int voice)
       hr = IDirectSoundBuffer_GetCurrentPosition(ds_voices[voice].ds_loop_buffer,
                                                  &play_cursor, &write_cursor);
       if (FAILED(hr)) {
-         _TRACE(PREFIX_E "digi_directsound_get_position() failed (%s).\n", ds_err(hr));
+         _TRACE("digi_directsound_get_position() failed (%s).\n", ds_err(hr));
          return -1;
       }
 
@@ -1027,7 +999,7 @@ static int digi_directsound_get_position(int voice)
       hr = IDirectSoundBuffer_GetCurrentPosition(ds_voices[voice].ds_buffer,
                                                  &play_cursor, &write_cursor);
       if (FAILED(hr)) {
-         _TRACE(PREFIX_E "digi_directsound_get_position() failed (%s).\n", ds_err(hr));
+         _TRACE("digi_directsound_get_position() failed (%s).\n", ds_err(hr));
          return -1;
       }
 
@@ -1064,7 +1036,7 @@ static void digi_directsound_set_position(int voice, int position)
       hr = IDirectSoundBuffer_SetCurrentPosition(ds_voices[voice].ds_loop_buffer,
                                                  pos * ds_voices[voice].bytes_per_sample);
       if (FAILED(hr)) {
-         _TRACE(PREFIX_E "digi_directsound_set_position() failed (%s).\n", ds_err(hr));
+         _TRACE("digi_directsound_set_position() failed (%s).\n", ds_err(hr));
       }
    }
 
@@ -1078,7 +1050,7 @@ static void digi_directsound_set_position(int voice, int position)
       hr = IDirectSoundBuffer_SetCurrentPosition(ds_voices[voice].ds_buffer,
                                                  pos * ds_voices[voice].bytes_per_sample);
       if (FAILED(hr)) {
-         _TRACE(PREFIX_E "digi_directsound_set_position() failed (%s).\n", ds_err(hr));
+         _TRACE("digi_directsound_set_position() failed (%s).\n", ds_err(hr));
       }
    }
 }
