@@ -35,14 +35,14 @@ static int sys_directx_init(void);
 static void sys_directx_exit(void);
 static void sys_directx_get_executable_name(char *output, int size);
 static void sys_directx_set_window_title(AL_CONST char *name);
-static int sys_directx_set_close_button_callback(void (*proc)(void));
+static int sys_directx_set_window_close_button(int enable);
+static void sys_directx_set_window_close_hook(void (*proc)(void));
 static void sys_directx_message(AL_CONST char *msg);
 static void sys_directx_assert(AL_CONST char *msg);
 static void sys_directx_save_console_state(void);
 static void sys_directx_restore_console_state(void);
 static int sys_directx_desktop_color_depth(void);
 static int sys_directx_get_desktop_resolution(int *width, int *height);
-static void sys_directx_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode);
 static void sys_directx_yield_timeslice(void);
 static int sys_directx_trace_handler(AL_CONST char *msg);
 
@@ -59,7 +59,8 @@ SYSTEM_DRIVER system_directx =
    sys_directx_get_executable_name,
    NULL,                        /* AL_METHOD(int, find_resource, (char *dest, char *resource, int size)); */
    sys_directx_set_window_title,
-   sys_directx_set_close_button_callback,
+   sys_directx_set_window_close_button,
+   sys_directx_set_window_close_hook,
    sys_directx_message,
    sys_directx_assert,
    sys_directx_save_console_state,
@@ -70,18 +71,15 @@ SYSTEM_DRIVER system_directx =
    NULL,                        /* AL_METHOD(void, created_sub_bitmap, (struct BITMAP *bmp)); */
    NULL,                        /* AL_METHOD(int, destroy_bitmap, (struct BITMAP *bitmap)); */
    NULL,                        /* AL_METHOD(void, read_hardware_palette, (void)); */
-   NULL,                        /* AL_METHOD(void, set_palette_range, (PALETTE p, int from, int to, int vsync)); */
+   NULL,                        /* AL_METHOD(void, set_palette_range, (struct RGB *p, int from, int to, int vsync)); */
    NULL,                        /* AL_METHOD(struct GFX_VTABLE *, get_vtable, (int color_depth)); */
    sys_directx_set_display_switch_mode,
+   sys_directx_set_display_switch_callback,
+   sys_directx_remove_display_switch_callback,
    NULL,                        /* AL_METHOD(void, display_switch_lock, (int lock)); */
    sys_directx_desktop_color_depth,
    sys_directx_get_desktop_resolution,
-   sys_directx_get_gfx_safe_mode,
    sys_directx_yield_timeslice,
-   sys_directx_create_mutex,
-   sys_directx_destroy_mutex,
-   sys_directx_lock_mutex,
-   sys_directx_unlock_mutex,
    NULL,                        /* AL_METHOD(_DRIVER_INFO *, gfx_drivers, (void)); */
    _get_win_digi_driver_list,   /* AL_METHOD(_DRIVER_INFO *, digi_drivers, (void)); */
    _get_win_midi_driver_list,   /* AL_METHOD(_DRIVER_INFO *, midi_drivers, (void)); */
@@ -124,7 +122,7 @@ static int sys_directx_init(void)
    HANDLE current_process;
 
    /* init thread */
-   thread_init();
+   win_init_thread();
 
    allegro_inst = GetModuleHandle(NULL);
 
@@ -210,7 +208,7 @@ static void sys_directx_exit(void)
    DeleteCriticalSection(&allegro_critical_section);
 
    /* shutdown thread */
-   thread_exit();
+   win_exit_thread();
 
    allegro_inst = NULL;
 }
@@ -222,14 +220,30 @@ static void sys_directx_exit(void)
  */
 static void sys_directx_get_executable_name(char *output, int size)
 {
-   char *temp = malloc(size);
+   unsigned char *cmd = GetCommandLine();
+   int pos = 0;
+   int i = 0;
+   int q;
 
-   if (GetModuleFileName(allegro_inst, temp, size))
-      do_uconvert(temp, U_ASCII, output, U_CURRENT, size);
+   while ((cmd[i]) && (uisspace(cmd[i])))
+      i++;
+
+   if ((cmd[i] == '\'') || (cmd[i] == '"'))
+      q = cmd[i++];
    else
-      usetc(output, 0);
+      q = 0;
 
-   free(temp);
+   size -= ucwidth(0);
+
+   while ((cmd[i]) && ((q) ? (cmd[i] != q) : (!uisspace(cmd[i])))) {
+      size -= ucwidth(cmd[i]);
+      if (size < 0)
+	 break;
+
+      pos += usetc(output + pos, cmd[i++]);
+   }
+
+   usetc(output + pos, 0);
 }
 
 
@@ -245,24 +259,24 @@ static void sys_directx_set_window_title(AL_CONST char *name)
 
 
 
-/* sys_directx_set_close_button_callback:
- *  Sets the close button callback function.
+/* sys_directx_set_window_close_button:
+ *  Enables or disables the window close button.
  */
-static int sys_directx_set_close_button_callback(void (*proc)(void))
+static int sys_directx_set_window_close_button(int enable)
 {
    DWORD class_style;
    HMENU sys_menu;
 
-   user_close_proc = proc;
-
-   /* get the old class style */
+   /* we get the old class style */
    class_style = GetClassLong(allegro_wnd, GCL_STYLE);
 
    /* and the system menu handle */
    sys_menu = GetSystemMenu(allegro_wnd, FALSE);
 
-   /* enable or disable the no_close_button flag and the close menu option */
-   if (proc) {
+   /* disable or enable the no_close_button flag
+    * and the close menu option
+    */
+   if (enable) {
       class_style &= ~CS_NOCLOSE;
       EnableMenuItem(sys_menu, SC_CLOSE, MF_BYCOMMAND | MF_ENABLED);
    }
@@ -274,8 +288,8 @@ static int sys_directx_set_close_button_callback(void (*proc)(void))
    /* change the class to the new style */
    SetClassLong(allegro_wnd, GCL_STYLE, class_style);
 
-   /* Redraw the whole window to display the changes of the button.
-    * (we use this because UpdateWindow() only works for the client area)
+   /* redraw the whole window to display the changes in the button
+    * note we use this because UpdateWindow() only works for the client area
     */
    RedrawWindow(allegro_wnd, NULL, NULL, RDW_FRAME | RDW_INVALIDATE | RDW_UPDATENOW);
 
@@ -284,6 +298,16 @@ static int sys_directx_set_close_button_callback(void (*proc)(void))
 
 
 
+/* sys_directx_set_window_close_hook:
+ *  Sets the close button user hook or restores default one.
+ */
+static void sys_directx_set_window_close_hook(void (*proc)(void))
+{
+   user_close_proc = proc;
+}
+
+
+
 /* sys_directx_message:
  *  Displays a message.
  */
@@ -306,17 +330,17 @@ static void sys_directx_message(AL_CONST char *msg)
 
 
 /* sys_directx_assert
- *  Handles assertions.
+ *  handles assertions
  */
 static void sys_directx_assert(AL_CONST char *msg)
 {
-   OutputDebugString(msg);  /* thread safe */
+   OutputDebugString(msg);
 }
 
 
 
 /* sys_directx_save_console_state:
- *  Saves console window size and position.
+ *  safes window size
  */
 static void sys_directx_save_console_state(void)
 {
@@ -326,17 +350,13 @@ static void sys_directx_save_console_state(void)
 
 
 /* sys_directx_restore_console_state:
- *  Restores console window size and position.
+ *  restores old window size
  */
 static void sys_directx_restore_console_state(void)
 {
    /* unacquire input devices */
-   wnd_schedule_proc(key_dinput_unacquire);
-   wnd_schedule_proc(mouse_dinput_unacquire);
-   wnd_schedule_proc(joystick_dinput_unacquire);
-
-   /* reset switch mode */
-   sys_reset_switch_mode();
+   wnd_unacquire_keyboard();
+   wnd_unacquire_mouse();
 
    /* re-size and hide window */
    SetWindowPos(allegro_wnd, HWND_TOP, wnd_rect.left, wnd_rect.top,
@@ -348,7 +368,7 @@ static void sys_directx_restore_console_state(void)
 
 
 /* sys_directx_desktop_color_depth:
- *  Returns the current desktop color depth.
+ *  returns the current desktop color depth
  */
 static int sys_directx_desktop_color_depth(void)
 {
@@ -378,7 +398,7 @@ static int sys_directx_desktop_color_depth(void)
 
 
 /* sys_directx_get_desktop_resolution:
- *  Returns the current desktop resolution.
+ *  returns the current desktop resolution
  */
 static int sys_directx_get_desktop_resolution(int *width, int *height)
 {
@@ -409,21 +429,8 @@ static int sys_directx_get_desktop_resolution(int *width, int *height)
 
 
 
-/* sys_directx_get_gfx_safe_mode:
- *  Defines the safe graphics mode for this system.
- */
-static void sys_directx_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode)
-{
-   *driver = GFX_GDI;
-   mode->width = 320;
-   mode->height = 200;
-   mode->bpp = 8;
-}
-
-
-
 /* sys_directx_yield_timeslice:
- *  Yields remaining timeslice portion to the system.
+ *  yields remaining timeslice portion to the system
  */
 static void sys_directx_yield_timeslice(void)
 {
@@ -433,11 +440,11 @@ static void sys_directx_yield_timeslice(void)
 
 
 /* sys_directx_trace_handler
- *  Handles trace output.
+ *  handles trace output
  */
 static int sys_directx_trace_handler(AL_CONST char *msg)
 {
-   OutputDebugString(msg);  /* thread safe */
+   OutputDebugString(msg);
    return 0;
 }
 
@@ -523,7 +530,7 @@ int _WinMain(void *_main, void *hInst, void *hPrev, char *Cmd, int nShow)
 
 
 /* win_err_str:
- *  Returns a error string for a window error.
+ *  returns a error string for a window error
  */
 char *win_err_str(long err)
 {
@@ -539,17 +546,17 @@ char *win_err_str(long err)
 
 
 /* thread_safe_trace:
- *  Outputs internal trace message.
+ *  outputs trace message inside threads
  */
 void thread_safe_trace(char *msg,...)
 {
    char buf[256];
    va_list ap;
 
-   /* todo, some day: use vsnprintf (C99) */
+   /* todo: use vsnprintf() */
    va_start(ap, msg);
    vsprintf(buf, msg, ap);
    va_end(ap);
 
-   OutputDebugString(buf);  /* thread safe */
+   OutputDebugString(buf);
 }
