@@ -29,6 +29,8 @@
 #define RESYNC()     XDGASync(_xwin.display, _xwin.screen);
 
 
+extern int _xwin_keycode_pressed[];
+
 static XDGADevice *dga_device = NULL;
 static char _xdga2_driver_desc[256] = EMPTY_STRING;
 static Colormap _dga_cmap = 0;
@@ -50,9 +52,9 @@ static void _xdga2_acquire(BITMAP *bmp);
 static GFX_MODE_LIST *_xdga2_fetch_mode_list(void);
 
 #ifdef ALLEGRO_NO_ASM
-uintptr_t _xdga2_write_line(BITMAP *bmp, int line);
+unsigned long _xdga2_write_line(BITMAP *bmp, int line);
 #else
-uintptr_t _xdga2_write_line_asm(BITMAP *bmp, int line);
+unsigned long _xdga2_write_line_asm(BITMAP *bmp, int line);
 #endif
 
 
@@ -70,7 +72,6 @@ static void _xaccel_blit_to_self(BITMAP *source, BITMAP *dest, int source_x, int
 static void _xaccel_draw_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y);
 static void _xaccel_masked_blit(BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height);
 
-#define DGA_MAX_EVENTS 5
 
 
 GFX_DRIVER gfx_xdga2 =
@@ -93,7 +94,6 @@ GFX_DRIVER gfx_xdga2 =
    NULL, NULL, NULL, NULL,
    NULL,
    NULL, NULL,
-   NULL,
    _xdga2_fetch_mode_list,
    640, 480,
    TRUE,
@@ -125,7 +125,6 @@ GFX_DRIVER gfx_xdga2_soft =
    NULL, NULL, NULL, NULL,
    NULL,
    NULL, NULL,
-   NULL,
    _xdga2_fetch_mode_list,
    640, 480,
    TRUE,
@@ -140,7 +139,7 @@ GFX_DRIVER gfx_xdga2_soft =
 /* _xdga2_fetch_mode_list:
  *  Creates list of available DGA2 video modes.
  */
-static GFX_MODE_LIST *_xdga2_private_fetch_mode_list(void)
+static GFX_MODE_LIST *_xdga2_fetch_mode_list(void)
 {
    XDGAMode *mode;
    int bpp, num_modes, stored_modes, i, j, already_there;
@@ -203,17 +202,6 @@ static GFX_MODE_LIST *_xdga2_private_fetch_mode_list(void)
 
 
 
-static GFX_MODE_LIST *_xdga2_fetch_mode_list(void)
-{
-   GFX_MODE_LIST *list;
-   XLOCK ();
-   list = _xdga2_private_fetch_mode_list();
-   XUNLOCK ();
-   return list;
-}
-
-
-
 /* _xdga2_find_mode:
  *  Returns id number of specified video mode if available, 0 otherwise.
  */
@@ -224,8 +212,6 @@ static int _xdga2_find_mode(int w, int h, int vw, int vh, int depth)
    int bpp, i, found;
 
    mode = XDGAQueryModes(_xwin.display, _xwin.screen, &num_modes);
-   if (!mode)
-      return 0;
 
    /* Let's first try setting also requested refresh rate */
    for (i=0; i<num_modes; i++) {
@@ -271,11 +257,11 @@ static int _xdga2_find_mode(int w, int h, int vw, int vh, int depth)
  */
 static void _xdga2_handle_input(void)
 {
-   int i, events, events_queued;
-   static XDGAEvent event[DGA_MAX_EVENTS + 1];
+   int i, events;
+   static XDGAEvent event[5];
    XDGAEvent *cur_event;
    XKeyEvent key;
-   int dx, dy, dz = 0;
+   int kcode, scode, dx, dy, dz = 0;
    static int mouse_buttons = 0;
 
    if (_xwin.display == 0)
@@ -284,48 +270,62 @@ static void _xdga2_handle_input(void)
    XSync(_xwin.display, False);
 
    /* How much events are available in the queue.  */
-   events = events_queued = XEventsQueued(_xwin.display, QueuedAlready);
+   events = XEventsQueued(_xwin.display, QueuedAlready);
    if (events <= 0)
       return;
 
    /* Limit amount of events we read at once.  */
-   if (events > DGA_MAX_EVENTS)
-      events = DGA_MAX_EVENTS;
+   if (events > 5)
+      events = 5;
 
    /* Read pending events.  */
    for (i = 0; i < events; i++)
       XNextEvent(_xwin.display, (XEvent *)&event[i]);
 
-   /* see xwin.c */
-   if (events_queued > events && event[i - 1].type == dga_event_base+KeyRelease) {
-      XNextEvent(_xwin.display, (XEvent *)&event[i]);
-      events++;
-   }
-
    /* Process all events.  */
    for (i = 0; i < events; i++) {
-      /* see xwin.c */
-      if (event[i].type == dga_event_base+KeyRelease && (i + 1) < events) {
-         if (event[i + 1].type == dga_event_base+KeyPress) {
-            if (event[i].xkey.keycode == event[i + 1].xkey.keycode &&
-               event[i].xkey.time == event[i + 1].xkey.time)
-               continue;
-         }
-      }
-
       cur_event = &event[i];
       switch (cur_event->type - dga_event_base) {
 
          case KeyPress:
             XDGAKeyEventToXKeyEvent(&cur_event->xkey, &key);
-	    key.type -= dga_event_base;
-	    _xwin_keyboard_handler (&key, TRUE);
+
+            if (keyboard_got_focus && _xwin_keyboard_focused) {
+               int state = 0;
+
+               if (key.state & Mod5Mask)
+                  state |= KB_SCROLOCK_FLAG;
+
+               if (key.state & Mod2Mask)
+                  state |= KB_NUMLOCK_FLAG;
+
+               if (key.state & LockMask)
+                  state |= KB_CAPSLOCK_FLAG;
+
+               (*_xwin_keyboard_focused)(TRUE, state);
+               keyboard_got_focus = FALSE;
+            }
+
+            kcode = key.keycode;
+            if ((kcode >= 0) && (kcode < 256) && (!_xwin_keycode_pressed[kcode])) {
+               scode = _xwin.keycode_to_scancode[kcode];
+               if ((scode > 0) && (_xwin_keyboard_interrupt != 0)) {
+                  _xwin_keycode_pressed[kcode] = TRUE;
+                  (*_xwin_keyboard_interrupt)(1, scode);
+               }
+            }
             break;
 
          case KeyRelease:
-	    XDGAKeyEventToXKeyEvent(&cur_event->xkey, &key);
-	    key.type -= dga_event_base;
-	    _xwin_keyboard_handler (&key, TRUE);
+            XDGAKeyEventToXKeyEvent(&cur_event->xkey, &key);
+            kcode = key.keycode;
+            if ((kcode >= 0) && (kcode < 256) && _xwin_keycode_pressed[kcode]) {
+               scode = _xwin.keycode_to_scancode[kcode];
+               if ((scode > 0) && (_xwin_keyboard_interrupt != 0)) {
+                  (*_xwin_keyboard_interrupt)(0, scode);
+                  _xwin_keycode_pressed[kcode] = FALSE;
+               }
+            }
             break;
 
          case ButtonPress:
@@ -355,8 +355,8 @@ static void _xdga2_handle_input(void)
             break;
 
          case MotionNotify:
-            dx = cur_event->xmotion.dx;
-            dy = cur_event->xmotion.dy;
+            dx = event->xmotion.dx;
+            dy = event->xmotion.dy;
             if (((dx != 0) || (dy != 0)) && _xwin_mouse_interrupt) {
                (*_xwin_mouse_interrupt)(dx, dy, 0, mouse_buttons);
             }
@@ -478,7 +478,7 @@ static BITMAP *_xdga2_private_gfxdrv_init_drv(GFX_DRIVER *drv, int w, int h, int
       return NULL;
    }
    _xwin.in_dga_mode = 2;
-   _set_current_refresh_rate(dga_device->mode.verticalRefresh);
+   _current_refresh_rate = dga_device->mode.verticalRefresh;
    set_display_switch_mode(SWITCH_NONE);
 
    /* Installs DGA color map */
@@ -544,7 +544,7 @@ static BITMAP *_xdga2_private_gfxdrv_init_drv(GFX_DRIVER *drv, int w, int h, int
    /* Creates screen bitmap */
    drv->linear = TRUE;
    bmp = _make_bitmap(dga_device->mode.imageWidth, dga_device->mode.imageHeight,
-         (uintptr_t)dga_device->data, drv, depth,
+         (unsigned long)dga_device->data, drv, depth,
          dga_device->mode.bytesPerScanline);
    if (!bmp) {
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Not enough memory"));
@@ -749,10 +749,7 @@ static int _xdga2_scroll_screen(int x, int y)
    _xwin.scroll_x = x;
    _xwin.scroll_y = y;
 
-   if (_wait_for_vsync)
-      while (XDGAGetViewportStatus(_xwin.display, _xwin.screen))
-         ;
-
+   while (XDGAGetViewportStatus(_xwin.display, _xwin.screen));
    XDGASetViewport(_xwin.display, _xwin.screen, x, y, XDGAFlipRetrace);
 
    XUNLOCK();
@@ -821,12 +818,12 @@ static void _xdga2_acquire(BITMAP *bmp)
 /* _xdga2_write_line:
  *  Returns new line and synchronizes framebuffer if needed.
  */
-uintptr_t _xdga2_write_line(BITMAP *bmp, int line)
+unsigned long _xdga2_write_line(BITMAP *bmp, int line)
 {
    if (!(bmp->id & BMP_ID_LOCKED))
       _xdga2_lock(bmp);
 
-   return (uintptr_t)(bmp->line[line]);
+   return (unsigned long)(bmp->line[line]);
 }
 
 #endif
@@ -1100,8 +1097,8 @@ static void _xaccel_masked_blit(BITMAP *source, BITMAP *dest, int source_x, int
 void _module_init(int system_driver)
 {
    if (system_driver != SYSTEM_XWINDOWS) return;
-   _unix_register_gfx_driver(GFX_XDGA2_SOFT, &gfx_xdga2_soft, FALSE, FALSE);
-   _unix_register_gfx_driver(GFX_XDGA2,      &gfx_xdga2,      FALSE, FALSE);
+   _unix_register_gfx_driver(GFX_XDGA2_SOFT, &gfx_xdga2_soft, TRUE, TRUE);
+   _unix_register_gfx_driver(GFX_XDGA2,      &gfx_xdga2,      TRUE, TRUE);
 }
 
 #endif
