@@ -217,18 +217,18 @@ MMAP af_linear_mem = NOMM;
 #ifdef ALLEGRO_DOS
 
 /* hooks to let the driver call BIOS routines */
-extern void _af_int86(void), _af_call_rm(void);
+extern void _af_int86(), _af_call_rm();
 
 #else
 
 /* no BIOS on this platform, my friend! */
-void nobios(void) { }
+void nobios() { }
 
 #endif
 
 
 
-void af_exit(void)
+void af_exit()
 {
    int c;
 
@@ -322,7 +322,7 @@ int load_vbeaf_driver(char *filename)
 
 
 
-void initialise_freebeaf_extensions(void)
+void initialise_freebeaf_extensions()
 {
    typedef unsigned long (*EXT_INIT_FUNC)(AF_DRIVER *af, unsigned long id);
    EXT_INIT_FUNC ext_init;
@@ -377,7 +377,7 @@ void initialise_freebeaf_extensions(void)
 
 
 
-static int initialise_vbeaf(void)
+static int initialise_vbeaf()
 {
    int c;
 
@@ -469,7 +469,7 @@ void print_af_attributes(unsigned long attrib)
 
 
 
-int get_af_info(void)
+int get_af_info()
 {
    static char *possible_filenames[] =
    {
@@ -735,8 +735,7 @@ int main(int argc, char *argv[])
 {
    int c;
 
-   if (allegro_init() != 0)
-      return 1;
+   allegro_init();
 
    #ifdef SYSTEM_XWINDOWS
       if (system_driver->id == SYSTEM_XWINDOWS) {
@@ -779,7 +778,7 @@ int main(int argc, char *argv[])
 #else       /* ifdef VBE/AF cool on this platform */
 
 
-int main(void)
+int main()
 {
    allegro_init();
    allegro_message("Sorry, the AFINFO program only works on DOS or Linux\n");
