@@ -17,7 +17,6 @@
 
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <time.h>
 
@@ -35,25 +34,22 @@ DATAFILE datedit_info = { info_msg, DAT_INFO, sizeof(info_msg), NULL };
 
 static int file_datasize;
 
-static DATAFILE_PROPERTY *builtin_prop = NULL;
+void (*grabber_sel_palette)() = NULL;
+void (*grabber_select_property)() = NULL;
+void (*grabber_get_grid_size)() = NULL;
+void (*grabber_rebuild_list)() = NULL;
+void (*grabber_get_selection_info)() = NULL;
+int (*grabber_foreach_selection)() = NULL;
+DATAFILE *(*grabber_single_selection)() = NULL;
+void (*grabber_set_selection)() = NULL;
+void (*grabber_busy_mouse)() = NULL;
+void (*grabber_modified)() = NULL;
 
-void (*grabber_sel_palette)(PALETTE pal) = NULL;
-void (*grabber_select_property)(int type) = NULL;
-void (*grabber_get_grid_size)(int *x, int *y) = NULL;
-void (*grabber_rebuild_list)(void *old, int clear) = NULL;
-void (*grabber_get_selection_info)(DATAFILE **dat, DATAFILE ***parent) = NULL;
-int (*grabber_foreach_selection)(int (*proc)(DATAFILE *, int *, int), int *count, int *param, int param2) = NULL;
-DATAFILE *(*grabber_single_selection)(void) = NULL;
-void (*grabber_set_selection)(void *object) = NULL;
-void (*grabber_busy_mouse)(int busy) = NULL;
-void (*grabber_modified)(int modified) = NULL;
-
-char grabber_data_file[FILENAME_LENGTH] = "";
 BITMAP *grabber_graphic = NULL;
 PALETTE grabber_palette;
 
 char grabber_import_file[256] = "";
-char grabber_graphic_origin[GRABBER_GRAPHIC_ORIGIN_SIZE] = "";
+char grabber_graphic_origin[256] = "";
 char grabber_graphic_date[256] = "";
 
 
@@ -71,36 +67,10 @@ static int menu_cmp(MENU *m1, MENU *m2)
 
 
 
-/* helper function for inserting a list of builtin properties */
-static void insert_builtin_prop(AL_CONST char *prop_types)
-{
-   int type;
-
-   while (TRUE) {
-      ASSERT(strlen(prop_types)>=4 && (!prop_types[4] || prop_types[4]==';'));
-      type = DAT_ID(prop_types[0], prop_types[1], prop_types[2], prop_types[3]);
-      datedit_insert_property(&builtin_prop, type, "*");
-      if (!prop_types[4])
-         break;
-      prop_types += 5;
-   }
-}
-
-
-
-/* main cleanup routine */
-static void datedit_exit(void)
-{
-   _al_free(builtin_prop);
-}
-
-
-
 /* main initialisation routine */
 void datedit_init(void)
 {
    int done, i;
-   AL_CONST char *prop_types;
 
    #if defined ALLEGRO_DJGPP
       #include "obj/djgpp/plugins.h"
@@ -118,8 +88,6 @@ void datedit_init(void)
       #include "obj/bcc32/plugins.h"
    #elif defined ALLEGRO_MPW
       #include "::obj:mpw:plugins.h"
-   #elif defined ALLEGRO_MACOSX
-      #include "obj/macosx/plugins.h"
    #elif defined ALLEGRO_QNX
       #include "obj/qnx/plugins.h"
    #else
@@ -151,24 +119,6 @@ void datedit_init(void)
 	 }
       }
    } while (!done);
-
-   /* Build the list of builtin properties. */
-   datedit_insert_property(&builtin_prop, DAT_ORIG, "*");
-   datedit_insert_property(&builtin_prop, DAT_DATE, "*");
-
-   for (i=0; datedit_grabber_info[i]->type != DAT_END; i++) {
-      prop_types = datedit_grabber_info[i]->prop_types;
-      if (prop_types)
-	 insert_builtin_prop(prop_types);
-   }
-
-   for (i=0; (datedit_menu_info[i]) && (datedit_menu_info[i+1]); i++) {
-      prop_types = datedit_menu_info[i]->prop_types;
-      if (prop_types)
-	 insert_builtin_prop(prop_types);
-   }
-
-   atexit(datedit_exit);
 }
 
 
@@ -177,24 +127,19 @@ void datedit_init(void)
 static int export_binary(AL_CONST DATAFILE *dat, AL_CONST char *filename)
 {
    PACKFILE *f = pack_fopen(filename, F_WRITE);
-   int ret = TRUE;
 
    if (f) {
-      if (pack_fwrite(dat->dat, dat->size, f) < dat->size)
-	 ret = FALSE;
-
+      pack_fwrite(dat->dat, dat->size, f);
       pack_fclose(f);
    }
-   else
-      ret = FALSE;
 
-   return ret;
+   return (errno == 0);
 }
 
 
 
 /* grab raw binary data */
-static DATAFILE *grab_binary(int type, AL_CONST char *filename, DATAFILE_PROPERTY **prop, int depth)
+static void *grab_binary(AL_CONST char *filename, long *size, int x, int y, int w, int h, int depth)
 {
    void *mem;
    long sz = file_size(filename);
@@ -211,26 +156,24 @@ static DATAFILE *grab_binary(int type, AL_CONST char *filename, DATAFILE_PROPERT
       return NULL; 
    }
 
-   if (pack_fread(mem, sz, f) < sz) {
-      pack_fclose(f);
+   pack_fread(mem, sz, f);
+   pack_fclose(f);
+
+   if (errno) {
       free(mem);
       return NULL;
    }
 
-   pack_fclose(f);
-
-   return datedit_construct(type, mem, sz, prop);
+   *size = sz;
+   return mem;
 }
 
 
 
 /* save raw binary data */
-static int save_binary(DATAFILE *dat, AL_CONST int *fixed_prop, int pack, int pack_kids, int strip, int sort, int verbose, int extra, PACKFILE *f)
+static void save_binary(DATAFILE *dat, int packed, int packkids, int strip, int verbose, int extra, PACKFILE *f)
 {
-   if (pack_fwrite(dat->dat, dat->size, f) < dat->size)
-      return FALSE;
-
-   return TRUE;
+   pack_fwrite(dat->dat, dat->size, f);
 }
 
 
@@ -238,15 +181,7 @@ static int save_binary(DATAFILE *dat, AL_CONST int *fixed_prop, int pack, int pa
 /* export a child datafile */
 static int export_datafile(AL_CONST DATAFILE *dat, AL_CONST char *filename)
 {
-   DATEDIT_SAVE_DATAFILE_OPTIONS options = {-1,    /* pack      */
-					    -1,    /* strip     */
-					    -1,    /* sort      */
-					    FALSE, /* verbose   */
-					    FALSE, /* write_msg */
-					    FALSE, /* backup    */
-					    FALSE  /* rel. path */ };
-
-   return datedit_save_datafile((DATAFILE *)dat->dat, filename, NULL, &options, NULL);
+   return datedit_save_datafile((DATAFILE *)dat->dat, filename, -1, -1, FALSE, FALSE, FALSE, NULL);
 }
 
 
@@ -291,10 +226,6 @@ static void load_header(DATAFILE *dat, AL_CONST char *filename)
 
       pack_fclose(f);
    }
-   else {
-      /* don't let the error propagate */
-      errno = 0;
-   }
 }
 
 
@@ -373,7 +304,7 @@ static DATAFILE *extract_info(DATAFILE *dat, int save)
 
 
 /* grabs a child datafile */
-static DATAFILE *grab_datafile(int type, AL_CONST char *filename, DATAFILE_PROPERTY **prop, int depth)
+static void *grab_datafile(AL_CONST char *filename, long *size, int x, int y, int w, int h, int depth)
 {
    DATAFILE *dat = load_datafile(filename);
 
@@ -381,31 +312,32 @@ static DATAFILE *grab_datafile(int type, AL_CONST char *filename, DATAFILE_PROPE
       load_header(dat, datedit_pretty_name(filename, "h", TRUE));
       generate_names(dat, 0);
       dat = extract_info(dat, FALSE);
-      return datedit_construct(type, dat, 0, prop);
    }
 
-   return NULL;
+   return dat;
 }
 
 
 
 /* queries whether this property belongs in the current strip mode */
-static int should_save_prop(int type, AL_CONST int *fixed_prop, int strip)
+static int should_save_prop(int type, int strip)
 {
-   if (strip == 0)
+   if (strip == 0) {
       return TRUE;
-
-   if (fixed_prop) {
-      while (*fixed_prop) {
-	 if (type == *fixed_prop++)
-	    return TRUE;
-      }
    }
-
-   if (strip >= 2)
+   else if (strip >= 2) {
       return FALSE;
-   else
-      return (datedit_get_property(&builtin_prop, type) == empty_string);  /* not builtin */
+   }
+   else {
+      return ((type != DAT_ORIG) &&
+	      (type != DAT_DATE) && 
+	      (type != DAT_XPOS) && 
+	      (type != DAT_YPOS) &&
+	      (type != DAT_XSIZ) && 
+	      (type != DAT_YSIZ) &&
+              (type != DAT_XCRP) &&
+              (type != DAT_YCRP));
+   }
 }
 
 
@@ -422,33 +354,34 @@ static int percent(int a, int b)
 
 
 /* saves an object */
-static int save_object(DATAFILE *dat, AL_CONST int *fixed_prop, int pack, int pack_kids, int strip, int sort, int verbose, PACKFILE *f)
+static void save_object(DATAFILE *dat, int packed, int packkids, int strip, int verbose, PACKFILE *f)
 {
-   int i, ret;
+   int i;
    DATAFILE_PROPERTY *prop;
-   int (*save)(DATAFILE *, AL_CONST int *, int, int, int, int, int, int, PACKFILE *);
+   void (*save)(DATAFILE *, int, int, int, int, int, PACKFILE *);
 
    prop = dat->prop;
    datedit_sort_properties(prop);
 
    while ((prop) && (prop->type != DAT_END)) {
-      if (should_save_prop(prop->type, fixed_prop, strip)) {
+      if (should_save_prop(prop->type, strip)) {
 	 pack_mputl(DAT_PROPERTY, f);
 	 pack_mputl(prop->type, f);
 	 pack_mputl(strlen(prop->dat), f);
-	 if (pack_fwrite(prop->dat, strlen(prop->dat), f) < (signed)strlen(prop->dat))
-	    return FALSE;
+	 pack_fwrite(prop->dat, strlen(prop->dat), f);
 	 file_datasize += 12 + strlen(prop->dat);
       }
 
       prop++;
+      if (errno)
+	 return;
    }
 
    if (verbose)
       datedit_startmsg("%-28s", get_datafile_property(dat, DAT_NAME));
 
    pack_mputl(dat->type, f);
-   f = pack_fopen_chunk(f, ((!pack) && (pack_kids) && (dat->type != DAT_FILE)));
+   f = pack_fopen_chunk(f, ((!packed) && (packkids) && (dat->type != DAT_FILE)));
    file_datasize += 12;
 
    save = NULL;
@@ -467,18 +400,18 @@ static int save_object(DATAFILE *dat, AL_CONST int *fixed_prop, int pack, int pa
       if (verbose)
 	 datedit_endmsg("");
 
-      ret = save((DATAFILE *)dat->dat, fixed_prop, pack, pack_kids, strip, sort, verbose, FALSE, f);
+      save((DATAFILE *)dat->dat, packed, packkids, strip, verbose, FALSE, f);
 
       if (verbose)
 	 datedit_startmsg("End of %-21s", get_datafile_property(dat, DAT_NAME));
    }
    else
-      ret = save(dat, fixed_prop, (pack || pack_kids), FALSE, strip, sort, verbose, FALSE, f);
+      save(dat, (packed || packkids), FALSE, strip, verbose, FALSE, f);
 
    pack_fclose_chunk(f);
 
    if (verbose) {
-      if ((!pack) && (pack_kids) && (dat->type != DAT_FILE)) {
+      if ((!packed) && (packkids) && (dat->type != DAT_FILE)) {
 	 datedit_endmsg("%7d bytes into %-7d (%d%%)", 
 			_packfile_datasize, _packfile_filesize, 
 			percent(_packfile_datasize, _packfile_filesize));
@@ -491,19 +424,16 @@ static int save_object(DATAFILE *dat, AL_CONST int *fixed_prop, int pack, int pa
       file_datasize += 4;
    else
       file_datasize += _packfile_datasize;
-
-   return ret;
 }
 
 
 
 /* saves a datafile */
-static int save_datafile(DATAFILE *dat, AL_CONST int *fixed_prop, int pack, int pack_kids, int strip, int sort, int verbose, int extra, PACKFILE *f)
+static void save_datafile(DATAFILE *dat, int packed, int packkids, int strip, int verbose, int extra, PACKFILE *f)
 {
    int c, size;
 
-   if (sort)
-      datedit_sort_datafile(dat);
+   datedit_sort_datafile(dat);
 
    size = 0;
    while (dat[size].type != DAT_END)
@@ -512,11 +442,11 @@ static int save_datafile(DATAFILE *dat, AL_CONST int *fixed_prop, int pack, int
    pack_mputl(extra ? size+1 : size, f);
 
    for (c=0; c<size; c++) {
-      if (!save_object(dat+c, fixed_prop, pack, pack_kids, strip, sort, verbose, f))
-	 return FALSE;
-   }
+      save_object(dat+c, packed, packkids, strip, verbose, f);
 
-   return TRUE;
+      if (errno)
+	 return;
+   }
 }
 
 
@@ -557,8 +487,7 @@ DATEDIT_GRABBER_INFO datfile_grabber =
    "dat",
    "dat",
    grab_datafile,
-   export_datafile,
-   NULL
+   export_datafile
 };
 
 
@@ -584,7 +513,6 @@ DATEDIT_GRABBER_INFO datend_grabber =
    NULL,
    NULL,
    NULL,
-   NULL,
    NULL
 };
 
@@ -841,91 +769,53 @@ int datedit_clean_typename(AL_CONST char *type)
 
 
 
-/* returns the property string for the specified property (if any) */
-AL_CONST char *datedit_get_property(DATAFILE_PROPERTY **prop, int type)
+/* sets an object property string */
+void datedit_set_property(DATAFILE *dat, int type, AL_CONST char *value)
 {
-   DATAFILE_PROPERTY *iter = *prop;
+   int i, size, pos;
 
-   if (iter) {
-      while (iter->type != DAT_END) {
-	 if (iter->type == type)
-	    return (iter->dat ? iter->dat : empty_string);
-
-	 iter++;
-      }
-   }
-
-   return empty_string;
-}
-
-
-
-/* insert a new property into a list without duplication */
-void datedit_insert_property(DATAFILE_PROPERTY **prop, int type, AL_CONST char *value)
-{
-   DATAFILE_PROPERTY *iter = *prop, *the_prop;
-   int size;
-
-   if (iter) {
-      size = 0;
-      the_prop = NULL;
-
-      for (; iter->type != DAT_END; iter++) {
-         size++;
-	 if (iter->type == type)
-	    the_prop = iter;
-	    /* no break, we need the full size of the array */
-      }
+   if (dat->prop) {
+      pos = -1;
+      for (size=0; dat->prop[size].type != DAT_END; size++)
+	 if (dat->prop[size].type == type)
+	    pos = size;
 
       if ((value) && (strlen(value) > 0)) {
-	 if (the_prop) {
-	    the_prop->dat = _al_realloc(the_prop->dat, strlen(value)+1);
-	    strcpy(the_prop->dat, value);
+	 if (pos >= 0) {
+	    dat->prop[pos].dat = _al_realloc(dat->prop[pos].dat, strlen(value)+1);
+	    strcpy(dat->prop[pos].dat, value);
 	 }
 	 else {
-	    *prop = _al_realloc(*prop, sizeof(DATAFILE_PROPERTY)*(size+2));
-	    the_prop = *prop + size;
-	    the_prop->type = type;
-	    the_prop->dat = _al_malloc(strlen(value)+1);
-	    strcpy(the_prop->dat, value);
-	    the_prop++;
-	    the_prop->type = DAT_END;
-	    the_prop->dat = NULL;
+	    dat->prop = _al_realloc(dat->prop, sizeof(DATAFILE_PROPERTY)*(size+2));
+	    dat->prop[size+1] = dat->prop[size];
+	    dat->prop[size].type = type;
+	    dat->prop[size].dat = _al_malloc(strlen(value)+1);
+	    strcpy(dat->prop[size].dat, value);
 	 }
       }
       else {
-	 if (the_prop) {
-	    _al_free(the_prop->dat);
-	    for (iter = the_prop; iter->type != DAT_END; iter++)
-	       *iter = *(iter+1);
-	    *prop = _al_realloc(*prop, sizeof(DATAFILE_PROPERTY)*size);
+	 if (pos >= 0) {
+	    _al_free(dat->prop[pos].dat);
+	    for (i=pos; i<size; i++)
+	       dat->prop[i] = dat->prop[i+1];
+	    dat->prop = _al_realloc(dat->prop, sizeof(DATAFILE_PROPERTY)*size);
 	 }
       }
    }
    else {
       if ((value) && (strlen(value) > 0)) {
-	 *prop = _al_malloc(sizeof(DATAFILE_PROPERTY)*2);
-	 the_prop = *prop;
-	 the_prop->type = type;
-	 the_prop->dat = _al_malloc(strlen(value)+1);
-	 strcpy(the_prop->dat, value);
-	 the_prop++;
-	 the_prop->type = DAT_END;
-	 the_prop->dat = NULL;
+	 dat->prop = _al_malloc(sizeof(DATAFILE_PROPERTY) * 2);
+	 dat->prop[0].type = type;
+	 dat->prop[0].dat = _al_malloc(strlen(value)+1);
+	 strcpy(dat->prop[0].dat, value);
+	 dat->prop[1].type = DAT_END;
+	 dat->prop[1].dat = NULL;
       }
    }
 }
 
 
 
-/* sets an object property string */
-void datedit_set_property(DATAFILE *dat, int type, AL_CONST char *value)
-{
-   datedit_insert_property(&dat->prop, type, value);
-}
-
-
-
 /* loads a datafile */
 DATAFILE *datedit_load_datafile(AL_CONST char *name, int compile_sprites, AL_CONST char *password)
 {
@@ -1095,17 +985,6 @@ DATAFILE *datedit_delete(DATAFILE *dat, int i)
 
 
 
-/* swaps two datafile objects */
-void datedit_swap(DATAFILE *dat, int i1, int i2)
-{
-   DATAFILE temp = dat[i1];
-
-   dat[i1] = dat[i2];
-   dat[i2] = temp;
-}
-
-
-
 /* fixup function for the strip options */
 int datedit_striptype(int strip)
 {
@@ -1131,7 +1010,7 @@ int datedit_packtype(int pack)
    else {
       AL_CONST char *p = get_datafile_property(&datedit_info, DAT_PACK);
 
-      if (*p)
+      if ((p) && (*p))
 	 return atoi(p);
       else
 	 return 0;
@@ -1140,46 +1019,22 @@ int datedit_packtype(int pack)
 
 
 
-/* fixup function for the sort options */
-int datedit_sorttype(int sort)
-{
-   if (sort >= 0) {
-      sort = (sort == 1 ? TRUE : FALSE);
-      datedit_set_property(&datedit_info, DAT_SORT, sort ? "y" : "n");
-
-      return sort;
-   }
-   else {
-      AL_CONST char *p = get_datafile_property(&datedit_info, DAT_SORT);
-
-      if (*p)
-	 return (utolower(*p)=='y');
-      else
-	 return TRUE;  /* sort if SORT property not present */
-   }
-}
-
-
-
 /* saves a datafile */
-int datedit_save_datafile(DATAFILE *dat, AL_CONST char *name, AL_CONST int *fixed_prop, AL_CONST DATEDIT_SAVE_DATAFILE_OPTIONS *options, AL_CONST char *password)
+int datedit_save_datafile(DATAFILE *dat, AL_CONST char *name, int strip, int pack, int verbose, int write_msg, int backup, AL_CONST char *password)
 {
    char *pretty_name;
    char backup_name[256];
-   int pack, strip, sort;
    PACKFILE *f;
-   int ret;
 
    packfile_password(password);
 
-   pack = datedit_packtype(options->pack);
-   strip = datedit_striptype(options->strip);
-   sort = datedit_sorttype(options->sort);
+   strip = datedit_striptype(strip);
+   pack = datedit_packtype(pack);
 
    strcpy(backup_name, datedit_pretty_name(name, "bak", TRUE));
    pretty_name = datedit_pretty_name(name, "dat", FALSE);
 
-   if (options->write_msg)
+   if (write_msg)
       datedit_msg("Writing %s", pretty_name);
 
    delete_file(backup_name);
@@ -1191,32 +1046,31 @@ int datedit_save_datafile(DATAFILE *dat, AL_CONST char *name, AL_CONST int *fixe
       pack_mputl(DAT_MAGIC, f);
       file_datasize = 12;
 
-      ret = save_datafile(dat, fixed_prop, (pack >= 2), (pack >= 1), strip, sort, options->verbose, (strip <= 0), f);
+      save_datafile(dat, (pack >= 2), (pack >= 1), strip, verbose, (strip <= 0), f);
 
-      if ((ret == TRUE) && (strip <= 0)) {
+      if (strip <= 0) {
 	 datedit_set_property(&datedit_info, DAT_NAME, "GrabberInfo");
-	 ret = save_object(&datedit_info, NULL, FALSE, FALSE, FALSE, FALSE, FALSE, f);
+	 save_object(&datedit_info, FALSE, FALSE, FALSE, FALSE, f);
       }
 
       pack_fclose(f); 
    }
-   else
-      ret = FALSE;
 
-   if (ret == FALSE) {
+   if (errno) {
       delete_file(pretty_name);
       datedit_error("Error writing %s", pretty_name);
       packfile_password(NULL);
       return FALSE;
    }
-   
-   if (!options->backup)
-      delete_file(backup_name);
+   else {
+      if (!backup)
+	 delete_file(backup_name);
 
-   if (options->verbose) {
-      int file_filesize = file_size(pretty_name);
-      datedit_msg("%-28s%7d bytes into %-7d (%d%%)", "- GLOBAL COMPRESSION -",
-		  file_datasize, file_filesize, percent(file_datasize, file_filesize));
+      if (verbose) {
+	 int file_filesize = file_size(pretty_name);
+	 datedit_msg("%-28s%7d bytes into %-7d (%d%%)", "- GLOBAL COMPRESSION -",
+		     file_datasize, file_filesize, percent(file_datasize, file_filesize));
+      }
    }
 
    packfile_password(NULL);
@@ -1513,78 +1367,25 @@ AL_CONST char *datedit_ftime2asc_int(long time)
 
 
 
-/* constructs a datafile object
- * WARNING: the object is private so it must be duplicated.
- */
-DATAFILE *datedit_construct(int type, void *dat, long size, DATAFILE_PROPERTY **prop)
+/* grabs an object from a disk file */
+DATAFILE *datedit_grab(AL_CONST char *filename, AL_CONST char *name, int type, int x, int y, int w, int h, int colordepth)
 {
-   static DATAFILE datafile;
+   static DATAFILE dat;
+   void *(*grab)(AL_CONST char *filename, long *size, int x, int y, int w, int h, int depth) = NULL;
+   char *ext = get_extension(filename);
+   char *tok;
+   char tmp[256];
+   int c;
 
-   if (!dat)
-      return NULL;
+   dat.dat = NULL;
+   dat.size = 0;
+   dat.prop = NULL;
 
-   ASSERT(size >= 0);
-
-   datafile.type = type;
-   datafile.dat = dat;
-   datafile.size = size;
-   if (prop != NULL) {
-      datafile.prop = *prop;
-
-      /* Make sure we own the property list. */
-      *prop = NULL;
+   if (type) {
+      dat.type = type;
    }
    else {
-      datafile.prop = NULL;
-   }
-
-   return &datafile;
-}
-
-
-
-/* clone properties */
-static DATAFILE_PROPERTY *clone_properties(DATAFILE_PROPERTY *prop)
-{
-   DATAFILE_PROPERTY *clone, *iter;
-   int size = 0, i;
-
-   if (!prop)
-      return NULL;
-
-   for (iter = prop; iter->type != DAT_END; iter++) {
-      size++;
-   }
-
-   clone = _al_malloc(sizeof(DATAFILE_PROPERTY)*(size+1));
-
-   for (i = 0; i <= size; i++) {
-       clone[i].type = prop[i].type;
-       if (prop[i].dat)
-          clone[i].dat = ustrdup(prop[i].dat);
-   }
-
-   return clone;
-}
-
-
-
-/* grabs an object from a disk file
- * WARNING: the object is private so it must be duplicated.
- */
-DATAFILE *datedit_grab(DATAFILE_PROPERTY *prop, AL_CONST DATEDIT_GRAB_PARAMETERS *params)
-{
-   DATAFILE *dat;
-   DATAFILE *(*grab)(int type, AL_CONST char *filename, DATAFILE_PROPERTY **prop, int depth);
-   char *ext = get_extension(params->filename);
-   char *tok;
-   char tmp[1024];
-   int type, type_index, c;
-
-   type = params->type;
-
-   if (!type) {
-      type = DAT_DATA;
+      dat.type = DAT_DATA;
 
       if ((ext) && (*ext)) {
 	 for (c=0; datedit_grabber_info[c]->type != DAT_END; c++) {
@@ -1593,7 +1394,7 @@ DATAFILE *datedit_grab(DATAFILE_PROPERTY *prop, AL_CONST DATEDIT_GRAB_PARAMETERS
 	       tok = strtok(tmp, ";");
 	       while (tok) {
 		  if (stricmp(tok, ext) == 0) {
-		     type = datedit_grabber_info[c]->type;
+		     dat.type = datedit_grabber_info[c]->type;
 		     goto found_type;
 		  }
 		  tok = strtok(NULL, ";");
@@ -1603,95 +1404,58 @@ DATAFILE *datedit_grab(DATAFILE_PROPERTY *prop, AL_CONST DATEDIT_GRAB_PARAMETERS
       }
    }
 
-  found_type:
-   type_index = -1;
+   found_type:
 
    for (c=0; datedit_grabber_info[c]->type != DAT_END; c++) {
-      if ((datedit_grabber_info[c]->type == type) && (datedit_grabber_info[c]->grab_ext) && (datedit_grabber_info[c]->grab)) {
+      if ((datedit_grabber_info[c]->type == dat.type) && (datedit_grabber_info[c]->grab_ext) && (datedit_grabber_info[c]->grab)) {
 	 if ((ext) && (*ext)) {
 	    strcpy(tmp, datedit_grabber_info[c]->grab_ext);
 	    tok = strtok(tmp, ";");
 	    while (tok) {
 	       if (stricmp(tok, ext) == 0) {
-		  type_index = c;
+		  grab = datedit_grabber_info[c]->grab;
 		  goto found_grabber;
 	       }
 	       tok = strtok(NULL, ";");
 	    }
 	 }
-	 if (!type_index < 0)
-	    type_index = c;
+	 if (!grab)
+	    grab = datedit_grabber_info[c]->grab;
       }
    }
 
-  found_grabber:
-   if (type_index >= 0)
-      grab = datedit_grabber_info[type_index]->grab;
-   else
+   if (!grab)
       grab = grab_binary;
 
-   if (prop)
-      prop = clone_properties(prop);
+   found_grabber:
+
+   dat.dat = grab(filename, &dat.size, x, y, w, h, colordepth);
+
+   if (dat.dat) {
+      datedit_set_property(&dat, DAT_NAME, name);
+      datedit_set_property(&dat, DAT_ORIG, filename);
+      datedit_set_property(&dat, DAT_DATE, datedit_ftime2asc(file_time(filename)));
+      errno = 0;
+   }
    else {
-      /* Set standard properties. */
-      datedit_insert_property(&prop, DAT_NAME, params->name);
-      if (params->relative) {
-	 make_relative_filename(tmp, params->datafile, params->filename, sizeof(tmp));
-	 datedit_insert_property(&prop, DAT_ORIG, tmp);
-      }
-      else
-	 datedit_insert_property(&prop, DAT_ORIG, params->filename);
-
-      /* Set specific properties. */
-      if ((type_index >= 0) && (datedit_grabber_info[type_index]->prop_types)) {
-	 strcpy(tmp, datedit_grabber_info[type_index]->prop_types);
-	 tok = strtok(tmp, ";");
-	 while (tok) {
-	    if (stricmp(tok, "XPOS") == 0) {
-	       sprintf(tmp, "%d", params->x);
-	       datedit_insert_property(&prop, DAT_XPOS, tmp);
-	    }
-	    else if (stricmp(tok, "YPOS") == 0) {
-	       sprintf(tmp, "%d", params->y);
-	       datedit_insert_property(&prop, DAT_YPOS, tmp);
-	    }
-	    else if (stricmp(tok, "XSIZ") == 0) {
-	       sprintf(tmp, "%d", params->w);
-	       datedit_insert_property(&prop, DAT_XSIZ, tmp);
-	    }
-	    else if (stricmp(tok, "YSIZ") == 0) {
-	       sprintf(tmp, "%d", params->h);
-	       datedit_insert_property(&prop, DAT_YSIZ, tmp);
-	    }
-
-	    tok = strtok(NULL, ";");
-	 }
-      }
+      datedit_error("Error reading %s as type %c%c%c%c", filename, 
+		    dat.type>>24, (dat.type>>16)&0xFF,
+		    (dat.type>>8)&0xFF, dat.type&0xFF);
    }
 
-   /* Always modify the date. */
-   datedit_insert_property(&prop, DAT_DATE, datedit_ftime2asc(file_time(params->filename)));
-
-   dat = grab(type, params->filename, &prop, params->colordepth);
-
-   if (!dat) {
-      datedit_error("Error reading %s as type %c%c%c%c", params->filename, 
-		    type>>24, (type>>16)&0xFF, (type>>8)&0xFF, type&0xFF);
-      if (prop)
-	 free(prop);
-   }
-
-   return dat;
+   return &dat;
 }
 
 
 
 /* grabs an object over the top of an existing one */
-int datedit_grabreplace(DATAFILE *dat, AL_CONST DATEDIT_GRAB_PARAMETERS *params)
+int datedit_grabreplace(DATAFILE *dat, AL_CONST char *filename, AL_CONST char *name, AL_CONST char *type, int colordepth, int x, int y, int w, int h)
 {
-   DATAFILE *tmp = datedit_grab(NULL, params);
+   DATAFILE *tmp = datedit_grab(filename, name, 
+				datedit_clean_typename(type), 
+				x, y, w, h, colordepth);
 
-   if (tmp) {
+   if (tmp->dat) {
       _unload_datafile_object(dat);
       *dat = *tmp;
       return TRUE;
@@ -1703,17 +1467,19 @@ int datedit_grabreplace(DATAFILE *dat, AL_CONST DATEDIT_GRAB_PARAMETERS *params)
 
 
 /* updates an object in-place */
-int datedit_grabupdate(DATAFILE *dat, DATEDIT_GRAB_PARAMETERS *params)
+int datedit_grabupdate(DATAFILE *dat, AL_CONST char *filename, int x, int y, int w, int h)
 {
-   DATAFILE *tmp;
+   DATAFILE *tmp = datedit_grab(filename, "dummyname", dat->type, x, y, w, h, -1);
+   DATAFILE_PROPERTY *tmp_prop;
 
-   params->name = "dummyname";
-   params->type = dat->type;
-   params->colordepth = -1;
+   if (tmp->dat) {
+      /* exchange properties */
+      tmp_prop = tmp->prop;
+      tmp->prop = dat->prop;
+      dat->prop = tmp_prop;
 
-   tmp = datedit_grab(dat->prop, params);
+      datedit_set_property(tmp, DAT_DATE, get_datafile_property(dat, DAT_DATE));
 
-   if (tmp) {
       /* adjust color depth? */
       if ((dat->type == DAT_BITMAP) || (dat->type == DAT_RLE_SPRITE) ||
 	  (dat->type == DAT_C_SPRITE) || (dat->type == DAT_XC_SPRITE)) {
@@ -1780,12 +1546,14 @@ int datedit_grabupdate(DATAFILE *dat, DATEDIT_GRAB_PARAMETERS *params)
 
 
 /* grabs a new object, inserting it into the datafile */
-DATAFILE *datedit_grabnew(DATAFILE *dat, AL_CONST DATEDIT_GRAB_PARAMETERS *params)
+DATAFILE *datedit_grabnew(DATAFILE *dat, AL_CONST char *filename, AL_CONST char *name, AL_CONST char *type, int colordepth, int x, int y, int w, int h)
 {
-   DATAFILE *tmp = datedit_grab(NULL, params);
+   DATAFILE *tmp = datedit_grab(filename, name, 
+				datedit_clean_typename(type), 
+				x, y, w, h, colordepth);
    int len;
 
-   if (tmp) {
+   if (tmp->dat) {
       len = 0;
       while (dat[len].type != DAT_END)
 	 len++;
@@ -1828,9 +1596,9 @@ DATAFILE *datedit_insert(DATAFILE *dat, DATAFILE **ret, AL_CONST char *name, int
 
 
 /* wrapper for examining numeric property values */
-int datedit_numprop(DATAFILE_PROPERTY **prop, int type)
+int datedit_numprop(AL_CONST DATAFILE *dat, int type)
 {
-   AL_CONST char *p = datedit_get_property(prop, type);
+   AL_CONST char *p = get_datafile_property(dat, type);
 
    if (*p)
       return atoi(p);
@@ -1921,46 +1689,33 @@ AL_CONST char *datedit_export_ext(int type)
 }
 
 
-/* (un)conditionally update an object */
-int datedit_update(DATAFILE *dat, AL_CONST char *datafile, int force, int verbose, int *changed)
+
+/* conditionally update an out-of-date object */
+int datedit_update(DATAFILE *dat, int verbose, int *changed)
 {
    AL_CONST char *name = get_datafile_property(dat, DAT_NAME);
    AL_CONST char *origin = get_datafile_property(dat, DAT_ORIG);
-   DATEDIT_GRAB_PARAMETERS params;
-   char filename[1024];
-   int relf;
+   AL_CONST char *date = get_datafile_property(dat, DAT_DATE);
+   time_t origt, datat;
 
    if (!*origin) {
       datedit_msg("%s has no origin data - skipping", name);
       return TRUE;
    }
 
-   if (is_relative_filename(origin)) {
-      make_absolute_filename(filename, datafile, origin, sizeof(filename));
-      relf = TRUE;
-   }
-   else {
-      strcpy(filename, origin);
-      relf = FALSE;
-   }
-   
-   if (!exists(filename)) {
+   if (!exists(origin)) {
       datedit_msg("%s: %s not found - skipping", name, origin);
       return TRUE;
    }
 
-   if (!force) {
-      AL_CONST char *date = get_datafile_property(dat, DAT_DATE);
+   if (*date) {
+      origt = file_time(origin);
+      datat = datedit_asc2ftime(date);
 
-      if (date[0]) {
-	 time_t origt = file_time(filename);
-	 time_t datat = datedit_asc2ftime(date);
-
-	 if ((origt/60) <= (datat/60)) {
-	    if (verbose)
-	       datedit_msg("%s: %s has not changed - skipping", name, origin);
-	    return TRUE;
-	 }
+      if ((origt/60) <= (datat/60)) {
+	 if (verbose)
+	    datedit_msg("%s: %s has not changed - skipping", name, origin);
+	 return TRUE;
       }
    }
 
@@ -1969,17 +1724,41 @@ int datedit_update(DATAFILE *dat, AL_CONST char *datafile, int force, int verbos
    if (changed)
       *changed = TRUE;
 
-   params.datafile = datafile;
-   params.filename = filename;
-   /* params.name = */
-   /* params.type = */
-   /* params.x = */
-   /* params.y = */
-   /* params.w = */
-   /* params.h = */
-   /* params.colordepth = */
-   params.relative = relf;
-
-   return datedit_grabupdate(dat, &params);
+   return datedit_grabupdate(dat, origin, 
+			     datedit_numprop(dat, DAT_XPOS), 
+			     datedit_numprop(dat, DAT_YPOS), 
+			     datedit_numprop(dat, DAT_XSIZ), 
+			     datedit_numprop(dat, DAT_YSIZ));
 }
 
+
+
+/* unconditionally update an object */
+int datedit_force_update(DATAFILE *dat, int verbose, int *changed)
+{
+   AL_CONST char *name = get_datafile_property(dat, DAT_NAME);
+   AL_CONST char *origin = get_datafile_property(dat, DAT_ORIG);
+
+   if (!*origin) {
+      datedit_msg("%s has no origin data - skipping", name);
+      return TRUE;
+   }
+
+   if (!exists(origin)) {
+      datedit_msg("%s: %s not found - skipping", name, origin);
+      return TRUE;
+   }
+
+   datedit_msg("Updating %s -> %s", origin, name);
+
+   if (changed)
+      *changed = TRUE;
+
+   return datedit_grabupdate(dat, origin, 
+			     datedit_numprop(dat, DAT_XPOS), 
+			     datedit_numprop(dat, DAT_YPOS), 
+			     datedit_numprop(dat, DAT_XSIZ), 
+			     datedit_numprop(dat, DAT_YSIZ));
+}
+
+
