@@ -20,15 +20,18 @@
 #include <string.h>
 
 #include "allegro.h"
-#include "allegro/aintern.h"
+#include "allegro/internal/aintern.h"
 #include "datedit.h"
 
-#ifdef ALLEGRO_DJGPP
+#if (defined ALLEGRO_DJGPP) && (!defined SCAN_DEPEND)
    #include <dos.h>
    #include <sys/exceptn.h>
 #endif
 
 
+/* 80 characters * maximum character width (6 bytes for UTF8) */
+#define FILENAME_LENGTH (80*6)
+
 
 typedef struct DATAITEM
 {
@@ -55,6 +58,8 @@ static char ygrid_string[16];
 static int current_view_object = -1;
 static int current_property_object = -1;
 
+static int is_modified = 0;
+
 static int busy_mouse = FALSE;
 
 static int entered_password = FALSE;
@@ -65,8 +70,10 @@ static int view_proc(int, DIALOG *, int);
 static int list_proc(int, DIALOG *, int);
 static int prop_proc(int, DIALOG *, int);
 static int droplist_proc(int, DIALOG *, int);
-static int dither_proc(int, DIALOG *, int);
+static int colorconv_proc(int, DIALOG *, int);
 static int custkey_proc(int, DIALOG *, int);
+static int edit_mod_proc(int, DIALOG *, int);
+static int droplist_mod_proc(int, DIALOG *, int);
 static char *list_getter(int, int *);
 static char *pack_getter(int, int *);
 static char *prop_getter(int, int *);
@@ -76,6 +83,7 @@ static int merger(void);
 static int saver(void);
 static int strip_saver(void);
 static int updater(void);
+static int sel_updater(void);
 static int force_updater(void);
 static int reader(void);
 static int viewer(void);
@@ -107,6 +115,7 @@ static MENU file_menu[32] =
    { "Save S&tripped",              strip_saver,      NULL,       0, NULL  },
    { "&Merge",                      merger,           NULL,       0, NULL  },
    { "&Update\t(ctrl+U)",           updater,          NULL,       0, NULL  },
+   { "Update S&election",           sel_updater,      NULL,       0, NULL  },
    { "&Force Update",               force_updater,    NULL,       0, NULL  },
    { "",                            NULL,             NULL,       0, NULL  },
    { "&Read Bitmap\t(ctrl+R)",      reader,           NULL,       0, NULL  },
@@ -134,7 +143,7 @@ static MENU objc_menu[32] =
    { "Set &Property\t(ctrl+P)",     property_insert,  NULL,       0, NULL  },
    { "&Shell Edit\t(ctrl+Z)",       sheller,          NULL,       0, NULL  },
    { "",                            NULL,             NULL,       0, NULL  },
-   { "&New...",                     NULL,             new_menu,   0, NULL  },
+   { "&New",                        NULL,             new_menu,   0, NULL  },
    { NULL,                          NULL,             NULL,       0, NULL  }
 };
 
@@ -168,7 +177,7 @@ static MENU popup_menu[32] =
    { "&Rename",                     renamer,          NULL,       0, NULL  },
    { "&Shell Edit",                 sheller,          NULL,       0, NULL  },
    { "",                            NULL,             NULL,       0, NULL  },
-   { "&New...",                     NULL,             new_menu,   0, NULL  },
+   { "&New",                        NULL,             new_menu,   0, NULL  },
    { NULL,                          NULL,             NULL,       0, NULL  }
 };
 
@@ -184,21 +193,22 @@ static DIALOG main_dlg[] =
    { d_clear_proc,      0,    0,    640,  480,  0,    0,    0,       0,          0,             0,       NULL,             NULL, NULL  },
    { d_menu_proc,       0,    0,    0,    0,    0,    0,    0,       0,          0,             0,       menu,             NULL, NULL  },
    { d_text_proc,       20,   30,   0,    0,    0,    0,    0,       0,          0,             0,       "Editing:",       NULL, NULL  },
-   { d_edit_proc,       100,  30,   320,  8,    0,    0,    0,       0,          255,           0,       data_file,        NULL, NULL  },
+   { edit_mod_proc,     100,  30,   320,  8,    0,    0,    0,       0,          255,           0,       data_file,        NULL, NULL  },
    { d_text_proc,       20,   42,   0,    0,    0,    0,    0,       0,          0,             0,       "Header:",        NULL, NULL  },
-   { d_edit_proc,       100,  42,   320,  8,    0,    0,    0,       0,          255,           0,       header_file,      NULL, NULL  },
+   { edit_mod_proc,     100,  42,   320,  8,    0,    0,    0,       0,          255,           0,       header_file,      NULL, NULL  },
    { d_text_proc,       20,   54,   0,    0,    0,    0,    0,       0,          0,             0,       "Prefix:",        NULL, NULL  },
-   { d_edit_proc,       100,  54,   320,  8,    0,    0,    0,       0,          255,           0,       prefix_string,    NULL, NULL  },
+   { edit_mod_proc,     100,  54,   320,  8,    0,    0,    0,       0,          255,           0,       prefix_string,    NULL, NULL  },
    { d_text_proc,       20,   66,   0,    0,    0,    0,    0,       0,          0,             0,       "Password:",      NULL, NULL  },
-   { d_edit_proc,       100,  66,   320,  8,    0,    0,    0,       0,          255,           0,       password,         NULL, NULL  },
+   { edit_mod_proc,     100,  66,   320,  8,    0,    0,    0,       0,          255,           0,       password,         NULL, NULL  },
    { d_text_proc,       200,  10,   0,    0,    0,    0,    0,       0,          0,             0,       "X-grid:",        NULL, NULL  },
    { d_edit_proc,       264,  10,   40,   8,    0,    0,    0,       0,          4,             0,       xgrid_string,     NULL, NULL  },
    { d_text_proc,       315,  10,   0,    0,    0,    0,    0,       0,          0,             0,       "Y-grid:",        NULL, NULL  },
    { d_edit_proc,       379,  10,   40,   8,    0,    0,    0,       0,          4,             0,       ygrid_string,     NULL, NULL  },
-   { d_check_proc,      430,  8,    82,   12,   0,    0,    0,       0,          0,             0,       "Backups:",       NULL, NULL  },
-   { dither_proc,       550,  8,    74,   12,   0,    0,    0,       0,          0,             0,       "Dither:",        NULL, NULL  },
-   { droplist_proc,     430,  30,   194,  27,   0,    0,    0,       0,          0,             0,       pack_getter,      NULL, NULL  },
-   { prop_proc,         260,  86,   364,  107,  0,    0,    0,       D_EXIT,     0,             0,       prop_getter,      NULL, NULL  },
+   { d_check_proc,      430,  8,    83,   13,   0,    0,    0,       0,          0,             0,       "Backups:",       NULL, NULL  },
+   { colorconv_proc,    550,  8,    75,   13,   0,    0,    0,       0,          0,             0,       "Dither:",        NULL, NULL  },
+   { colorconv_proc,    430,  24,   195,  13,   0,    0,    0,       0,          0,             1,       "Preserve transparency:", NULL, NULL},
+   { droplist_mod_proc, 430,  48,   195,  28,   0,    0,    0,       0,          0,             0,       pack_getter,      NULL, NULL  },
+   { prop_proc,         260,  86,   365,  107,  0,    0,    0,       D_EXIT,     0,             0,       prop_getter,      NULL, NULL  },
    { d_keyboard_proc,   0,    0,    0,    0,    0,    0,    C('l'),  0,          0,             0,       loader,           NULL, NULL  },
    { d_keyboard_proc,   0,    0,    0,    0,    0,    0,    C('s'),  0,          0,             0,       saver,            NULL, NULL  },
    { d_keyboard_proc,   0,    0,    0,    0,    0,    0,    C('u'),  0,          0,             0,       updater,          NULL, NULL  },
@@ -229,7 +239,7 @@ static DIALOG main_dlg[] =
    { custkey_proc,      0,    0,    0,    0,    0,    0,    0,       D_DISABLED, 0,             0,       NULL,             NULL, NULL  },
    { custkey_proc,      0,    0,    0,    0,    0,    0,    0,       D_DISABLED, 0,             0,       NULL,             NULL, NULL  },
    { custkey_proc,      0,    0,    0,    0,    0,    0,    0,       D_DISABLED, 0,             0,       NULL,             NULL, NULL  },
-   { list_proc,         20,   86,   208,  387,  0,    0,    0,       D_EXIT,     0,             0,       list_getter,      NULL, NULL  },
+   { list_proc,         20,   86,   209,  388,  0,    0,    0,       D_EXIT,     0,             0,       list_getter,      NULL, NULL  },
    { view_proc,         260,  218,  0,    0,    0,    0,    0,       0,          0,             0,       NULL,             NULL, NULL  },
    { NULL,              0,    0,    0,    0,    0,    0,    0,       0,          0,             0,       NULL,             NULL, NULL  }
 };
@@ -244,11 +254,12 @@ static DIALOG main_dlg[] =
 #define DLG_YGRIDSTRING       13
 #define DLG_BACKUPCHECK       14
 #define DLG_DITHERCHECK       15
-#define DLG_PACKLIST          16
-#define DLG_PROP              17
-#define DLG_FIRSTWHITE        18
-#define DLG_LIST              48
-#define DLG_VIEW              49
+#define DLG_TRANSCHECK        16
+#define DLG_PACKLIST          17
+#define DLG_PROP              18
+#define DLG_FIRSTWHITE        19
+#define DLG_LIST              49
+#define DLG_VIEW              50
 
 
 #define SELECTED_ITEM         main_dlg[DLG_LIST].d1
@@ -270,8 +281,58 @@ static int box_active = FALSE;
 
 
 
+/* updates the window title */
+static void update_title(void)
+{
+   char buf[1024];
+
+   if (data_file[0])
+      sprintf(buf, "grabber - %s%s", data_file, is_modified ? " (modified)" : "");
+   else
+      strcpy(buf, "grabber");
+
+   set_window_title(buf);
+}
+
+
+
+/* indicates a possible modification of the currently loaded datafile */
+static void modified(int yes_or_no)
+{
+   is_modified = yes_or_no;
+   update_title();
+}
+
+
+
+/* standard edit proc, enhanced with possible modification detection */
+static int edit_mod_proc(int msg, DIALOG *d, int c)
+{
+   int ret = d_edit_proc(msg, d, c);
+   
+   if ((msg == MSG_CHAR) || (msg == MSG_UCHAR))
+      modified(1);
+
+   return ret;
+}
+
+
+
+/* droplist proc, enhanced with possible modification detection */
+static int droplist_mod_proc(int msg, DIALOG *d, int c)
+{
+   int ret = droplist_proc(msg, d, c);
+   
+   if ((msg == MSG_CHAR) || (msg == MSG_UCHAR) || (msg == MSG_CLICK))
+      modified(1);
+
+   return ret;
+}
+
+
+
 /* starts outputting a progress message */
-static void box_start()
+static void box_start(void)
 {
    show_mouse(NULL);
 
@@ -314,7 +375,7 @@ static void box_out(char *msg)
 
 
 /* outputs text to the progress message */
-static void box_eol()
+static void box_eol(void)
 {
    BITMAP *mouse_screen = _mouse_screen;
 
@@ -369,7 +430,7 @@ static void box_end(int pause)
 
 
 /* returns the currently selected object, or -1 if there is a multiple sel */
-static int single_selection()
+static int single_selection(void)
 {
    int sel = SELECTED_ITEM;
    int i;
@@ -384,7 +445,7 @@ static int single_selection()
 
 
 /* returns a pointer to the currently selected object */
-static DATAFILE *get_single_selection()
+static DATAFILE *get_single_selection(void)
 {
    int sel = single_selection();
 
@@ -671,7 +732,7 @@ static MENU *which_menu(int sel)
 	 if ((dat) && (dat->type != DAT_FILE))
 	    ok = TRUE;
       }
-      else if (compare_menu_names(popup_menu[i].text, "New...") == 0) {
+      else if (compare_menu_names(popup_menu[i].text, "New") == 0) {
 	 ok = TRUE;
       }
       else if (!popup_menu[i].text[0]) {
@@ -879,7 +940,7 @@ static int view_proc(int msg, DIALOG *d, int c)
    if (msg == MSG_IDLE) {
       if (current_view_object != SELECTED_ITEM) {
 	 show_mouse(NULL);
-	 SEND_MESSAGE(d, MSG_DRAW, 0);
+	 object_message(d, MSG_DRAW, 0);
 	 show_mouse(screen);
       }
    }
@@ -929,8 +990,8 @@ static int view_proc(int msg, DIALOG *d, int c)
 static int droplist_proc(int msg, DIALOG *d, int c)
 {
    if (msg == MSG_DRAW) {
-      hline(screen, d->x+1, d->y+d->h+1, d->x+d->w+1, d->fg);
-      vline(screen, d->x+d->w+1, d->y+1, d->y+d->h+1, d->fg);
+      hline(screen, d->x+1, d->y+d->h, d->x+d->w, d->fg);
+      vline(screen, d->x+d->w, d->y+1, d->y+d->h, d->fg);
    }
 
    return d_list_proc(msg, d, c);
@@ -938,26 +999,27 @@ static int droplist_proc(int msg, DIALOG *d, int c)
 
 
 
-/* dialog procedure for toggling dither mode on and off */
-static int dither_proc(int msg, DIALOG *d, int c)
+/* dialog procedure for setting the color conversion mode */
+static int colorconv_proc(int msg, DIALOG *d, int c)
 {
-   static int current = -1;
-   int ret, wanted;
+   static int current[2] = {-1, -1};
+   int ret, mode, wanted[2];
 
    ret = d_check_proc(msg, d, c);
 
    if (d->flags & D_SELECTED)
-      wanted = 1;
+      wanted[d->d2] = 1;
    else
-      wanted = 0;
+      wanted[d->d2] = 0;
 
-   if (wanted != current) {
-      if (wanted)
-	 set_color_conversion(COLORCONV_DITHER);
+   if (wanted[d->d2] != current[d->d2]) {
+      current[d->d2] = wanted[d->d2];
+
+      mode = (current[0] ? COLORCONV_DITHER : 0) | (current[1] ? COLORCONV_KEEP_TRANS : 0);
+      if (mode)
+         set_color_conversion(mode);
       else
-	 set_color_conversion(COLORCONV_NONE);
-
-      current = wanted;
+         set_color_conversion(COLORCONV_NONE);
    } 
 
    return ret;
@@ -1011,10 +1073,10 @@ static int list_proc(int msg, DIALOG *d, int c)
 
    if ((msg == MSG_DRAW) && (!keypressed())) {
       if (current_view_object != d->d1)
-	 SEND_MESSAGE(main_dlg+DLG_VIEW, MSG_DRAW, 0);
+	 object_message(main_dlg+DLG_VIEW, MSG_DRAW, 0);
 
       if (current_property_object != d->d1)
-	 SEND_MESSAGE(main_dlg+DLG_PROP, MSG_DRAW, 0);
+	 object_message(main_dlg+DLG_PROP, MSG_DRAW, 0);
    }
 
    if (ret & D_CLOSE) {
@@ -1077,7 +1139,7 @@ static int prop_proc(int msg, DIALOG *d, int c)
       case MSG_IDLE:
 	 if (current_property_object != SELECTED_ITEM) {
 	    show_mouse(NULL);
-	    SEND_MESSAGE(d, MSG_DRAW, 0);
+	    object_message(d, MSG_DRAW, 0);
 	    show_mouse(screen);
 	 }
 	 break;
@@ -1174,7 +1236,7 @@ static void select_property(int type)
       }
    }
 
-   SEND_MESSAGE(main_dlg+DLG_PROP, MSG_START, 0);
+   object_message(main_dlg+DLG_PROP, MSG_START, 0);
 }
 
 
@@ -1229,14 +1291,15 @@ static char prop_value_string[256];
 static DIALOG prop_dlg[] =
 {
    /* (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key)    (flags)     (d1)           (d2)     (dp)                 (dp2) (dp3) */
-   { d_shadow_box_proc, 0,    0,    400,  112,  0,    0,    0,       0,          0,             0,       NULL,                NULL, NULL  },
+   { d_shadow_box_proc, 0,    0,    401,  113,  0,    0,    0,       0,          0,             0,       NULL,                NULL, NULL  },
    { d_ctext_proc,      200,  8,    0,    0,    0,    0,    0,       0,          0,             0,       NULL,                NULL, NULL  },
    { d_text_proc,       16,   32,   0,    0,    0,    0,    0,       0,          0,             0,       NULL,                NULL, NULL  },
    { d_edit_proc,       72,   32,   40,   8,    0,    0,    0,       0,          4,             0,       prop_type_string,    NULL, NULL  },
    { d_text_proc,       16,   48,   0,    0,    0,    0,    0,       0,          0,             0,       NULL,                NULL, NULL  },
    { d_edit_proc,       72,   48,   320,  8,    0,    0,    0,       0,          255,           0,       prop_value_string,   NULL, NULL  },
-   { d_button_proc,     112,  80,   80,   16,   0,    0,    13,      D_EXIT,     0,             0,       "OK",                NULL, NULL  }, 
-   { d_button_proc,     208,  80,   80,   16,   0,    0,    27,      D_EXIT,     0,             0,       "Cancel",            NULL, NULL  }, 
+   { d_button_proc,     112,  80,   81,   17,   0,    0,    13,      D_EXIT,     0,             0,       "OK",                NULL, NULL  }, 
+   { d_button_proc,     208,  80,   81,   17,   0,    0,    27,      D_EXIT,     0,             0,       "Cancel",            NULL, NULL  }, 
+   { d_yield_proc,      0,    0,    0,    0,    0,    0,    0,       0,          0,             0,       NULL,                NULL, NULL  },
    { NULL,              0,    0,    0,    0,    0,    0,    0,       0,          0,             0,       NULL,                NULL, NULL  }
 };
 
@@ -1252,7 +1315,7 @@ static DIALOG prop_dlg[] =
 
 
 /* brings up the property/new object dialog */
-static int do_edit(char *title, char *type_string, char *value_string, int type, char *val, int change_type, int show_type)
+static int do_edit(char *title, char *type_string, char *value_string, int type, AL_CONST char *val, int change_type, int show_type)
 {
    prop_dlg[PROP_DLG_TITLE].dp = title;
    prop_dlg[PROP_DLG_TYPE_STRING].dp = type_string;
@@ -1285,13 +1348,15 @@ static int do_edit(char *title, char *type_string, char *value_string, int type,
    else
       prop_dlg[PROP_DLG_TYPE].proc = d_text_proc;
 
+   modified(1);
+
    return (do_dialog(prop_dlg, (change_type ? PROP_DLG_TYPE : PROP_DLG_VALUE)) == PROP_DLG_OK);
 }
 
 
 
 /* brings up the property editing dialog */
-static int edit_property(char *title, char *value_string, int type, char *val, int change_type, int show_type)
+static int edit_property(char *title, char *value_string, int type, AL_CONST char *val, int change_type, int show_type)
 {
    int sel = single_selection();
    DATAITEM *dat;
@@ -1317,7 +1382,7 @@ static int edit_property(char *title, char *value_string, int type, char *val, i
 
 
 /* handle the set property command */
-static int property_insert()
+static int property_insert(void)
 {
    CHECK_MENU_HOOK("Set Property", DATEDIT_MENU_OBJECT);
 
@@ -1327,7 +1392,7 @@ static int property_insert()
 
 
 /* handle the change property command */
-static int property_change()
+static int property_change(void)
 {
    int sel = single_selection();
    DATAFILE *dat;
@@ -1354,7 +1419,7 @@ static int property_change()
 
 
 /* handle the rename command */
-static int renamer()
+static int renamer(void)
 {
    DATAFILE *dat;
    int sel;
@@ -1378,7 +1443,7 @@ static int renamer()
 
 
 /* handle the delete property command */
-static int property_delete()
+static int property_delete(void)
 {
    int sel = single_selection();
    DATAFILE *dat;
@@ -1478,7 +1543,7 @@ static void rebuild_list(void *old, int clear)
       }
    }
 
-   SEND_MESSAGE(main_dlg+DLG_LIST, MSG_START, 0);
+   object_message(main_dlg+DLG_LIST, MSG_START, 0);
 }
 
 
@@ -1515,8 +1580,9 @@ static char *pack_getter(int index, int *list_size)
       "Compressed"
    };
 
+   ASSERT(sizeof(s) / sizeof(s[0]) == sizeof(s2) / sizeof(s2[0]));
    if (index < 0) {
-      *list_size = 3;
+      *list_size = sizeof(s) / sizeof(s[0]);
       return NULL;
    }
 
@@ -1529,7 +1595,7 @@ static char *pack_getter(int index, int *list_size)
 
 
 /* updates the info chunk with the current settings */
-static void update_info()
+static void update_info(void)
 {
    char buf[8];
 
@@ -1553,7 +1619,7 @@ static void update_info()
 /* helper for recovering data stored in the info chunk */
 static void retrieve_property(int object, int type, char *def)
 {
-   char *p = get_datafile_property(&datedit_info, type);
+   AL_CONST char *p = get_datafile_property(&datedit_info, type);
 
    if ((p) && (*p))
       strcpy(main_dlg[object].dp, p);
@@ -1571,6 +1637,7 @@ static void load(char *filename, int flush)
    DATAFILE *new_datafile;
    int obj_count;
    int new_obj_count;
+   int items_num;
 
    set_mouse_sprite(my_busy_pointer);
    busy_mouse = TRUE;
@@ -1607,9 +1674,13 @@ static void load(char *filename, int flush)
       free(new_datafile);
 
       datedit_sort_datafile(datafile);
+      
+      modified(1);
    }
-   else
+   else {
       datafile = new_datafile;
+      modified(0);
+   }
 
    SELECTED_ITEM = 0;
 
@@ -1629,8 +1700,9 @@ static void load(char *filename, int flush)
       main_dlg[DLG_DITHERCHECK].flags &= ~D_SELECTED;
 
    main_dlg[DLG_PACKLIST].d1 = atoi(get_datafile_property(&datedit_info, DAT_PACK));
-   if (main_dlg[DLG_PACKLIST].d1 > 2)
-      main_dlg[DLG_PACKLIST].d1 = 2;
+   pack_getter(-1, &items_num);
+   if (main_dlg[DLG_PACKLIST].d1 >= items_num)
+      main_dlg[DLG_PACKLIST].d1 = items_num-1;
    else if (main_dlg[DLG_PACKLIST].d1 < 0)
       main_dlg[DLG_PACKLIST].d1 = 0;
 
@@ -1643,16 +1715,16 @@ static void load(char *filename, int flush)
 
 
 /* handle the load command */
-static int loader()
+static int loader(void)
 {
-   char buf[256];
+   char buf[FILENAME_LENGTH];
 
    CHECK_MENU_HOOK("Load", DATEDIT_MENU_FILE);
 
    strcpy(buf, data_file);
    *get_filename(buf) = 0;
 
-   if (file_select("Load datafile", buf, "dat")) {
+   if (file_select_ex("Load datafile", buf, "dat", sizeof(buf), 0, 0)) {
       fix_filename_case(buf);
       load(buf, 1);
    }
@@ -1663,16 +1735,16 @@ static int loader()
 
 
 /* handle the merge command */
-static int merger()
+static int merger(void)
 {
-   char buf[256];
+   char buf[FILENAME_LENGTH];
 
    CHECK_MENU_HOOK("Merge", DATEDIT_MENU_FILE);
 
    strcpy(buf, data_file);
    *get_filename(buf) = 0;
 
-   if (file_select("Merge datafile", buf, "dat")) {
+   if (file_select_ex("Merge datafile", buf, "dat", sizeof(buf), 0, 0)) {
       fix_filename_case(buf);
       load(buf, 0);
    }
@@ -1685,12 +1757,12 @@ static int merger()
 /* do the actual work of saving a file */
 static int save(int strip)
 {
-   char buf[256], buf2[256];
+   char buf[FILENAME_LENGTH], buf2[256];
    int err = FALSE;
 
    strcpy(buf, data_file);
 
-   if (file_select("Save datafile", buf, "dat")) {
+   if (file_select_ex("Save datafile", buf, "dat", sizeof(buf), 0, 0)) {
       if ((stricmp(data_file, buf) != 0) && (exists(buf))) {
 	 sprintf(buf2, "%s already exists, overwrite?", buf);
 	 if (alert(buf2, NULL, NULL, "Yes", "Cancel", 'y', 27) != 1)
@@ -1710,6 +1782,8 @@ static int save(int strip)
 
       if (!datedit_save_datafile(datafile, data_file, strip, -1, TRUE, FALSE, (main_dlg[DLG_BACKUPCHECK].flags & D_SELECTED), password))
 	 err = TRUE;
+      else
+	 modified(0);
 
       if ((header_file[0]) && (!err)) {
 	 box_eol();
@@ -1737,7 +1811,7 @@ static int save(int strip)
 
 
 /* handle the save command */
-static int saver()
+static int saver(void)
 {
    CHECK_MENU_HOOK("Save", DATEDIT_MENU_FILE);
 
@@ -1758,7 +1832,7 @@ static char *striplist_getter(int index, int *list_size)
 
    if (index < 0) {
       if (list_size)
-	 *list_size = 3;
+	 *list_size = sizeof(str) / sizeof(str[0]);
       return NULL;
    }
 
@@ -1770,11 +1844,12 @@ static char *striplist_getter(int index, int *list_size)
 static DIALOG strip_dlg[] =
 {
    /* (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key)    (flags)     (d1)           (d2)     (dp)              (dp2) (dp3) */
-   { d_shadow_box_proc, 0,    0,    300,  112,  0,    0,    0,       0,          0,             0,       NULL,             NULL, NULL  },
+   { d_shadow_box_proc, 0,    0,    301,  113,  0,    0,    0,       0,          0,             0,       NULL,             NULL, NULL  },
    { d_ctext_proc,      150,  8,    0,    0,    0,    0,    0,       0,          0,             0,       "Save Stripped",  NULL, NULL  },
-   { d_list_proc,       22,   32,   256,  27,   0,    0,    0,       D_EXIT,     0,             0,       striplist_getter, NULL, NULL  },
-   { d_button_proc,     62,   80,   80,   16,   0,    0,    13,      D_EXIT,     0,             0,       "OK",             NULL, NULL  }, 
-   { d_button_proc,     158,  80,   80,   16,   0,    0,    27,      D_EXIT,     0,             0,       "Cancel",         NULL, NULL  }, 
+   { d_list_proc,       22,   32,   257,  28,   0,    0,    0,       D_EXIT,     0,             0,       striplist_getter, NULL, NULL  },
+   { d_button_proc,     62,   80,   81,   17,   0,    0,    13,      D_EXIT,     0,             0,       "OK",             NULL, NULL  }, 
+   { d_button_proc,     158,  80,   81,   17,   0,    0,    27,      D_EXIT,     0,             0,       "Cancel",         NULL, NULL  }, 
+   { d_yield_proc,      0,    0,    0,    0,    0,    0,    0,       0,          0,             0,       NULL,             NULL, NULL  },
    { NULL,              0,    0,    0,    0,    0,    0,    0,       0,          0,             0,       NULL,             NULL, NULL  }
 };
 
@@ -1786,7 +1861,7 @@ static DIALOG strip_dlg[] =
 
 
 /* handle the save stripped command */
-static int strip_saver()
+static int strip_saver(void)
 {
    CHECK_MENU_HOOK("Save Stripped", DATEDIT_MENU_FILE);
 
@@ -1802,7 +1877,7 @@ static int strip_saver()
 
 
 /* handle the update command */
-static int updater()
+static int updater(void)
 {
    int c;
    int nowhere;
@@ -1837,13 +1912,60 @@ static int updater()
 
    select_property(DAT_NAME);
 
+   modified(1);
+
+   return D_REDRAW;
+}
+
+
+
+/* handle the update_selection command */
+static int sel_updater(void)
+{
+   int c;
+   int nowhere;
+   int err = FALSE;
+
+   CHECK_MENU_HOOK("Update selection", DATEDIT_MENU_FILE);
+
+   box_start();
+
+   set_mouse_sprite(my_busy_pointer);
+   busy_mouse = TRUE;
+
+   for (c=1; c<data_count; c++) {
+      if ((c==SELECTED_ITEM) || data_sel[c]) {
+	 if (data[c].dat->type != DAT_FILE) {
+	    if (!datedit_update(data[c].dat, FALSE, &nowhere)) {
+	       err = TRUE;
+	       break;
+	    }
+	    datedit_sort_properties(data[c].dat->prop);
+	 }
+      }
+   }
+
+   set_mouse_sprite(my_mouse_pointer);
+   busy_mouse = FALSE;
+
+   if (!err) {
+      box_out("Done!");
+      box_eol();
+   }
+
+   box_end(!err);
+
+   select_property(DAT_NAME);
+
+   modified(1);
+
    return D_REDRAW;
 }
 
 
 
 /* handle the force_update command */
-static int force_updater()
+static int force_updater(void)
 {
    int c;
    int nowhere;
@@ -1878,13 +2000,15 @@ static int force_updater()
 
    select_property(DAT_NAME);
 
+   modified(1);
+
    return D_REDRAW;
 }
 
 
 
 /* formats a heading for the file select dialog */
-static void format_file_select_heading(char *dest, char *s1, char *s2, char *ext)
+static void format_file_select_heading(char *dest, char *s1, char *s2, AL_CONST char *ext)
 {
    int len;
 
@@ -1916,11 +2040,11 @@ static void format_file_select_heading(char *dest, char *s1, char *s2, char *ext
 
 
 /* handle the read command */
-static int reader()
+static int reader(void)
 {
    DATAFILE *dat;
-   char buf[256], buf2[256];
-   char *s;
+   char buf[FILENAME_LENGTH], buf2[256];
+   AL_CONST char *s;
 
    CHECK_MENU_HOOK("Read", DATEDIT_MENU_FILE);
 
@@ -1938,7 +2062,7 @@ static int reader()
 
    format_file_select_heading(buf2, "Read bitmap file", NULL, s);
 
-   if (file_select(buf2, buf, s)) {
+   if (file_select_ex(buf2, buf, s, sizeof(buf), 0, 0)) {
       fix_filename_case(buf);
       set_mouse_sprite(my_busy_pointer);
       busy_mouse = TRUE;
@@ -1980,7 +2104,7 @@ static int reader()
 
 
 /* handle the view command */
-static int viewer()
+static int viewer(void)
 {
    CHECK_MENU_HOOK("View", DATEDIT_MENU_FILE);
 
@@ -1999,25 +2123,38 @@ static int viewer()
 
 
 /* handle the quit command */
-static int quitter()
+static int quitter(void)
 {
+   int r;
+
    CHECK_MENU_HOOK("Quit", DATEDIT_MENU_FILE);
 
-   if (alert("Really want to quit?", NULL, NULL, "Yes", "Cancel", 'y', 27) == 1)
+   if (!is_modified)
       return D_CLOSE;
-   else
+   
+   r = alert3("Data may have been modified.",
+	      "Do you want to save before quitting?", NULL,
+	      "Save", "Quit", "Cancel", 's', 'q', 27);
+
+   if (r == 2)
+      return D_CLOSE;
+
+   if (r == 3)
       return D_O_K;
+
+   saver();
+   return D_CLOSE;
 }
 
 
 
 /* handle the grab command */
-static int grabber()
+static int grabber(void)
 {
    DATAFILE *dat;
    char *desc = "binary data";
-   char *ext = NULL;
-   char buf[256], name[256], type[8];
+   AL_CONST char *ext;
+   char buf[256], name[FILENAME_LENGTH], type[8];
    int sel;
    int i;
 
@@ -2052,7 +2189,7 @@ static int grabber()
 
    format_file_select_heading(buf, "Grab", desc, ext);
 
-   if (file_select(buf, name, ext)) {
+   if (file_select_ex(buf, name, ext, sizeof(name), 0, 0)) {
       fix_filename_case(name);
       set_mouse_sprite(my_busy_pointer);
       busy_mouse = TRUE;
@@ -2062,6 +2199,8 @@ static int grabber()
 
       datedit_grabreplace(dat, name, get_datafile_property(dat, DAT_NAME), type, -1, -1, -1, -1, -1);
 
+      modified(1);
+
       if (dat->type == DAT_FILE)
 	 rebuild_list(NULL, TRUE);
 
@@ -2078,11 +2217,11 @@ static int grabber()
 
 
 /* handle the export command */
-static int exporter()
+static int exporter(void)
 {
    char *desc = "binary data";
-   char *ext = NULL;
-   char buf[256], name[256];
+   AL_CONST char *ext;
+   char buf[256], name[FILENAME_LENGTH];
    DATAFILE *dat;
    int sel;
    int i;
@@ -2120,7 +2259,7 @@ static int exporter()
 
    format_file_select_heading(buf, "Export", desc, ext);
 
-   if (file_select(buf, name, ext)) {
+   if (file_select_ex(buf, name, ext, sizeof(name), 0, 0)) {
       fix_filename_case(name);
       set_mouse_sprite(my_busy_pointer);
       busy_mouse = TRUE;
@@ -2139,14 +2278,14 @@ static int exporter()
 
 
 /* handle the delete command */
-static int deleter()
+static int deleter(void)
 {
    void *dat;
    void **todel = NULL;
    int todel_count = 0;
    int todel_alloc = 0;
    char buf[256];
-   char *name = "";
+   AL_CONST char *name = "";
    int first = 0;
    int i;
 
@@ -2194,6 +2333,8 @@ static int deleter()
 
    free(todel);
 
+   modified(1);
+
    return D_REDRAW;
 }
 
@@ -2312,12 +2453,12 @@ static int d_helptext_proc(int msg, DIALOG *d, int c)
       d->d2 = 0;
 
       show_mouse(NULL);
-      SEND_MESSAGE(d, MSG_DRAW, c);
+      object_message(d, MSG_DRAW, c);
       show_mouse(screen);
 
       return D_O_K;
    }
-   else 
+   else
       return d_textbox_proc(msg, d, c);
 };
 
@@ -2336,7 +2477,7 @@ static DIALOG grabber_help[] =
 
 
 /* handle the help command */
-static int helper()
+static int helper(void)
 {
    char buf[256];
    PACKFILE *f;
@@ -2432,15 +2573,17 @@ static DIALOG sys_dlg[] =
    { d_shadow_box_proc, 0,    0,    0,    0,    0,    0,    0,       0,          0,       0,       NULL,    NULL, NULL  },
    { d_textbox_proc,    0,    0,    0,    0,    0,    0,    0,       0,          0,       0,       NULL,    NULL, NULL  },
    { d_button_proc,     0,    0,    0,    0,    0,    0,    0,       D_EXIT,     0,       0,       "Exit",  NULL, NULL  },
+   { d_yield_proc,      0,    0,    0,    0,    0,    0,    0,       0,          0,       0,       NULL,    NULL, NULL  },
    { NULL,              0,    0,    0,    0,    0,    0,    0,       0,          0,       0,       NULL,    NULL, NULL  }
 };
 
 
 
 /* handle the system info */
-static int sysinfo()
+static int sysinfo(void)
 {
-   char *systext, *s;
+   char *systext;
+   AL_CONST char* s;
    int i, type;
 
    CHECK_MENU_HOOK("System", DATEDIT_MENU_HELP);
@@ -2451,18 +2594,25 @@ static int sysinfo()
    strcpy(systext, "System Status\n=============\n\n\n\n");
 
    switch (os_type) {
-      case OSTYPE_UNKNOWN:    s = "DOS (score: 7)";                     break;
-      case OSTYPE_WIN3:       s = "Windows 3.1 or earlier (score: -1)"; break;
-      case OSTYPE_WIN95:      s = "Windows 95 (score: 3)";              break;
-      case OSTYPE_WIN98:      s = "Windows 98 (score: 3)";              break;
-      case OSTYPE_WINNT:      s = "Windows NT (score: 0)";              break;
-      case OSTYPE_OS2:        s = "OS/2 (score: 5)";                    break;
-      case OSTYPE_WARP:       s = "OS/2 Warp 3 (score: 6)";             break;
-      case OSTYPE_LINUX:      s = "Linux (score: 42)";                  break;
-      case OSTYPE_UNIX:       s = "Unix (score: 11)";                   break;
-      case OSTYPE_DOSEMU:     s = "Linux DOSEMU (score: 11)";           break;
-      case OSTYPE_OPENDOS:    s = "Caldera OpenDOS (score: 8)";         break;
-      default:                s = "Unknown (score: 3.14)";              break;
+      case OSTYPE_UNKNOWN:    s = "DOS";                        break;
+      case OSTYPE_WIN3:       s = "Windows 3.x";                break;
+      case OSTYPE_WIN95:      s = "Windows 95";                 break;
+      case OSTYPE_WIN98:      s = "Windows 98";                 break;
+      case OSTYPE_WINME:      s = "Windows ME";                 break;
+      case OSTYPE_WINNT:      s = "Windows NT";                 break;
+      case OSTYPE_WIN2000:    s = "Windows 2000";               break;
+      case OSTYPE_WINXP:      s = "Windows XP";                 break;
+      case OSTYPE_OS2:        s = "OS/2";                       break;
+      case OSTYPE_WARP:       s = "OS/2 Warp 3";                break;
+      case OSTYPE_DOSEMU:     s = "Linux DOSEMU";               break;
+      case OSTYPE_OPENDOS:    s = "Caldera OpenDOS";            break;
+      case OSTYPE_LINUX:      s = "Linux";                      break;
+      case OSTYPE_FREEBSD:    s = "FreeBSD";                    break;
+      case OSTYPE_UNIX:       s = "Unix";                       break;
+      case OSTYPE_BEOS:       s = "BeOS";                       break;
+      case OSTYPE_QNX:        s = "QNX";                        break;
+      case OSTYPE_MACOS:      s = "MacOS";                      break;
+      default:                s = "Unknown";                    break;
    }
 
    sprintf(systext+strlen(systext), "Platform: %s\n\n\n\n", s);
@@ -2471,14 +2621,26 @@ static int sysinfo()
 
       sprintf(systext+strlen(systext), "CPU: %s %d86", cpu_vendor, cpu_family);
 
-      if (cpu_fpu)
-	 strcat(systext, ", FPU");
+      if (cpu_capabilities & CPU_FPU)
+	 strcat(systext, " / FPU");
 
-      if (cpu_mmx)
-	 strcat(systext, ", MMX");
+      if (cpu_capabilities & CPU_SSE)
+	 strcat(systext, " / SSE");
+	 
+      if (cpu_capabilities & CPU_SSE2)
+	 strcat(systext, " / SSE2");
+	 
+      if (cpu_capabilities & CPU_MMX)
+	 strcat(systext, " / MMX");
 
-      if (cpu_3dnow)
-	 strcat(systext, ", 3DNow!");
+      if (cpu_capabilities & CPU_MMXPLUS)
+	 strcat(systext, " / MMX+");
+
+      if (cpu_capabilities & CPU_3DNOW)
+	 strcat(systext, " / 3DNow!");
+
+      if (cpu_capabilities & CPU_ENH3DNOW)
+	 strcat(systext, " / Enh 3DNow!");
 
       strcat(systext, "\n\n\n\n");
 
@@ -2515,18 +2677,18 @@ static int sysinfo()
 
    sys_dlg[0].x = 0;
    sys_dlg[0].y = 0;
-   sys_dlg[0].w = SCREEN_W*3/4;
-   sys_dlg[0].h = SCREEN_H*3/4;
+   sys_dlg[0].w = SCREEN_W*3/4+1;
+   sys_dlg[0].h = SCREEN_H*3/4+1;
 
    sys_dlg[1].x = 0;
    sys_dlg[1].y = 0;
-   sys_dlg[1].w = sys_dlg[0].w-1;
-   sys_dlg[1].h = sys_dlg[0].h-32;
+   sys_dlg[1].w = sys_dlg[0].w;
+   sys_dlg[1].h = sys_dlg[0].h-31;
 
    sys_dlg[2].x = (sys_dlg[0].w-80)/2;
    sys_dlg[2].y = sys_dlg[0].h-24;
-   sys_dlg[2].w = 80;
-   sys_dlg[2].h = 16;
+   sys_dlg[2].w = 81;
+   sys_dlg[2].h = 17;
 
    sys_dlg[1].d1 = 0;
    sys_dlg[1].d2 = 0;
@@ -2611,6 +2773,8 @@ static int add_new(int type)
 	 datedit_sort_datafile(*df);
 	 rebuild_list(v, TRUE);
 	 select_property(DAT_NAME);
+         
+	 modified(1);
       }
    }
 
@@ -2632,7 +2796,8 @@ static int sheller(void)
 {
    DATAFILE *dat;
    char buf[256], cmd[256], ext[256], filename[256];
-   char *s, *s2;
+   AL_CONST char *s, * s2;
+
    int oldw = SCREEN_W;
    int oldh = SCREEN_H;
    int export, delfile;
@@ -2797,6 +2962,8 @@ static int sheller(void)
    datedit_sort_properties(dat->prop);
    select_property(DAT_NAME);
 
+   modified(1);
+
    ohwellitwasaniceidea:
 
    if (delfile)
@@ -2811,7 +2978,7 @@ static int sheller(void)
 
 
 /* callback for the datedit functions to display a message */
-void datedit_msg(char *fmt, ...)
+void datedit_msg(AL_CONST char *fmt, ...)
 {
    va_list args;
    char buf[1024];
@@ -2833,7 +3000,7 @@ void datedit_msg(char *fmt, ...)
 
 
 /* callback for the datedit functions to start a multi-part message */
-void datedit_startmsg(char *fmt, ...)
+void datedit_startmsg(AL_CONST char *fmt, ...)
 {
    va_list args;
    char buf[1024];
@@ -2848,7 +3015,7 @@ void datedit_startmsg(char *fmt, ...)
 
 
 /* callback for the datedit functions to end a multi-part message */
-void datedit_endmsg(char *fmt, ...)
+void datedit_endmsg(AL_CONST char *fmt, ...)
 {
    va_list args;
    char buf[1024];
@@ -2870,7 +3037,7 @@ void datedit_endmsg(char *fmt, ...)
 
 
 /* callback for the datedit functions to report an error */
-void datedit_error(char *fmt, ...)
+void datedit_error(AL_CONST char *fmt, ...)
 {
    va_list args;
    char buf[1024];
@@ -2890,7 +3057,7 @@ void datedit_error(char *fmt, ...)
 
 
 /* callback for the datedit functions to ask a question */
-int datedit_ask(char *fmt, ...)
+int datedit_ask(AL_CONST char *fmt, ...)
 {
    va_list args;
    char buf[1024];
@@ -2917,6 +3084,14 @@ int datedit_ask(char *fmt, ...)
 
 
 
+/* close button callback */
+static void close_hook(void)
+{  
+   simulate_keypress(27 + (KEY_ESC << 8));   
+}
+
+
+
 int main(int argc, char *argv[])
 {
    int i, j;
@@ -2987,11 +3162,13 @@ int main(int argc, char *argv[])
    else {
       for (i=0; color_depths[i]; i++) {
 	 bpp = color_depths[i];
-	 set_color_depth(color_depths[i]);
+	 set_color_depth(bpp);
 	 ret = set_gfx_mode(GFX_AUTODETECT, w, h, 0, 0);
 	 if (ret == 0)
 	    break;
       }
+      if (ret != 0)
+	 ret = set_gfx_mode(GFX_SAFE, w, h, 0, 0);
    }
 
    if (ret != 0) {
@@ -3000,14 +3177,17 @@ int main(int argc, char *argv[])
       return 1;
    }
 
+   update_title();
+
+   set_window_close_hook(close_hook);
+
    if (no_sound) {
       install_sound(DIGI_NONE, MIDI_NONE, NULL);
    }
    else {
       if (install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL) != 0) {
-	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-	 allegro_message("Error initialising sound\n%s\n", allegro_error);
-	 return 1;
+	 alert ("Error initialising sound", allegro_error, NULL, "OK", NULL, 0, 0);
+	 install_sound(DIGI_NONE, MIDI_NONE, NULL);
       }
    }
 
@@ -3047,10 +3227,10 @@ int main(int argc, char *argv[])
       main_dlg[DLG_BACKUPCHECK].y += 20;
 
       main_dlg[DLG_DITHERCHECK].flags |= (D_HIDDEN | D_DISABLED);
+      main_dlg[DLG_TRANSCHECK].flags |= (D_HIDDEN | D_DISABLED);
 
       main_dlg[DLG_PACKLIST].x = main_dlg[DLG_PACKLIST].x * SCREEN_W / 640;
       main_dlg[DLG_PACKLIST].w = main_dlg[DLG_PACKLIST].w * SCREEN_W / 640;
-      main_dlg[DLG_PACKLIST].y += 18;
 
       for (i=DLG_PROP; main_dlg[i].proc; i++) {
 	 main_dlg[i].x = main_dlg[i].x * SCREEN_W / 640;
@@ -3095,6 +3275,7 @@ int main(int argc, char *argv[])
    grabber_single_selection = get_single_selection;
    grabber_set_selection = set_selection;
    grabber_busy_mouse = set_busy_mouse;
+   grabber_modified = modified;
 
    datedit_init();
 
@@ -3165,14 +3346,19 @@ int main(int argc, char *argv[])
       sprintf(ygrid_string, "%d", get_config_int("grabber", "ygrid", 16));
 
       if (strpbrk(get_config_string("grabber", "backups", ""), "yY1"))
-	 main_dlg[DLG_BACKUPCHECK].flags |= D_SELECTED;
+         main_dlg[DLG_BACKUPCHECK].flags |= D_SELECTED;
       else
-	 main_dlg[DLG_BACKUPCHECK].flags &= ~D_SELECTED;
+         main_dlg[DLG_BACKUPCHECK].flags &= ~D_SELECTED;
 
       if (strpbrk(get_config_string("grabber", "dither", ""), "yY1"))
-	 main_dlg[DLG_DITHERCHECK].flags |= D_SELECTED;
+         main_dlg[DLG_DITHERCHECK].flags |= D_SELECTED;
       else
-	 main_dlg[DLG_DITHERCHECK].flags &= ~D_SELECTED;
+         main_dlg[DLG_DITHERCHECK].flags &= ~D_SELECTED;
+
+      if (strpbrk(get_config_string("grabber", "transparency", ""), "yY1"))
+         main_dlg[DLG_TRANSCHECK].flags |= D_SELECTED;
+      else
+         main_dlg[DLG_TRANSCHECK].flags &= ~D_SELECTED;
    }
 
    do_dialog(main_dlg, DLG_LIST);
@@ -3202,6 +3388,11 @@ int main(int argc, char *argv[])
    else
       set_config_string("grabber", "dither", "n");
 
+   if (main_dlg[DLG_TRANSCHECK].flags & D_SELECTED)
+      set_config_string("grabber", "transparency", "y");
+   else
+      set_config_string("grabber", "transparency", "n");
+
    return 0;
 }
 
